@@ -1,14 +1,18 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 University of Houston. All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc. All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -25,11 +29,12 @@
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/coll/base/base.h"
 #include "ompi/mca/topo/base/base.h"
-#include "orte/mca/ns/base/base.h"
 #include "ompi/runtime/params.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/attribute/attribute.h"
 #include "ompi/mca/topo/topo.h"
+#include "ompi/mca/dpm/dpm.h"
+#include "ompi/memchecker.h"
 
 /*
 ** Table for Fortran <-> C communicator handle conversion
@@ -37,11 +42,11 @@
 ** on cid.
 ** 
 */
-ompi_pointer_array_t ompi_mpi_communicators; 
+opal_pointer_array_t ompi_mpi_communicators; 
 
-ompi_communicator_t  ompi_mpi_comm_world;
-ompi_communicator_t  ompi_mpi_comm_self;
-ompi_communicator_t  ompi_mpi_comm_null;
+ompi_predefined_communicator_t  ompi_mpi_comm_world;
+ompi_predefined_communicator_t  ompi_mpi_comm_self;
+ompi_predefined_communicator_t  ompi_mpi_comm_null;
 ompi_communicator_t  *ompi_mpi_comm_parent;
 
 static void ompi_comm_construct(ompi_communicator_t* comm);
@@ -64,90 +69,105 @@ int ompi_comm_init(void)
     size_t size;
 
     /* Setup communicator array */
-    OBJ_CONSTRUCT(&ompi_mpi_communicators, ompi_pointer_array_t); 
-
+    OBJ_CONSTRUCT(&ompi_mpi_communicators, opal_pointer_array_t); 
+    if( OPAL_SUCCESS != opal_pointer_array_init(&ompi_mpi_communicators, 0,
+                                                OMPI_FORTRAN_HANDLE_MAX, 64) ) {
+        return OMPI_ERROR;
+    }
 
     /* Setup MPI_COMM_WORLD */
     OBJ_CONSTRUCT(&ompi_mpi_comm_world, ompi_communicator_t);
     group = OBJ_NEW(ompi_group_t);
     group->grp_proc_pointers = ompi_proc_world(&size);
-    group->grp_proc_count    = size;
-    group->grp_flags        |= OMPI_GROUP_INTRINSIC;
+    group->grp_proc_count    = (int)size;
+    OMPI_GROUP_SET_INTRINSIC (group);
+    OMPI_GROUP_SET_DENSE (group);
     ompi_set_group_rank(group, ompi_proc_local());
     ompi_group_increment_proc_count (group);
 
-    ompi_mpi_comm_world.c_contextid    = 0;
-    ompi_mpi_comm_world.c_f_to_c_index = 0;
-    ompi_mpi_comm_world.c_my_rank      = group->grp_my_rank;
-    ompi_mpi_comm_world.c_local_group  = group;
-    ompi_mpi_comm_world.c_remote_group = group;
-    ompi_mpi_comm_world.c_cube_dim     = opal_cube_dim(size);
-    ompi_mpi_comm_world.error_handler  = &ompi_mpi_errors_are_fatal;
-    OBJ_RETAIN( &ompi_mpi_errors_are_fatal );
-    OMPI_COMM_SET_PML_ADDED(&ompi_mpi_comm_world);
-    ompi_pointer_array_set_item (&ompi_mpi_communicators, 0, &ompi_mpi_comm_world);
+    ompi_mpi_comm_world.comm.c_contextid    = 0;
+    ompi_mpi_comm_world.comm.c_id_start_index = 4;
+    ompi_mpi_comm_world.comm.c_id_available = 4;
+    ompi_mpi_comm_world.comm.c_f_to_c_index = 0;
+    ompi_mpi_comm_world.comm.c_my_rank      = group->grp_my_rank;
+    ompi_mpi_comm_world.comm.c_local_group  = group;
+    ompi_mpi_comm_world.comm.c_remote_group = group;
+    OBJ_RETAIN(ompi_mpi_comm_world.comm.c_remote_group);
+    ompi_mpi_comm_world.comm.c_cube_dim     = opal_cube_dim((int)size);
+    ompi_mpi_comm_world.comm.error_handler  = &ompi_mpi_errors_are_fatal.eh;
+    OBJ_RETAIN( &ompi_mpi_errors_are_fatal.eh );
+    OMPI_COMM_SET_PML_ADDED(&ompi_mpi_comm_world.comm);
+    opal_pointer_array_set_item (&ompi_mpi_communicators, 0, &ompi_mpi_comm_world);
 
-    strncpy (ompi_mpi_comm_world.c_name, "MPI_COMM_WORLD", 
+    MEMCHECKER (memset (ompi_mpi_comm_world.comm.c_name, 0, MPI_MAX_OBJECT_NAME));
+    strncpy (ompi_mpi_comm_world.comm.c_name, "MPI_COMM_WORLD",
              strlen("MPI_COMM_WORLD")+1 );
-    ompi_mpi_comm_world.c_flags |= OMPI_COMM_NAMEISSET;
-    ompi_mpi_comm_world.c_flags |= OMPI_COMM_INTRINSIC;
+    ompi_mpi_comm_world.comm.c_flags |= OMPI_COMM_NAMEISSET;
+    ompi_mpi_comm_world.comm.c_flags |= OMPI_COMM_INTRINSIC;
 
     /* We have to create a hash (although it is legal to leave this
        filed NULL -- the attribute accessor functions will intepret
        this as "there are no attributes cached on this object")
        because MPI_COMM_WORLD has some predefined attributes. */
-    ompi_attr_hash_init(&ompi_mpi_comm_world.c_keyhash);
+    ompi_attr_hash_init(&ompi_mpi_comm_world.comm.c_keyhash);
 
     /* Setup MPI_COMM_SELF */
     OBJ_CONSTRUCT(&ompi_mpi_comm_self, ompi_communicator_t);
     group = OBJ_NEW(ompi_group_t);
     group->grp_proc_pointers = ompi_proc_self(&size);
     group->grp_my_rank       = 0;
-    group->grp_proc_count    = size;
-    group->grp_flags        |= OMPI_GROUP_INTRINSIC;
+    group->grp_proc_count    = (int)size;
+    OMPI_GROUP_SET_INTRINSIC (group);
+    OMPI_GROUP_SET_DENSE (group);
+    
+    ompi_mpi_comm_self.comm.c_contextid    = 1;
+    ompi_mpi_comm_self.comm.c_f_to_c_index = 1;
+    ompi_mpi_comm_self.comm.c_id_start_index = 20;
+    ompi_mpi_comm_self.comm.c_id_available = 20;
+    ompi_mpi_comm_self.comm.c_my_rank      = group->grp_my_rank;
+    ompi_mpi_comm_self.comm.c_local_group  = group;
+    ompi_mpi_comm_self.comm.c_remote_group = group;
+    OBJ_RETAIN(ompi_mpi_comm_self.comm.c_remote_group);
+    ompi_mpi_comm_self.comm.error_handler  = &ompi_mpi_errors_are_fatal.eh;
+    OBJ_RETAIN( &ompi_mpi_errors_are_fatal.eh );
+    OMPI_COMM_SET_PML_ADDED(&ompi_mpi_comm_self.comm);
+    opal_pointer_array_set_item (&ompi_mpi_communicators, 1, &ompi_mpi_comm_self);
 
-    ompi_mpi_comm_self.c_contextid    = 1;
-    ompi_mpi_comm_self.c_f_to_c_index = 1;
-    ompi_mpi_comm_self.c_my_rank      = group->grp_my_rank;
-    ompi_mpi_comm_self.c_local_group  = group;
-    ompi_mpi_comm_self.c_remote_group = group;
-    ompi_mpi_comm_self.error_handler  = &ompi_mpi_errors_are_fatal;
-    OBJ_RETAIN( &ompi_mpi_errors_are_fatal );
-    OMPI_COMM_SET_PML_ADDED(&ompi_mpi_comm_self);
-    ompi_pointer_array_set_item (&ompi_mpi_communicators, 1, &ompi_mpi_comm_self);
-
-    strncpy(ompi_mpi_comm_self.c_name,"MPI_COMM_SELF",strlen("MPI_COMM_SELF")+1);
-    ompi_mpi_comm_self.c_flags |= OMPI_COMM_NAMEISSET;
-    ompi_mpi_comm_self.c_flags |= OMPI_COMM_INTRINSIC;
+    MEMCHECKER (memset (ompi_mpi_comm_self.comm.c_name, 0, MPI_MAX_OBJECT_NAME));
+    strncpy(ompi_mpi_comm_self.comm.c_name,"MPI_COMM_SELF",strlen("MPI_COMM_SELF")+1);
+    ompi_mpi_comm_self.comm.c_flags |= OMPI_COMM_NAMEISSET;
+    ompi_mpi_comm_self.comm.c_flags |= OMPI_COMM_INTRINSIC;
 
     /* We can set MPI_COMM_SELF's keyhash to NULL because it has no
        predefined attributes.  If a user defines an attribute on
        MPI_COMM_SELF, the keyhash will automatically be created. */
-    ompi_mpi_comm_self.c_keyhash = NULL;
-    
+    ompi_mpi_comm_self.comm.c_keyhash = NULL;
+
     /* Setup MPI_COMM_NULL */
     OBJ_CONSTRUCT(&ompi_mpi_comm_null, ompi_communicator_t);
-    ompi_mpi_comm_null.c_local_group  = &ompi_mpi_group_null;
-    ompi_mpi_comm_null.c_remote_group = &ompi_mpi_group_null;
-    OBJ_RETAIN(&ompi_mpi_group_null); 
+    ompi_mpi_comm_null.comm.c_local_group  = &ompi_mpi_group_null.group;
+    ompi_mpi_comm_null.comm.c_remote_group = &ompi_mpi_group_null.group;
+    OBJ_RETAIN(&ompi_mpi_group_null.group); 
+    OBJ_RETAIN(&ompi_mpi_group_null.group);
 
-    ompi_mpi_comm_null.c_contextid    = 2;
-    ompi_mpi_comm_null.c_f_to_c_index = 2;
-    ompi_mpi_comm_null.c_my_rank      = MPI_PROC_NULL;
+    ompi_mpi_comm_null.comm.c_contextid    = 2;
+    ompi_mpi_comm_null.comm.c_f_to_c_index = 2;
+    ompi_mpi_comm_null.comm.c_my_rank      = MPI_PROC_NULL;
 
-    ompi_mpi_comm_null.error_handler  = &ompi_mpi_errors_are_fatal;
-    OBJ_RETAIN( &ompi_mpi_errors_are_fatal );
-    ompi_pointer_array_set_item (&ompi_mpi_communicators, 2, &ompi_mpi_comm_null);
+    ompi_mpi_comm_null.comm.error_handler  = &ompi_mpi_errors_are_fatal.eh;
+    OBJ_RETAIN( &ompi_mpi_errors_are_fatal.eh );
+    opal_pointer_array_set_item (&ompi_mpi_communicators, 2, &ompi_mpi_comm_null);
 
-    strncpy(ompi_mpi_comm_null.c_name,"MPI_COMM_NULL",strlen("MPI_COMM_NULL")+1);
-    ompi_mpi_comm_null.c_flags |= OMPI_COMM_NAMEISSET;
-    ompi_mpi_comm_null.c_flags |= OMPI_COMM_INTRINSIC;
+    MEMCHECKER (memset (ompi_mpi_comm_null.comm.c_name, 0, MPI_MAX_OBJECT_NAME));
+    strncpy(ompi_mpi_comm_null.comm.c_name,"MPI_COMM_NULL",strlen("MPI_COMM_NULL")+1);
+    ompi_mpi_comm_null.comm.c_flags |= OMPI_COMM_NAMEISSET;
+    ompi_mpi_comm_null.comm.c_flags |= OMPI_COMM_INTRINSIC;
 
     /* Initialize the parent communicator to MPI_COMM_NULL */
-    ompi_mpi_comm_parent = &ompi_mpi_comm_null;
+    ompi_mpi_comm_parent = &ompi_mpi_comm_null.comm;
     OBJ_RETAIN(&ompi_mpi_comm_null);
-    OBJ_RETAIN(&ompi_mpi_group_null);
-    OBJ_RETAIN(&ompi_mpi_errors_are_fatal);
+    OBJ_RETAIN(&ompi_mpi_group_null.group);
+    OBJ_RETAIN(&ompi_mpi_errors_are_fatal.eh);
 
     /* initialize the comm_reg stuff for multi-threaded comm_cid
        allocation */
@@ -159,7 +179,7 @@ int ompi_comm_init(void)
 
 ompi_communicator_t *ompi_comm_allocate ( int local_size, int remote_size )
 {
-    ompi_communicator_t *new_comm=NULL;
+    ompi_communicator_t *new_comm;
 
     /* create new communicator element */
     new_comm = OBJ_NEW(ompi_communicator_t);
@@ -167,8 +187,7 @@ ompi_communicator_t *ompi_comm_allocate ( int local_size, int remote_size )
     if ( 0 < remote_size ) {
         new_comm->c_remote_group = ompi_group_allocate (remote_size);
         new_comm->c_flags |= OMPI_COMM_INTER;
-    }
-    else {
+    } else {
         /* 
          * simplifies some operations (e.g. p2p), if 
          * we can always use the remote group 
@@ -192,13 +211,13 @@ int ompi_comm_finalize(void)
     OBJ_DESTRUCT( &ompi_mpi_comm_self );
 
     /* disconnect all dynamic communicators */
-    ompi_comm_dyn_finalize();
+    ompi_dpm.dyn_finalize();
 
     /* Shut down MPI_COMM_WORLD */
     OBJ_DESTRUCT( &ompi_mpi_comm_world );
 
     /* Shut down the parent communicator, if it exists */
-    if( ompi_mpi_comm_parent != &ompi_mpi_comm_null ) {
+    if( ompi_mpi_comm_parent != &ompi_mpi_comm_null.comm ) {
         /* Note that we pass ompi_mpi_comm_parent here
            (vs. &ompi_mpi_comm_parent) because it is of type
            (ompi_communicator_t*), *NOT* (ompi_communicator_t).  This
@@ -206,19 +225,37 @@ int ompi_comm_finalize(void)
            during init, and we just set this pointer to it.  Hence, we
            just pass in the pointer here. */
        OBJ_DESTRUCT (ompi_mpi_comm_parent);
+
+       /* Please note, that the we did increase the reference count
+          for ompi_mpi_comm_null, ompi_mpi_group_null, and 
+          ompi_mpi_errors_are_fatal in ompi_comm_init because of 
+          ompi_mpi_comm_parent.  In case a 
+          parent communicator is really created, the ref. counters
+          for these objects are decreased again by one. However, in a 
+          static scenario, we should ideally decrease the ref. counter
+          for these objects by one here. The problem just is, that 
+          if the app had a parent_comm, and this has been freed/disconnected,
+          ompi_comm_parent points again to ompi_comm_null, the reference count 
+          for these objects has not been increased again.
+          So the point is, if ompi_mpi_comm_parent == &ompi_mpi_comm_null
+          we do not know whether we have to decrease the ref count for
+          those three objects or not. Since this is a constant, non-increasing
+          amount of memory, we stick with the current solution for now, 
+          namely don't do anything.
+       */  
     }
 
     /* Shut down MPI_COMM_NULL */
     OBJ_DESTRUCT( &ompi_mpi_comm_null );
 
     /* Check whether we have some communicators left */
-    max = ompi_pointer_array_get_size(&ompi_mpi_communicators);
+    max = opal_pointer_array_get_size(&ompi_mpi_communicators);
     for ( i=3; i<max; i++ ) {
-        comm = (ompi_communicator_t *)ompi_pointer_array_get_item(&ompi_mpi_communicators, i);
+        comm = (ompi_communicator_t *)opal_pointer_array_get_item(&ompi_mpi_communicators, i);
         if ( NULL != comm ) {
             /* Communicator has not been freed before finalize */
             OBJ_RELEASE(comm);
-            comm=(ompi_communicator_t *)ompi_pointer_array_get_item(&ompi_mpi_communicators, i);
+            comm=(ompi_communicator_t *)opal_pointer_array_get_item(&ompi_mpi_communicators, i);
             if ( NULL != comm ) {
                 /* Still here ? */
                 if ( ompi_debug_show_handle_leaks && !(OMPI_COMM_IS_FREED(comm)) ){
@@ -257,6 +294,8 @@ static void ompi_comm_construct(ompi_communicator_t* comm)
     comm->c_f_to_c_index = MPI_UNDEFINED;
     comm->c_name[0]      = '\0';
     comm->c_contextid    = MPI_UNDEFINED;
+    comm->c_id_available = MPI_UNDEFINED;
+    comm->c_id_start_index = MPI_UNDEFINED;
     comm->c_flags        = 0;
     comm->c_my_rank      = 0;
     comm->c_cube_dim     = 0;
@@ -264,7 +303,6 @@ static void ompi_comm_construct(ompi_communicator_t* comm)
     comm->c_remote_group = NULL;
     comm->error_handler  = NULL;
     comm->c_pml_comm     = NULL;
-    comm->c_pml_procs    = NULL;
     comm->c_topo         = NULL;
     comm->c_topo_component = NULL;
     comm->c_topo_comm    = NULL; 
@@ -274,22 +312,16 @@ static void ompi_comm_construct(ompi_communicator_t* comm)
        this communiucator */
     comm->c_keyhash = NULL;
 
-#if OMPI_ENABLE_DEBUG
-    memset (&(comm->c_coll), 0, sizeof(mca_coll_base_module_1_0_0_t));
-#endif
-
-    comm->c_coll_selected_component = NULL;
-    comm->c_coll_selected_module    = NULL;
-    comm->c_coll_selected_data      = NULL;
-    comm->c_coll_basic_module       = NULL;
-    comm->c_coll_basic_data         = NULL;
-
     comm->errhandler_type           = OMPI_ERRHANDLER_TYPE_COMM;
 #ifdef OMPI_WANT_PERUSE
     comm->c_peruse_handles          = NULL;
 #endif
 
-    return;
+    /* Need to zero out the collectives module because we sometimes
+       call coll_unselect without a matching call to coll_select, and
+       we need an easy way for the coll base code to realize we've
+       done this. */
+    memset(&comm->c_coll, 0, sizeof(mca_coll_base_comm_coll_t));
 }
 
 static void ompi_comm_destruct(ompi_communicator_t* comm)
@@ -307,7 +339,7 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
 
     /*  Check if the communicator is a topology */
     if ( MPI_COMM_NULL != comm && 
-	 (OMPI_COMM_IS_CART(comm) || OMPI_COMM_IS_GRAPH(comm))) {
+         (OMPI_COMM_IS_CART(comm) || OMPI_COMM_IS_GRAPH(comm))) {
 
         /* check and free individual things */
         
@@ -366,6 +398,9 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
         OBJ_RELEASE ( comm->c_local_group );
         comm->c_local_group = NULL;
         if ( OMPI_COMM_IS_INTRA(comm) ) {
+            /* We have to decrement the ref count on the remote group
+               even if it is identical to the local one in case of intra-comm */
+            OBJ_RELEASE ( comm->c_remote_group );
             comm->c_remote_group = NULL;
         }
     }
@@ -383,9 +418,9 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
 
     /* reset the ompi_comm_f_to_c_table entry */
     if ( MPI_UNDEFINED != comm->c_f_to_c_index && 
-         NULL != ompi_pointer_array_get_item(&ompi_mpi_communicators,
+         NULL != opal_pointer_array_get_item(&ompi_mpi_communicators,
                                              comm->c_f_to_c_index )) {
-        ompi_pointer_array_set_item ( &ompi_mpi_communicators,
+        opal_pointer_array_set_item ( &ompi_mpi_communicators,
                                       comm->c_f_to_c_index, NULL);
     }
 
