@@ -10,6 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      UT-Battelle, LLC
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,10 +19,15 @@
  */
 
 #include "ompi_config.h"
+#include <stdio.h>
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
+#include <stdio.h>
+
+#include "opal/prefetch.h"
+#include "opal/util/arch.h"
 
 #include "ompi/datatype/datatype.h"
 #include "ompi/datatype/convertor.h"
@@ -29,9 +35,8 @@
 #include "ompi/datatype/datatype_checksum.h"
 #include "ompi/datatype/datatype_prototypes.h"
 #include "ompi/datatype/convertor_internal.h"
-#include "ompi/datatype/dt_arch.h"
 
-extern int ompi_ddt_local_sizes[DT_MAX_PREDEFINED];
+extern size_t ompi_ddt_local_sizes[DT_MAX_PREDEFINED];
 extern int ompi_convertor_create_stack_with_pos_general( ompi_convertor_t* convertor,
                                                          int starting_point, const int* sizes );
 
@@ -52,6 +57,9 @@ OBJ_CLASS_INSTANCE(ompi_convertor_t, opal_object_t, ompi_convertor_construct, om
 
 static ompi_convertor_master_t* ompi_convertor_master_list = NULL;
 
+extern conversion_fct_t ompi_ddt_heterogeneous_copy_functions[DT_MAX_PREDEFINED];
+extern conversion_fct_t ompi_ddt_copy_functions[DT_MAX_PREDEFINED];
+
 void ompi_convertor_destroy_masters( void )
 {
     ompi_convertor_master_t* master = ompi_convertor_master_list;
@@ -59,19 +67,27 @@ void ompi_convertor_destroy_masters( void )
     while( NULL != master ) {
         ompi_convertor_master_list = master->next;
         master->next = NULL;
+        /* Cleanup the conversion function if not one of the defaults */
+        if( (master->pFunctions != ompi_ddt_heterogeneous_copy_functions) &&
+            (master->pFunctions != ompi_ddt_copy_functions) )
+            free( master->pFunctions );
+
         free( master );
         master = ompi_convertor_master_list;
     }
 }
 
-extern conversion_fct_t ompi_ddt_heterogeneous_copy_functions[DT_MAX_PREDEFINED];
-extern conversion_fct_t ompi_ddt_copy_functions[DT_MAX_PREDEFINED];
-
-ompi_convertor_master_t* ompi_convertor_find_or_create_master( uint32_t remote_arch )
+/**
+ * Find or create a convertor suitable for the remote architecture. If there
+ * is already a master convertor for this architecture then return it.
+ * Otherwise, create and initialize a full featured master convertor.
+ */
+ompi_convertor_master_t*
+ompi_convertor_find_or_create_master( uint32_t remote_arch )
 {
     ompi_convertor_master_t* master = ompi_convertor_master_list;
     int i;
-    int32_t* remote_sizes;
+    size_t* remote_sizes;
 
     while( NULL != master ) {
         if( master->remote_arch == remote_arch )
@@ -86,27 +102,39 @@ ompi_convertor_master_t* ompi_convertor_find_or_create_master( uint32_t remote_a
     ompi_convertor_master_list = master;
     master->remote_arch = remote_arch;
     master->flags       = 0;
+    master->hetero_mask = 0;
     /* Most of the sizes will be identical, so for now just make a copy of
      * the local ones. As master->remote_sizes is defined as being an array of
      * consts we have to manually cast it before using it for writing purposes.
      */
-    remote_sizes = (int32_t*)master->remote_sizes;
-
+    remote_sizes = (size_t*)master->remote_sizes;
     for( i = DT_CHAR; i < DT_MAX_PREDEFINED; i++ ) {
         remote_sizes[i] = ompi_ddt_local_sizes[i];
     }
+    /**
+     * If the local and remote architecture are the same there is no need
+     * to check for the remote data sizes. They will always be the same as
+     * the local ones.
+     */
+    if( master->remote_arch == ompi_mpi_local_arch ) {
+        master->pFunctions = ompi_ddt_copy_functions;
+        master->flags |= CONVERTOR_HOMOGENEOUS;
+        return master;
+    }
+
     /* Find out the remote bool size */
-    if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_BOOLIS8 ) ) {
+    if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_BOOLIS8 ) ) {
         remote_sizes[DT_CXX_BOOL] = 1;
-    } else if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LOGICALIS16 ) ) {
+    } else if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_BOOLIS16 ) ) {
         remote_sizes[DT_CXX_BOOL] = 2;
-    } else if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LOGICALIS32 ) ) {
+    } else if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_BOOLIS32 ) ) {
         remote_sizes[DT_CXX_BOOL] = 4;
     } else {
         opal_output( 0, "Unknown sizeof(bool) for the remote architecture\n" );
     }
+
     /* check the length of the long */
-    if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LONGIS64 ) ) {
+    if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_LONGIS64 ) ) {
         remote_sizes[DT_LONG]               = 8;
         remote_sizes[DT_UNSIGNED_LONG]      = 8;
         remote_sizes[DT_LONG_LONG_INT]      = 8;
@@ -116,23 +144,48 @@ ompi_convertor_master_t* ompi_convertor_find_or_create_master( uint32_t remote_a
      * unknown (if Fortran is not supported on the remote library). If this is
      * the case, just let the remote logical size to match the local size.
      */
-    if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LOGICALIS8 ) ) {
+    if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_LOGICALIS8 ) ) {
         remote_sizes[DT_LOGIC] = 1;
-    } else if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LOGICALIS16 ) ) {
+    } else if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_LOGICALIS16 ) ) {
         remote_sizes[DT_LOGIC] = 2;
-    } else if( ompi_arch_checkmask( &master->remote_arch, OMPI_ARCH_LOGICALIS32 ) ) {
+    } else if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_LOGICALIS32 ) ) {
         remote_sizes[DT_LOGIC] = 4;
     } else {
         opal_output( 0, "Unknown sizeof(fortran logical) for the remote architecture\n" );
     }
 
-    if( master->remote_arch == ompi_mpi_local_arch ) {
-        master->pFunctions = ompi_ddt_copy_functions;
-        master->flags |= CONVERTOR_HOMOGENEOUS;
-    } else {
-        master->pFunctions = ompi_ddt_heterogeneous_copy_functions;
+    /**
+     * Now we can compute the conversion mask. For all sizes where the remote
+     * and local architecture differ a conversion is needed. Moreover, if the
+     * 2 architectures don't have the same endianess all data with a length
+     * over 2 bytes (with the exception of logicals) have to be byte-swapped.
+     */
+    for( i = DT_CHAR; i < DT_MAX_PREDEFINED; i++ ) {
+        if( remote_sizes[i] != ompi_ddt_local_sizes[i] )
+            master->hetero_mask |= (((uint64_t)1) << i);
     }
+    if( opal_arch_checkmask( &master->remote_arch, OPAL_ARCH_ISBIGENDIAN ) !=
+        opal_arch_checkmask( &ompi_mpi_local_arch, OPAL_ARCH_ISBIGENDIAN ) ) {
+        uint64_t hetero_mask = 0;
 
+        for( i = DT_CHAR; i < DT_MAX_PREDEFINED; i++ ) {
+            if( remote_sizes[i] > 1 )
+                hetero_mask |= (((uint64_t)1) << i);
+        }
+        hetero_mask &= ~((((uint64_t)1) << DT_LOGIC) | (((uint64_t)1) << DT_CXX_BOOL));
+        master->hetero_mask |= hetero_mask;
+    }
+    master->pFunctions = (conversion_fct_t*)malloc( sizeof(ompi_ddt_heterogeneous_copy_functions) );
+    /**
+     * Usually the heterogeneous functions are slower than the copy ones. Let's
+     * try to minimize the usage of the heterogeneous versions.
+     */
+    for( i = DT_CHAR; i < DT_MAX_PREDEFINED; i++ ) {
+        if( master->hetero_mask & (((uint64_t)1) << i) )
+            master->pFunctions[i] = ompi_ddt_heterogeneous_copy_functions[i];
+        else
+            master->pFunctions[i] = ompi_ddt_copy_functions[i];
+    }
     /* We're done so far, return the mater convertor */
     return master;
 }
@@ -154,17 +207,16 @@ ompi_convertor_t* ompi_convertor_create( int32_t remote_arch, int32_t mode )
 
 #define OMPI_CONVERTOR_SET_STATUS_BEFORE_PACK_UNPACK( CONVERTOR, IOV, OUT, MAX_DATA ) \
     do {                                                                \
-        (CONVERTOR)->checksum = OPAL_CSUM_ZERO;                         \
-        (CONVERTOR)->csum_ui1 = 0;                                      \
-        (CONVERTOR)->csum_ui2 = 0;                                      \
-                                                                        \
         /* protect against over packing data */                         \
-        if( (CONVERTOR)->flags & CONVERTOR_COMPLETED ) {                \
+        if( OPAL_UNLIKELY((CONVERTOR)->flags & CONVERTOR_COMPLETED) ) { \
             (IOV)[0].iov_len = 0;                                       \
             *(OUT) = 0;                                                 \
             *(MAX_DATA) = 0;                                            \
             return 1;  /* nothing to do */                              \
         }                                                               \
+        (CONVERTOR)->checksum = OPAL_CSUM_ZERO;                         \
+        (CONVERTOR)->csum_ui1 = 0;                                      \
+        (CONVERTOR)->csum_ui2 = 0;                                      \
         assert( (CONVERTOR)->bConverted < (CONVERTOR)->local_size );    \
     } while(0)
 
@@ -176,105 +228,101 @@ ompi_convertor_t* ompi_convertor_create( int32_t remote_arch, int32_t mode )
  */
 int32_t ompi_convertor_pack( ompi_convertor_t* pConv,
                              struct iovec* iov, uint32_t* out_size,
-                             size_t* max_data, int32_t* freeAfter )
+                             size_t* max_data )
 {
     OMPI_CONVERTOR_SET_STATUS_BEFORE_PACK_UNPACK( pConv, iov, out_size, max_data );
 
-    if( !(pConv->flags & CONVERTOR_WITH_CHECKSUM) &&
-        (pConv->flags & DT_FLAG_NO_GAPS) ) {
-        /* We are doing conversion on a predefined contiguous datatype. The
-         * convertor contain minimal informations, we only use the bConverted
-         * to manage the conversion.
-         */
-        uint32_t i;
-        size_t initial_bConverted = pConv->bConverted;
-        size_t pending_length = pConv->local_size - pConv->bConverted;
-        char* base_pointer;
-
-        if( (*max_data) < pending_length )
-            pending_length = (*max_data);
-
-        for( i = 0; (i < *out_size) && (0 != pending_length); i++ ) {
-            base_pointer = pConv->pBaseBuf + pConv->bConverted + pConv->pDesc->true_lb;
-
-            if( iov[i].iov_len > pending_length )
-                iov[i].iov_len = pending_length;
-
-            if( NULL == iov[i].iov_base ) {
-                iov[i].iov_base = base_pointer;
-            } else {
-                MEMCPY( iov[i].iov_base, base_pointer, iov[i].iov_len );
-            }
-            pConv->bConverted += iov[i].iov_len;
-            pending_length -= iov[i].iov_len;
-        }
-        *out_size = i;
-        *max_data = pConv->bConverted - initial_bConverted;
-        if( pConv->bConverted == pConv->local_size ) {
-            pConv->flags |= CONVERTOR_COMPLETED;
-            return 1;
-        }
-        return 0;
-    }
-
-    return pConv->fAdvance( pConv, iov, out_size, max_data, freeAfter );
-}
-
-inline int32_t ompi_convertor_unpack( ompi_convertor_t* pConv,
-                                      struct iovec* iov, uint32_t* out_size,
-                                      size_t* max_data, int32_t* freeAfter )
-{
-    OMPI_CONVERTOR_SET_STATUS_BEFORE_PACK_UNPACK( pConv, iov, out_size, max_data );
-
-    if( !(pConv->flags & CONVERTOR_WITH_CHECKSUM) &&
-        ((pConv->flags & (CONVERTOR_HOMOGENEOUS | DT_FLAG_NO_GAPS)) ==
-         (CONVERTOR_HOMOGENEOUS | DT_FLAG_NO_GAPS)) ) {
+    if( OPAL_LIKELY(pConv->flags & CONVERTOR_NO_OP) ) {
         /* We are doing conversion on a contiguous datatype on a homogeneous
          * environment. The convertor contain minimal informations, we only
          * use the bConverted to manage the conversion.
          */
         uint32_t i;
-        size_t next_length;
-        char* base_pointer;
+        unsigned char* base_pointer;
+        size_t pending_length = pConv->local_size - pConv->bConverted;
+
+        *max_data = pending_length;
+        ompi_convertor_get_current_pointer( pConv, (void**)&base_pointer );
 
-        *max_data = pConv->bConverted;
         for( i = 0; i < *out_size; i++ ) {
-            base_pointer = pConv->pBaseBuf + pConv->bConverted + pConv->pDesc->true_lb;
-            next_length = pConv->bConverted + iov[i].iov_len;
-            if( next_length >= pConv->local_size ) {
-                pConv->bConverted = pConv->local_size;
-                iov[i].iov_len -= (next_length - pConv->local_size);
-                MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
-                /*opal_output( 0, "copy at %p %d bytes [initial ptr %p] *last*\n", base_pointer,
-                  iov[i].iov_len, pConv->pBaseBuf );*/
-                goto predefined_data_unpack;
+            if( iov[i].iov_len >= pending_length ) {
+                goto complete_contiguous_data_pack;
             }
-            MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
-            /*opal_output( 0, "copy at %p %d bytes [initial ptr %p]\n", base_pointer,
-              iov[i].iov_len, pConv->pBaseBuf );*/
-            pConv->bConverted = next_length;
+            if( OPAL_LIKELY(NULL == iov[i].iov_base) )
+                iov[i].iov_base = (IOVBASE_TYPE *) base_pointer;
+            else
+                MEMCPY( iov[i].iov_base, base_pointer, iov[i].iov_len );
+            pending_length -= iov[i].iov_len;
+            base_pointer += iov[i].iov_len;
         }
-        *max_data = pConv->bConverted - (*max_data);
+        *max_data -= pending_length;
+        pConv->bConverted += (*max_data);
         return 0;
-    predefined_data_unpack:
+    complete_contiguous_data_pack:
+        iov[i].iov_len = pending_length;
+        if( OPAL_LIKELY(NULL == iov[i].iov_base) )
+            iov[i].iov_base = (IOVBASE_TYPE *) base_pointer;
+        else
+            MEMCPY( iov[i].iov_base, base_pointer, iov[i].iov_len );
+        pConv->bConverted = pConv->local_size;
         *out_size = i + 1;
-        *max_data = pConv->bConverted - (*max_data);
         pConv->flags |= CONVERTOR_COMPLETED;
         return 1;
     }
 
-    return pConv->fAdvance( pConv, iov, out_size, max_data, freeAfter );
+    return pConv->fAdvance( pConv, iov, out_size, max_data );
+}
+
+int32_t ompi_convertor_unpack( ompi_convertor_t* pConv,
+                               struct iovec* iov, uint32_t* out_size,
+                               size_t* max_data )
+{
+    OMPI_CONVERTOR_SET_STATUS_BEFORE_PACK_UNPACK( pConv, iov, out_size, max_data );
+
+    if( OPAL_LIKELY(pConv->flags & CONVERTOR_NO_OP) ) {
+        /* We are doing conversion on a contiguous datatype on a homogeneous
+         * environment. The convertor contain minimal informations, we only
+         * use the bConverted to manage the conversion.
+         */
+        uint32_t i;
+        unsigned char* base_pointer;
+        size_t pending_length = pConv->local_size - pConv->bConverted;
+
+        *max_data = pending_length;
+        ompi_convertor_get_current_pointer( pConv, (void**)&base_pointer );
+
+        for( i = 0; i < *out_size; i++ ) {
+            if( iov[i].iov_len >= pending_length ) {
+                goto complete_contiguous_data_unpack;
+            }
+            MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
+            pending_length -= iov[i].iov_len;
+            base_pointer += iov[i].iov_len;
+        }
+        *max_data -= pending_length;
+        pConv->bConverted += (*max_data);
+        return 0;
+    complete_contiguous_data_unpack:
+        iov[i].iov_len = pending_length;
+        MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
+        pConv->bConverted = pConv->local_size;
+        *out_size = i + 1;
+        pConv->flags |= CONVERTOR_COMPLETED;
+        return 1;
+    }
+
+    return pConv->fAdvance( pConv, iov, out_size, max_data );
 }
 
 static inline
 int ompi_convertor_create_stack_with_pos_contig( ompi_convertor_t* pConvertor,
-                                                 int starting_point, const int* sizes )
+                                                 size_t starting_point, const size_t* sizes )
 {
     dt_stack_t* pStack;   /* pointer to the position on the stack */
     const ompi_datatype_t* pData = pConvertor->pDesc;
     dt_elem_desc_t* pElems;
     uint32_t count;
-    long extent;
+    ptrdiff_t extent;
 
     pStack = pConvertor->pStack;
     /* The prepare function already make the selection on which data representation
@@ -282,21 +330,20 @@ int ompi_convertor_create_stack_with_pos_contig( ompi_convertor_t* pConvertor,
      */
     pElems = pConvertor->use_desc->desc;
 
-    count = starting_point / pData->size;
+    count = (uint32_t)(starting_point / pData->size);
     extent = pData->ub - pData->lb;
 
     pStack[0].type     = DT_LOOP;  /* the first one is always the loop */
     pStack[0].count    = pConvertor->count - count;
     pStack[0].index    = -1;
-    pStack[0].end_loop = pConvertor->use_desc->used;
     pStack[0].disp     = count * extent;
 
     /* now compute the number of pending bytes */
-    count = starting_point - count * pData->size;
+    count = (uint32_t)(starting_point - count * pData->size);
     /* we save the current displacement starting from the begining
      * of this data.
      */
-    if( 0 == count ) {
+    if( OPAL_LIKELY(0 == count) ) {
         pStack[1].type     = pElems->elem.common.type;
         pStack[1].count    = pElems->elem.count;
         pStack[1].disp     = pElems->elem.disp;
@@ -306,7 +353,6 @@ int ompi_convertor_create_stack_with_pos_contig( ompi_convertor_t* pConvertor,
         pStack[1].disp  = pData->true_lb + count;
     }
     pStack[1].index    = 0;  /* useless */
-    pStack[1].end_loop = 0;  /* useless */
 
     pConvertor->bConverted = starting_point;
     pConvertor->stack_pos = 1;
@@ -316,11 +362,16 @@ int ompi_convertor_create_stack_with_pos_contig( ompi_convertor_t* pConvertor,
 
 static inline
 int ompi_convertor_create_stack_at_begining( ompi_convertor_t* convertor,
-                                             const int* sizes )
+                                             const size_t* sizes )
 {
     dt_stack_t* pStack = convertor->pStack;
     dt_elem_desc_t* pElems;
 
+    /* The prepare function already make the selection on which data representation
+     * we have to use: normal one or the optimized version ?
+     */
+    pElems = convertor->use_desc->desc;
+
     convertor->stack_pos      = 1;
     convertor->partial_length = 0;
     convertor->bConverted     = 0;
@@ -331,15 +382,9 @@ int ompi_convertor_create_stack_at_begining( ompi_convertor_t* convertor,
     pStack[0].index = -1;
     pStack[0].count = convertor->count;
     pStack[0].disp  = 0;
-    pStack[0].end_loop = convertor->use_desc->used;
-    /* The prepare function already make the selection on which data representation
-     * we have to use: normal one or the optimized version ?
-     */
-    pElems = convertor->use_desc->desc;
 
     pStack[1].index = 0;
     pStack[1].disp = 0;
-    pStack[1].end_loop = 0;
     if( pElems[0].elem.common.type == DT_LOOP ) {
         pStack[1].count = pElems[0].loop.loops;
     } else {
@@ -361,7 +406,7 @@ int32_t ompi_convertor_set_position_nocheck( ompi_convertor_t* convertor,
         rc = ompi_convertor_create_stack_at_begining( convertor, ompi_ddt_local_sizes );
         if( 0 == (*position) ) return rc;
     }
-    if( convertor->flags & DT_FLAG_CONTIGUOUS ) {
+    if( OPAL_LIKELY(convertor->flags & DT_FLAG_CONTIGUOUS) ) {
         rc = ompi_convertor_create_stack_with_pos_contig( convertor, (*position),
                                                           ompi_ddt_local_sizes );
     } else {
@@ -371,69 +416,90 @@ int32_t ompi_convertor_set_position_nocheck( ompi_convertor_t* convertor,
     return rc;
 }
 
-/* This macro will initialize a convertor based on a previously created convertor. The idea
- * is the move outside these function the heavy selection of architecture features for the convertors.
- *
- * I consider here that the convertor is clean, either never initialized or already cleanup.
+/**
+ * Compute the remote size.
+ */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#define OMPI_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask) \
+{                                                                         \
+    if( OPAL_UNLIKELY(0 != bdt_mask) ) {                                  \
+        ompi_convertor_master_t* master;                                  \
+        int i;                                                            \
+        convertor->flags ^= CONVERTOR_HOMOGENEOUS;                        \
+        bdt_mask = datatype->bdt_used;                                    \
+        master = convertor->master;                                       \
+        convertor->remote_size = 0;                                       \
+        for( i = DT_CHAR; i < DT_MAX_PREDEFINED; i++ ) {                  \
+            if( bdt_mask & ((uint64_t)1 << i) ) {                         \
+                convertor->remote_size += (datatype->btypes[i] *          \
+                                           master->remote_sizes[i]);      \
+            }                                                             \
+        }                                                                 \
+        convertor->remote_size *= convertor->count;                       \
+        convertor->use_desc = &(datatype->desc);                          \
+    }                                                                     \
+}
+#else
+#define OMPI_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask)
+#endif  /* OMPI_ENABLE_HETEROGENEOUS_SUPPORT */
+
+/**
+ * This macro will initialize a convertor based on a previously created
+ * convertor. The idea is the move outside these function the heavy
+ * selection of architecture features for the convertors. I consider
+ * here that the convertor is clean, either never initialized or already
+ * cleaned.
  */
 #define OMPI_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf )  \
     {                                                                   \
-        convertor->pBaseBuf        = (char*)pUserBuf;                   \
-        convertor->count           = count;                             \
+        uint64_t bdt_mask;                                              \
                                                                         \
-        /* Grab the datatype part of the flags */                       \
-        convertor->flags         &= CONVERTOR_TYPE_MASK;                \
-        convertor->flags         |= (CONVERTOR_DATATYPE_MASK & datatype->flags); \
-        convertor->pDesc          = (ompi_datatype_t*)datatype;         \
-                                                                        \
-        /* Compute the local and remote sizes */                        \
-        convertor->local_size = convertor->count * datatype->size;      \
         /* If the data is empty we just mark the convertor as           \
          * completed. With this flag set the pack and unpack functions  \
-         * will not do anything. In order to decrease the data          \
-         * dependencies (and to speed-up this code) we will not test    \
-         * the convertor->local_size but we can test the 2 components.  \
+         * will not do anything.                                        \
          */                                                             \
-        if( (0 == convertor->count) || (0 == datatype->size) ) {        \
+        if( OPAL_UNLIKELY((0 == count) || (0 == datatype->size)) ) {    \
             convertor->flags |= CONVERTOR_COMPLETED;                    \
             convertor->local_size = convertor->remote_size = 0;         \
             return OMPI_SUCCESS;                                        \
         }                                                               \
+        bdt_mask = datatype->bdt_used & convertor->master->hetero_mask; \
+        /* Compute the local in advance */                              \
+        convertor->local_size = count * datatype->size;                 \
+        convertor->pBaseBuf   = (unsigned char*)pUserBuf;               \
+        convertor->count      = count;                                  \
                                                                         \
-        if( convertor->remoteArch == ompi_mpi_local_arch ) {            \
-            convertor->remote_size = convertor->local_size;             \
-            convertor->use_desc = &(datatype->opt_desc);                \
-            convertor->flags |= CONVERTOR_HOMOGENEOUS;                  \
-        } else {                                                        \
-            int i;                                                      \
-            uint64_t bdt_mask = datatype->bdt_used >> DT_CHAR;          \
-            ompi_convertor_master_t* master;                            \
-            master = convertor->master;                                 \
-            convertor->remote_size = 0;                                 \
-            for( i = DT_CHAR; bdt_mask != 0; i++, bdt_mask >>= 1 ) {    \
-                if( bdt_mask & ((unsigned long long)1) ) {              \
-                    convertor->remote_size += (datatype->btypes[i] *    \
-                                               master->remote_sizes[i]);\
-                }                                                       \
+        /* Grab the datatype part of the flags */                       \
+        convertor->flags     &= CONVERTOR_TYPE_MASK;                    \
+        convertor->flags     |= (CONVERTOR_DATATYPE_MASK & datatype->flags); \
+        convertor->flags     |= (CONVERTOR_NO_OP | CONVERTOR_HOMOGENEOUS); \
+        convertor->pDesc      = (ompi_datatype_t*)datatype;             \
+        convertor->bConverted = 0;                                      \
+        /* By default consider the optimized description */             \
+        convertor->use_desc = &(datatype->opt_desc);                    \
+                                                                        \
+        convertor->remote_size = convertor->local_size;                 \
+        if( OPAL_LIKELY(convertor->remoteArch == ompi_mpi_local_arch) ) { \
+            if( (convertor->flags & (CONVERTOR_WITH_CHECKSUM | DT_FLAG_NO_GAPS)) == DT_FLAG_NO_GAPS ) { \
+                return OMPI_SUCCESS;                                    \
             }                                                           \
-            convertor->remote_size *= convertor->count;                 \
-            convertor->use_desc = &(datatype->desc);                    \
-            if( convertor->flags & CONVERTOR_SEND ) {                   \
-                convertor->bConverted = 0;                              \
+            if( ((convertor->flags & (CONVERTOR_WITH_CHECKSUM | DT_FLAG_CONTIGUOUS)) \
+                 == DT_FLAG_CONTIGUOUS) && (1 == count) ) {             \
                 return OMPI_SUCCESS;                                    \
             }                                                           \
         }                                                               \
+                                                                        \
+        OMPI_CONVERTOR_COMPUTE_REMOTE_SIZE( convertor, datatype,        \
+                                            bdt_mask );                 \
         assert( NULL != convertor->use_desc->desc );                    \
         /* For predefined datatypes (contiguous) do nothing more */     \
         /* if checksum is enabled then always continue */               \
-        if( !(convertor->flags & CONVERTOR_WITH_CHECKSUM) &&            \
-            (convertor->flags & DT_FLAG_NO_GAPS) &&                     \
-            ((convertor->flags & CONVERTOR_SEND) ||                     \
-             (convertor->flags & CONVERTOR_HOMOGENEOUS)) ) {            \
-            convertor->bConverted = 0;                                  \
+        if( ((convertor->flags & (CONVERTOR_WITH_CHECKSUM | DT_FLAG_NO_GAPS)) \
+             == DT_FLAG_NO_GAPS) &&                                     \
+            (convertor->flags & (CONVERTOR_SEND | CONVERTOR_HOMOGENEOUS)) ) { \
             return OMPI_SUCCESS;                                        \
         }                                                               \
-                                                                        \
+        convertor->flags &= ~CONVERTOR_NO_OP;                           \
         {                                                               \
             uint32_t required_stack_length = datatype->btypes[DT_LOOP] + 1; \
                                                                         \
@@ -457,8 +523,7 @@ ompi_convertor_prepare_for_recv( ompi_convertor_t* convertor,
 {
     /* Here I should check that the data is not overlapping */
 
-    convertor->flags      |= CONVERTOR_RECV;
-    convertor->memAlloc_fn = NULL;
+    convertor->flags |= CONVERTOR_RECV;
 
     OMPI_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
@@ -469,7 +534,6 @@ ompi_convertor_prepare_for_recv( ompi_convertor_t* convertor,
         } else
 #endif
         if( convertor->pDesc->flags & DT_FLAG_CONTIGUOUS ) {
-            assert( convertor->flags & DT_FLAG_CONTIGUOUS );
             convertor->fAdvance = ompi_unpack_homogeneous_contig_checksum;
         } else {
             convertor->fAdvance = ompi_generic_simple_unpack_checksum;
@@ -481,7 +545,6 @@ ompi_convertor_prepare_for_recv( ompi_convertor_t* convertor,
         } else
 #endif
         if( convertor->pDesc->flags & DT_FLAG_CONTIGUOUS ) {
-            assert( convertor->flags & DT_FLAG_CONTIGUOUS );
             convertor->fAdvance = ompi_unpack_homogeneous_contig;
         } else {
             convertor->fAdvance = ompi_generic_simple_unpack;
@@ -496,15 +559,13 @@ ompi_convertor_prepare_for_send( ompi_convertor_t* convertor,
                                  int32_t count,
                                  const void* pUserBuf )
 {
-    convertor->flags            |= CONVERTOR_SEND;
-    convertor->memAlloc_fn       = NULL;
+    convertor->flags |= CONVERTOR_SEND;
 
     OMPI_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
     if( convertor->flags & CONVERTOR_WITH_CHECKSUM ) {
         if( datatype->flags & DT_FLAG_CONTIGUOUS ) {
-            assert( convertor->flags & DT_FLAG_CONTIGUOUS );
-            if( ((datatype->ub - datatype->lb) == (long)datatype->size) 
+            if( ((datatype->ub - datatype->lb) == (ptrdiff_t)datatype->size) 
                 || (1 >= convertor->count) )
                 convertor->fAdvance = ompi_pack_homogeneous_contig_checksum;
             else
@@ -514,8 +575,7 @@ ompi_convertor_prepare_for_send( ompi_convertor_t* convertor,
         }
     } else {
         if( datatype->flags & DT_FLAG_CONTIGUOUS ) {
-            assert( convertor->flags & DT_FLAG_CONTIGUOUS );
-            if( ((datatype->ub - datatype->lb) == (long)datatype->size) 
+            if( ((datatype->ub - datatype->lb) == (ptrdiff_t)datatype->size) 
                 || (1 >= convertor->count) )
                 convertor->fAdvance = ompi_pack_homogeneous_contig;
             else
@@ -542,19 +602,17 @@ int ompi_convertor_clone( const ompi_convertor_t* source,
                           int32_t copy_stack )
 {
     destination->remoteArch        = source->remoteArch;
-    destination->flags             = source->flags | CONVERTOR_CLONE;
+    destination->flags             = source->flags;
     destination->pDesc             = source->pDesc;
     destination->use_desc          = source->use_desc;
     destination->count             = source->count;
     destination->pBaseBuf          = source->pBaseBuf;
     destination->fAdvance          = source->fAdvance;
-    destination->memAlloc_fn       = source->memAlloc_fn;
-    destination->memAlloc_userdata = source->memAlloc_userdata;
     destination->master            = source->master;
     destination->local_size        = source->local_size;
     destination->remote_size       = source->remote_size;
     /* create the stack */
-    if( source->stack_size > DT_STATIC_STACK_SIZE ) {
+    if( OPAL_UNLIKELY(source->stack_size > DT_STATIC_STACK_SIZE) ) {
         destination->pStack = (dt_stack_t*)malloc(sizeof(dt_stack_t) * source->stack_size );
     } else {
         destination->pStack = destination->static_stack;
@@ -562,7 +620,7 @@ int ompi_convertor_clone( const ompi_convertor_t* source,
     destination->stack_size = source->stack_size;
 
     /* initialize the stack */
-    if( 0 == copy_stack ) {
+    if( OPAL_LIKELY(0 == copy_stack) ) {
         destination->bConverted = -1;
         destination->stack_pos  = -1;
     } else {
@@ -591,13 +649,13 @@ void ompi_ddt_dump_stack( const dt_stack_t* pStack, int stack_pos,
 {
     opal_output( 0, "\nStack %p stack_pos %d name %s\n", (void*)pStack, stack_pos, name );
     for( ; stack_pos >= 0; stack_pos-- ) {
-        opal_output( 0, "%d: pos %d count %d disp %ld end_loop %d ", stack_pos, pStack[stack_pos].index,
-                     pStack[stack_pos].count, pStack[stack_pos].disp, pStack[stack_pos].end_loop );
+        opal_output( 0, "%d: pos %d count %d disp %ld ", stack_pos, pStack[stack_pos].index,
+                     (int)pStack[stack_pos].count, (long)pStack[stack_pos].disp );
         if( pStack->index != -1 )
-            opal_output( 0, "\t[desc count %d disp %ld extent %d]\n",
+            opal_output( 0, "\t[desc count %d disp %ld extent %ld]\n",
                          pDesc[pStack[stack_pos].index].elem.count,
-                         pDesc[pStack[stack_pos].index].elem.disp,
-                         pDesc[pStack[stack_pos].index].elem.extent );
+                         (long)pDesc[pStack[stack_pos].index].elem.disp,
+                         (long)pDesc[pStack[stack_pos].index].elem.extent );
         else
             opal_output( 0, "\n" );
     }
