@@ -6,7 +6,7 @@
 ! Copyright (c) 2004-2005 The University of Tennessee and The University
 !                         of Tennessee Research Foundation.  All rights
 !                         reserved.
-! Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+! Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
 !                         University of Stuttgart.  All rights reserved.
 ! Copyright (c) 2004-2005 The Regents of the University of California.
 !                         All rights reserved.
@@ -81,7 +81,7 @@
 !
       integer MPI_ANY_SOURCE, MPI_ANY_TAG
       integer MPI_PROC_NULL, MPI_MAX_PROCESSOR_NAME
-      integer MPI_ROOT, MPI_MAX_DATAREP_STRING
+      integer MPI_ROOT
       integer MPI_MAX_ERROR_STRING, MPI_UNDEFINED
       integer MPI_CART, MPI_GRAPH, MPI_KEYVAL_INVALID
       integer MPI_SOURCE, MPI_TAG, MPI_ERROR
@@ -97,12 +97,9 @@
       integer MPI_DISTRIBUTE_NONE, MPI_DISTRIBUTE_DFLT_DARG
       integer MPI_TYPECLASS_INTEGER, MPI_TYPECLASS_REAL
       integer MPI_TYPECLASS_COMPLEX
-      integer MPI_SEEK_SET, MPI_SEEK_CUR, MPI_SEEK_END 
-      integer MPI_MODE_CREATE
-      integer MPI_MODE_RDONLY, MPI_MODE_WRONLY, MPI_MODE_RDWR
-      integer MPI_MODE_DELETE_ON_CLOSE, MPI_MODE_UNIQUE_OPEN
-      integer MPI_MODE_EXCL, MPI_MODE_APPEND, MPI_MODE_SEQUENTIAL
-      integer MPI_DISPLACEMENT_CURRENT
+      integer MPI_MODE_NOCHECK, MPI_MODE_NOPRECEDE, MPI_MODE_NOPUT
+      integer MPI_MODE_NOSTORE, MPI_MODE_NOSUCCEED
+      integer MPI_LOCK_EXCLUSIVE, MPI_LOCK_SHARED
       integer MPI_WIN_BASE, MPI_WIN_SIZE, MPI_WIN_DISP_UNIT
       
       parameter (MPI_ANY_SOURCE=-1)
@@ -111,7 +108,6 @@
       parameter (MPI_ROOT=-4)
       parameter (MPI_MAX_PROCESSOR_NAME=255)
       parameter (MPI_MAX_ERROR_STRING=255)
-      parameter (MPI_MAX_DATAREP_STRING=128)
       parameter (MPI_UNDEFINED=-32766)
       parameter (MPI_CART=1)
       parameter (MPI_GRAPH=2)
@@ -126,9 +122,9 @@
       parameter (MPI_APPNUM=4)
       parameter (MPI_LASTUSEDCODE=5)
       parameter (MPI_UNIVERSE_SIZE=6)
-      parameter (MPI_WIN_BASE=6)
-      parameter (MPI_WIN_SIZE=7)
-      parameter (MPI_WIN_DISP_UNIT=8)
+      parameter (MPI_WIN_BASE=7)
+      parameter (MPI_WIN_SIZE=8)
+      parameter (MPI_WIN_DISP_UNIT=9)
       parameter (IMPI_CLIENT_SIZE=10)
       parameter (IMPI_CLIENT_COLOR=11)
       parameter (IMPI_HOST_SIZE=12)
@@ -137,7 +133,7 @@
       parameter (MPI_BSEND_OVERHEAD=128)
       parameter (MPI_MAX_INFO_KEY=35)
       parameter (MPI_MAX_INFO_VAL=255)
-      parameter (MPI_MAX_PORT_NAME=35)
+      parameter (MPI_MAX_PORT_NAME=255)
       parameter (MPI_MAX_OBJECT_NAME=63)
       parameter (MPI_ORDER_C=0)
       parameter (MPI_ORDER_FORTRAN=1)
@@ -148,41 +144,53 @@
       parameter (MPI_TYPECLASS_INTEGER=1)
       parameter (MPI_TYPECLASS_REAL=2)
       parameter (MPI_TYPECLASS_COMPLEX=3)
-      parameter (MPI_SEEK_SET=600)
-      parameter (MPI_SEEK_CUR=602)
-      parameter (MPI_SEEK_END=604)
-      parameter (MPI_MODE_CREATE=1)
-      parameter (MPI_MODE_RDONLY=2)
-      parameter (MPI_MODE_WRONLY=4)
-      parameter (MPI_MODE_RDWR=8)
-      parameter (MPI_MODE_DELETE_ON_CLOSE=16)
-      parameter (MPI_MODE_UNIQUE_OPEN=32)
-      parameter (MPI_MODE_EXCL=64)
-      parameter (MPI_MODE_APPEND=128)
-      parameter (MPI_MODE_SEQUENTIAL=256)
-      parameter (MPI_DISPLACEMENT_CURRENT=-54278278)
+      parameter (MPI_MODE_NOCHECK=1)
+      parameter (MPI_MODE_NOPRECEDE=2)
+      parameter (MPI_MODE_NOPUT=4)
+      parameter (MPI_MODE_NOSTORE=8)
+      parameter (MPI_MODE_NOSUCCEED=16)
+      parameter (MPI_LOCK_EXCLUSIVE=1)
+      parameter (MPI_LOCK_SHARED=2)
+
 !     
-!     global variables
-!     
-      double complex MPI_BOTTOM, MPI_IN_PLACE
+!     MPI sentinel values
+!
+!     Several of these types were chosen with care to match specific
+!     overloaded functions in the F90 bindings.  They should also match
+!     the types of their corresponding C variables.  Do not arbitrarily
+!     change their types without also updating the F90 bindings and
+!     their corresponding types in ompi/mpi/f77/constants.h and
+!     ompi/mpi/runtime/ompi_init.c!
+!
+!     MPI_BOTTOM is only used where choice buffers can be used (meaning
+!     that we already have overloaded F90 bindings for all available
+!     types), so any type is fine.
+      integer MPI_BOTTOM
+!     MPI_IN_PLACE has the same rationale as MPI_BOTTOM.
+      integer MPI_IN_PLACE
+!     Making MPI_ARGV_NULL be the same type as the parameter that is
+!     exepected in the F90 binding for MPI_COMM_SPAWN means that we
+!     don't need another binding for MPI_COMM_SPAWN.
       character MPI_ARGV_NULL(1)
-!     MPI_ARGVS_NULL must not be a character array so that we can match
-!     an appropriate F90 MPI bindings interface function; see comments
-!     in mpi-f90-interfaces.h for a full explanation.  It's an analogous
-!     situation for MPI_STATUSES_IGNORE -- we make it of a type that no
-!     one should ever try to pass to (for example) MPI_WAITSOME so that
-!     we get the desired type matching (i.e., we wouldn't want to make
-!     MPI_STATUSED_IGNORE an integer array size of MPI_STATUS_SIZE
-!     because someone may accidentally pass in a 1D integer array of
-!     size MPI_STATUS_SIZE [vs. an array of integer arrays, like they're
-!     supposed to] -- and that's the type of mistake that these
-!     strong-typing bindings are supposed to catch :-) ).
-      integer MPI_ARGVS_NULL
-      double complex MPI_STATUSES_IGNORE
-!     These must be integer arrays so that we can match the proper
-!     prototypes in the F90 MPI bindings.
+!     The array_of_argv parameter in the F90 bindings for
+!     MPI_COMM_SPAWN_MULTIPLE takes a variable number of dimensions
+!     (specified by the "count" parameter), so it's not possible to have
+!     a single variable match all possible values.  Hence, make it an
+!     entirely different type (one that would never likely be used by a
+!     correct program, e.g., double) and have a separate F90 binding for
+!     matching just this type.
+      double precision MPI_ARGVS_NULL
+!     MPI_ERRCODES_IGNORE has similar rationale to MPI_ARGV_NULL.  The
+!     F77 functions are all smart enough to check that the errcodes
+!     parameter is not ERRCODES_IGNORE before assigning values into it
+!     (hence, the fact that this is an array of only 1 element does not
+!     matter -- we'll never overrun it because we never assign values
+!     into it).
       integer MPI_ERRCODES_IGNORE(1)
+!     MPI_STATUS_IGNORE has similar rationale to MPI_ERRCODES_IGNORE.
       integer MPI_STATUS_IGNORE(MPI_STATUS_SIZE)
+!     MPI_STATUSES_IGNORE has similar rationale to MPI_ARGVS_NULL.
+      double precision MPI_STATUSES_IGNORE
       
       common/mpi_fortran_bottom/MPI_BOTTOM
       common/mpi_fortran_in_place/MPI_IN_PLACE
@@ -401,12 +409,17 @@
       integer MPI_CHARACTER, MPI_LOGICAL
       integer MPI_INTEGER, MPI_INTEGER1, MPI_INTEGER2, MPI_INTEGER4
       integer MPI_INTEGER8, MPI_INTEGER16
-      integer MPI_REAL, MPI_REAL4, MPI_REAL8, MPI_REAL16
+      integer MPI_REAL, MPI_REAL2, MPI_REAL4, MPI_REAL8, MPI_REAL16
       integer MPI_DOUBLE_PRECISION
       integer MPI_COMPLEX, MPI_COMPLEX8, MPI_COMPLEX16, MPI_COMPLEX32
       integer MPI_DOUBLE_COMPLEX
       integer MPI_2REAL, MPI_2DOUBLE_PRECISION, MPI_2INTEGER
-      
+      integer MPI_2COMPLEX, MPI_2DOUBLE_COMPLEX
+      integer MPI_LOGICAL1, MPI_LOGICAL2, MPI_LOGICAL4, MPI_LOGICAL8
+
+!
+!     Do NOT change the order of these parameters
+!
       parameter (MPI_BYTE=1)
       parameter (MPI_PACKED=2)
       parameter (MPI_UB=3)
@@ -432,6 +445,13 @@
       parameter (MPI_2REAL=23)
       parameter (MPI_2DOUBLE_PRECISION=24)
       parameter (MPI_2INTEGER=25)
+      parameter (MPI_2COMPLEX=26)
+      parameter (MPI_2DOUBLE_COMPLEX=27)
+      parameter (MPI_REAL2=28)
+      parameter (MPI_LOGICAL1=29)
+      parameter (MPI_LOGICAL2=30)
+      parameter (MPI_LOGICAL4=31)
+      parameter (MPI_LOGICAL8=32)
       
       integer MPI_MAX, MPI_MIN, MPI_SUM, MPI_PROD, MPI_LAND
       integer MPI_BAND, MPI_LOR, MPI_BOR, MPI_LXOR, MPI_BXOR
