@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,10 +24,14 @@
 
 #include "base.h"
 #include "btl_base_error.h"
-#include "opal/util/show_help.h"
-#include "orte/util/sys_info.h"
 
-int mca_btl_base_debug;
+#include "orte/util/show_help.h"
+#include "orte/util/proc_info.h"
+#include "orte/types.h"
+#include "orte/util/name_fns.h"
+#include "orte/runtime/orte_globals.h"
+
+int mca_btl_base_verbose;
 
 int mca_btl_base_err(const char* fmt, ...)
 {
@@ -56,13 +61,15 @@ void mca_btl_base_error_no_nics(const char* transport,
                                 const char* nic_name)
 {
     char *procid;
-    asprintf(&procid, "[%lu,%lu,%lu]", 
-             ORTE_NAME_ARGS(orte_process_info.my_name));
+    if (mca_btl_base_warn_component_unused) {
+        /* print out no-nic warning if user told us to */
+        asprintf(&procid, "%s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
 
-    opal_show_help("help-mpi-btl-base.txt", "btl:no-nics",
-                   true, procid, transport, orte_system_info.nodename,
-                   nic_name);
-    free(procid);
+        orte_show_help("help-mpi-btl-base.txt", "btl:no-nics",
+                       true, procid, transport, orte_process_info.nodename,
+                       nic_name);
+        free(procid);
+    }
 }
 
 
