@@ -15,6 +15,7 @@
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -106,7 +107,11 @@ static void show_init_error(const char *file, int line,
         struct rlimit limit;
         char *str_limit = NULL;
 
+#if HAVE_DECL_RLIMIT_MEMLOCK
         ret = getrlimit(RLIMIT_MEMLOCK, &limit);
+#else
+        ret = -1;
+#endif
         if (0 != ret) {
             asprintf(&str_limit, "Unknown");
         } else if (limit.rlim_cur == RLIM_INFINITY) {
@@ -982,7 +987,8 @@ static int mca_btl_openib_finalize_resources(struct mca_btl_base_module_t* btl)
                         &openib_btl->qps[qp].u.srq_qp.pending_frags[0]);
                 MCA_BTL_OPENIB_CLEAN_PENDING_FRAGS(
                         &openib_btl->qps[qp].u.srq_qp.pending_frags[1]);
-                if (ibv_destroy_srq(openib_btl->qps[qp].u.srq_qp.srq)){
+                if (NULL != openib_btl->qps[qp].u.srq_qp.srq &&
+                        ibv_destroy_srq(openib_btl->qps[qp].u.srq_qp.srq)){
                     BTL_VERBOSE(("Failed to close SRQ %d", qp));
                     rc = OMPI_ERROR;
                 }
