@@ -2,17 +2,18 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /**
@@ -27,13 +28,23 @@
 #include "btl_sm.h"
 
 
-typedef enum {
-    MCA_BTL_SM_FRAG_SEND,
-    MCA_BTL_SM_FRAG_PUT,
-    MCA_BTL_SM_FRAG_GET,
-    MCA_BTL_SM_FRAG_ACK
-} mca_btl_sm_frag_type_t;
+#define MCA_BTL_SM_FRAG_TYPE_MASK ((uintptr_t)0x3)
+#define MCA_BTL_SM_FRAG_SEND ((uintptr_t)0x0)
+#define MCA_BTL_SM_FRAG_ACK ((uintptr_t)0x1)
+#define MCA_BTL_SM_FRAG_PUT ((uintptr_t)0x2)
+#define MCA_BTL_SM_FRAG_GET ((uintptr_t)0x3)
 
+#define MCA_BTL_SM_FRAG_STATUS_MASK ((uintptr_t)0x4)
+
+struct mca_btl_sm_frag_t;
+
+struct mca_btl_sm_hdr_t {
+    struct mca_btl_sm_frag_t *frag;
+    size_t len;
+    int my_smp_rank;
+    mca_btl_base_tag_t tag;
+};
+typedef struct mca_btl_sm_hdr_t mca_btl_sm_hdr_t;
 
 /**
  * shared memory send fragment derived type.
@@ -42,10 +53,9 @@ struct mca_btl_sm_frag_t {
     mca_btl_base_descriptor_t base;
     mca_btl_base_segment_t segment;
     struct mca_btl_base_endpoint_t *endpoint;
-    mca_btl_sm_frag_type_t type;
-    mca_btl_base_tag_t tag;
     size_t size;
-    int rc;
+    /* pointer written to the FIFO, this is the base of the shared memory region */
+    mca_btl_sm_hdr_t *hdr;
     ompi_free_list_t* my_list;
 };
 typedef struct mca_btl_sm_frag_t mca_btl_sm_frag_t;
@@ -56,17 +66,17 @@ OBJ_CLASS_DECLARATION(mca_btl_sm_frag_t);
 OBJ_CLASS_DECLARATION(mca_btl_sm_frag1_t);
 OBJ_CLASS_DECLARATION(mca_btl_sm_frag2_t);
 
-#define MCA_BTL_SM_FRAG_ALLOC1(frag, rc)                                \
+#define MCA_BTL_SM_FRAG_ALLOC_EAGER(frag, rc)                           \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
-    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags1, item, rc);      \
+    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags_eager, item, rc); \
     frag = (mca_btl_sm_frag_t*)item;                                    \
 }
 
-#define MCA_BTL_SM_FRAG_ALLOC2(frag, rc)                                \
+#define MCA_BTL_SM_FRAG_ALLOC_MAX(frag, rc)                             \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
-    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags2, item, rc);      \
+    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags_max, item, rc);   \
     frag = (mca_btl_sm_frag_t*)item;                                    \
 }
 
