@@ -34,7 +34,8 @@ int mca_coll_self_allgatherv_intra(void *sbuf, int scount,
                                    struct ompi_datatype_t *sdtype, 
                                    void * rbuf, int *rcounts, int *disps, 
                                    struct ompi_datatype_t *rdtype, 
-                                   struct ompi_communicator_t *comm)
+                                   struct ompi_communicator_t *comm,
+                                   mca_coll_base_module_t *module)
 {
     if (MPI_IN_PLACE == sbuf) {
         return MPI_SUCCESS;
@@ -46,6 +47,6 @@ int mca_coll_self_allgatherv_intra(void *sbuf, int scount,
             return OMPI_ERROR;
         }
         return ompi_ddt_sndrcv(sbuf, scount, sdtype,
-                               ((char *) rbuf) + disps[0], rcounts[0], rdtype);
+                               ((char *) rbuf) + disps[0] * extent, rcounts[0], rdtype);
     }
 }
