@@ -1,14 +1,15 @@
 /* 
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,11 +25,13 @@
 
 #include "mpi.h"
 #include "opal/mca/mca.h"
-#include "orte/mca/ns/ns_types.h"
+#include "orte/types.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
 
+BEGIN_C_DECLS
+
 /*
  * Horrid debugging macro
  */
@@ -39,16 +42,13 @@
 #define D(foo)
 #endif
 
-#if OMPI_ENABLE_DEBUG
-#include <sched.h>
-#define SPIN sched_yield()
-#else
-#define SPIN continue
-#endif
-
-
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
+#ifdef HAVE_SCHED_YIELD
+#  include <sched.h>
+#  define SPIN sched_yield()
+#elif defined(__WINDOWS__)
+#  define SPIN SwitchToThread()
+#else  /* no switch available */
+#  define SPIN
 #endif
 
     /**
@@ -128,7 +128,7 @@ extern "C" {
      */
     struct mca_coll_sm_component_t {
         /** Base coll component */
-        mca_coll_base_component_1_0_0_t super;
+        mca_coll_base_component_2_0_0_t super;
 
         /** MCA parameter: Priority of this component */
         int sm_priority;
@@ -271,7 +271,7 @@ extern "C" {
      * into the data mpool for this comm's sm collective operations
      * area. 
      */
-    struct mca_coll_base_comm_t {
+    struct mca_coll_sm_comm_t {
         /** If this process is the one that invoked mpool_alloc() for
             the data segment, the return value will be in here.
             Otherwise, it will be NULL (i.e., only the allocating
@@ -325,120 +325,143 @@ extern "C" {
     /**
      * Convenience typedef
      */
-    typedef struct mca_coll_base_comm_t mca_coll_base_comm_t;
-        
+    typedef struct mca_coll_sm_comm_t mca_coll_sm_comm_t;
+
+
+    struct mca_coll_sm_module_t {
+	mca_coll_base_module_t super;
+
+	mca_coll_sm_comm_t *sm_data;
+	mca_coll_base_module_reduce_fn_t previous_reduce;
+	mca_coll_base_module_t *previous_reduce_module;
+    };
+    typedef struct mca_coll_sm_module_t mca_coll_sm_module_t;
+    OBJ_CLASS_DECLARATION(mca_coll_sm_module_t);
+
     
     /**
      * Global component instance
      */
-    extern mca_coll_sm_component_t mca_coll_sm_component;
+    OMPI_MODULE_DECLSPEC extern mca_coll_sm_component_t mca_coll_sm_component;
 
     /*
      * coll module functions
      */
 
     int mca_coll_sm_init_query(bool enable_progress_threads,
-                               bool enable_mpi_threads);
+			       bool enable_mpi_threads);
 
-    const struct mca_coll_base_module_1_0_0_t *
-    mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority,
-                           struct mca_coll_base_comm_t **data);
-
-    int mca_coll_sm_comm_unquery(struct ompi_communicator_t *comm,
-                                 struct mca_coll_base_comm_t *data);
+    mca_coll_base_module_t *
+    mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority);
 
     int mca_coll_sm_bootstrap_finalize(void);
 
     int mca_coll_sm_allgather_intra(void *sbuf, int scount, 
-                                    struct ompi_datatype_t *sdtype, 
-                                    void *rbuf, int rcount, 
-                                    struct ompi_datatype_t *rdtype, 
-                                    struct ompi_communicator_t *comm);
-    int mca_coll_sm_allgatherv_intra(void *sbuf, int scount, 
-                                     struct ompi_datatype_t *sdtype, 
-                                     void * rbuf, int *rcounts, int *disps, 
-                                     struct ompi_datatype_t *rdtype, 
-                                     struct ompi_communicator_t *comm);
-    int mca_coll_sm_allreduce_intra(void *sbuf, void *rbuf, int count, 
-                                    struct ompi_datatype_t *dtype, 
-                                    struct ompi_op_t *op, 
-                                    struct ompi_communicator_t *comm);
-    int mca_coll_sm_alltoall_intra(void *sbuf, int scount, 
-                                   struct ompi_datatype_t *sdtype, 
-                                   void* rbuf, int rcount, 
-                                   struct ompi_datatype_t *rdtype, 
-                                   struct ompi_communicator_t *comm);
-    int mca_coll_sm_alltoallv_intra(void *sbuf, int *scounts, int *sdisps, 
-                                    struct ompi_datatype_t *sdtype, 
-                                    void *rbuf, int *rcounts, int *rdisps, 
-                                    struct ompi_datatype_t *rdtype, 
-                                    struct ompi_communicator_t *comm);
-    int mca_coll_sm_alltoallw_intra(void *sbuf, int *scounts, int *sdisps, 
-                                    struct ompi_datatype_t **sdtypes, 
-                                    void *rbuf, int *rcounts, int *rdisps, 
-                                    struct ompi_datatype_t **rdtypes, 
-                                    struct ompi_communicator_t *comm);
-    int mca_coll_sm_barrier_intra(struct ompi_communicator_t *comm);
-    int mca_coll_sm_bcast_intra(void *buff, int count, 
-                                struct ompi_datatype_t *datatype,
-                                int root, 
-                                struct ompi_communicator_t *comm);
-    int mca_coll_sm_bcast_log_intra(void *buff, int count, 
-                                    struct ompi_datatype_t *datatype, 
-                                    int root, 
-                                    struct ompi_communicator_t *comm);
-    int mca_coll_sm_exscan_intra(void *sbuf, void *rbuf, int count, 
-                                 struct ompi_datatype_t *dtype, 
-                                 struct ompi_op_t *op, 
-                                 struct ompi_communicator_t *comm);
-    int mca_coll_sm_gather_intra(void *sbuf, int scount, 
-                                 struct ompi_datatype_t *sdtype, void *rbuf, 
-                                 int rcount, struct ompi_datatype_t *rdtype, 
-                                 int root, struct ompi_communicator_t *comm);
-    int mca_coll_sm_gatherv_intra(void *sbuf, int scount, 
-                                  struct ompi_datatype_t *sdtype, void *rbuf, 
-                                  int *rcounts, int *disps, 
-                                  struct ompi_datatype_t *rdtype, int root, 
-                                  struct ompi_communicator_t *comm);
-    int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count, 
-                                 struct ompi_datatype_t *dtype, 
-                                 struct ompi_op_t *op, 
-                                 int root,
-                                 struct ompi_communicator_t *comm);
-    int mca_coll_sm_reduce_log_intra(void *sbuf, void* rbuf, int count, 
-                                     struct ompi_datatype_t *dtype, 
-                                     struct ompi_op_t *op, 
-                                     int root, 
-                                     struct ompi_communicator_t *comm);
-    int mca_coll_sm_reduce_scatter_intra(void *sbuf, void *rbuf, 
-                                         int *rcounts, 
-                                         struct ompi_datatype_t *dtype, 
-                                         struct ompi_op_t *op, 
-                                         struct ompi_communicator_t *comm);
-    int mca_coll_sm_scan_intra(void *sbuf, void *rbuf, int count, 
-                               struct ompi_datatype_t *dtype, 
-                               struct ompi_op_t *op, 
-                               struct ompi_communicator_t *comm);
-    int mca_coll_sm_scatter_intra(void *sbuf, int scount, 
-                                  struct ompi_datatype_t *sdtype, void *rbuf, 
-                                  int rcount, struct ompi_datatype_t *rdtype, 
-                                  int root, struct ompi_communicator_t *comm);
-    int mca_coll_sm_scatterv_intra(void *sbuf, int *scounts, int *disps, 
-                                   struct ompi_datatype_t *sdtype, 
-                                   void* rbuf, int rcount, 
-                                   struct ompi_datatype_t *rdtype, int root, 
-                                   struct ompi_communicator_t *comm);
-    
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+				    struct ompi_datatype_t *sdtype, 
+				    void *rbuf, int rcount, 
+				    struct ompi_datatype_t *rdtype, 
+				    struct ompi_communicator_t *comm,
+				    mca_coll_base_module_t *module);
 
+    int mca_coll_sm_allgatherv_intra(void *sbuf, int scount, 
+				     struct ompi_datatype_t *sdtype, 
+				     void * rbuf, int *rcounts, int *disps, 
+				     struct ompi_datatype_t *rdtype, 
+				     struct ompi_communicator_t *comm,
+				     mca_coll_base_module_t *module);
+    int mca_coll_sm_allreduce_intra(void *sbuf, void *rbuf, int count, 
+				    struct ompi_datatype_t *dtype, 
+				    struct ompi_op_t *op, 
+				    struct ompi_communicator_t *comm,
+				    mca_coll_base_module_t *module);
+    int mca_coll_sm_alltoall_intra(void *sbuf, int scount, 
+				   struct ompi_datatype_t *sdtype, 
+				   void* rbuf, int rcount, 
+				   struct ompi_datatype_t *rdtype, 
+				   struct ompi_communicator_t *comm,
+				   mca_coll_base_module_t *module);
+    int mca_coll_sm_alltoallv_intra(void *sbuf, int *scounts, int *sdisps, 
+				    struct ompi_datatype_t *sdtype, 
+				    void *rbuf, int *rcounts, int *rdisps, 
+				    struct ompi_datatype_t *rdtype, 
+				    struct ompi_communicator_t *comm,
+				    mca_coll_base_module_t *module);
+    int mca_coll_sm_alltoallw_intra(void *sbuf, int *scounts, int *sdisps, 
+				    struct ompi_datatype_t **sdtypes, 
+				    void *rbuf, int *rcounts, int *rdisps, 
+				    struct ompi_datatype_t **rdtypes, 
+				    struct ompi_communicator_t *comm,
+				    mca_coll_base_module_t *module);
+    int mca_coll_sm_barrier_intra(struct ompi_communicator_t *comm,
+				  mca_coll_base_module_t *module);
+    int mca_coll_sm_bcast_intra(void *buff, int count, 
+				struct ompi_datatype_t *datatype,
+				int root, 
+				struct ompi_communicator_t *comm,
+				mca_coll_base_module_t *module);
+    int mca_coll_sm_bcast_log_intra(void *buff, int count, 
+				    struct ompi_datatype_t *datatype, 
+				    int root, 
+				    struct ompi_communicator_t *comm, 
+				    mca_coll_base_module_t *module);
+    int mca_coll_sm_exscan_intra(void *sbuf, void *rbuf, int count, 
+				 struct ompi_datatype_t *dtype, 
+				 struct ompi_op_t *op, 
+				 struct ompi_communicator_t *comm, 
+				 mca_coll_base_module_t *module);
+    int mca_coll_sm_gather_intra(void *sbuf, int scount, 
+				 struct ompi_datatype_t *sdtype, void *rbuf, 
+				 int rcount, struct ompi_datatype_t *rdtype, 
+				 int root, struct ompi_communicator_t *comm,
+				 mca_coll_base_module_t *module);
+    int mca_coll_sm_gatherv_intra(void *sbuf, int scount, 
+				  struct ompi_datatype_t *sdtype, void *rbuf, 
+				  int *rcounts, int *disps, 
+				  struct ompi_datatype_t *rdtype, int root, 
+				  struct ompi_communicator_t *comm,
+				  mca_coll_base_module_t *module);
+    int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count, 
+				 struct ompi_datatype_t *dtype, 
+				 struct ompi_op_t *op, 
+				 int root,
+				 struct ompi_communicator_t *comm,
+				 mca_coll_base_module_t *module);
+    int mca_coll_sm_reduce_log_intra(void *sbuf, void* rbuf, int count, 
+				     struct ompi_datatype_t *dtype, 
+				     struct ompi_op_t *op, 
+				     int root, 
+				     struct ompi_communicator_t *comm,
+				     mca_coll_base_module_t *module);
+    int mca_coll_sm_reduce_scatter_intra(void *sbuf, void *rbuf, 
+					 int *rcounts, 
+					 struct ompi_datatype_t *dtype, 
+					 struct ompi_op_t *op, 
+					 struct ompi_communicator_t *comm,
+					 mca_coll_base_module_t *module);
+    int mca_coll_sm_scan_intra(void *sbuf, void *rbuf, int count, 
+			       struct ompi_datatype_t *dtype, 
+			       struct ompi_op_t *op, 
+			       struct ompi_communicator_t *comm,
+			       mca_coll_base_module_t *module);
+    int mca_coll_sm_scatter_intra(void *sbuf, int scount, 
+				  struct ompi_datatype_t *sdtype, void *rbuf, 
+				  int rcount, struct ompi_datatype_t *rdtype, 
+				  int root, struct ompi_communicator_t *comm,
+				  mca_coll_base_module_t *module);
+    int mca_coll_sm_scatterv_intra(void *sbuf, int *scounts, int *disps, 
+				   struct ompi_datatype_t *sdtype, 
+				   void* rbuf, int rcount, 
+				   struct ompi_datatype_t *rdtype, int root, 
+				   struct ompi_communicator_t *comm,
+				   mca_coll_base_module_t *module);
+
+    int mca_coll_sm_ft_event(int state);
+    
 /**
  * Global variables used in the macros (essentially constants, so
  * these are thread safe)
  */
 extern uint32_t mca_coll_sm_iov_size;
-extern int32_t mca_coll_sm_bogus_free_after;
 
 
 /**
@@ -486,7 +509,7 @@ extern int32_t mca_coll_sm_bogus_free_after;
         ((rank) * mca_coll_sm_component.sm_fragment_size); \
     (max_data) = (iov).iov_len = mca_coll_sm_component.sm_fragment_size; \
     ompi_convertor_pack(&(convertor), &(iov), &mca_coll_sm_iov_size, \
-                        &(max_data), &mca_coll_sm_bogus_free_after)
+                        &(max_data) )
 
 /**
  * Macro to copy a single segment out from a shared segment to a user
@@ -496,7 +519,7 @@ extern int32_t mca_coll_sm_bogus_free_after;
     (iov).iov_base = (((char*) (index)->mcbmi_data) + \
                       ((src_rank) * mca_coll_sm_component.sm_fragment_size)); \
     ompi_convertor_unpack(&(convertor), &(iov), &mca_coll_sm_iov_size, \
-                          &(max_data), &mca_coll_sm_bogus_free_after)
+                          &(max_data) )
 
 /**
  * Macro to memcpy a fragment between one shared segment and another
@@ -565,4 +588,6 @@ extern int32_t mca_coll_sm_bogus_free_after;
         *ptr = 0; \
     } while (0)
 
+END_C_DECLS
+
 #endif /* MCA_COLL_SM_EXPORT_H */
