@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -27,7 +28,7 @@
 #include "ompi/constants.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
-#include "opal/util/show_help.h"
+#include "orte/util/show_help.h"
 #include "coll_sm.h"
 
 
@@ -53,44 +54,34 @@ static int sm_close(void);
 
 mca_coll_sm_component_t mca_coll_sm_component = {
 
-    /* First, fill in the super (mca_coll_base_component_1_0_0_t) */
+    /* First, fill in the super */
 
     {
         /* First, the mca_component_t struct containing meta
            information about the component itself */
         
         {
-            /* Indicate that we are a coll v1.0.0 component (which
-               also implies a specific MCA version) */
-
-            MCA_COLL_BASE_VERSION_1_0_0,
+            MCA_COLL_BASE_VERSION_2_0_0,
 
             /* Component name and version */
-
             "sm",
             OMPI_MAJOR_VERSION,
             OMPI_MINOR_VERSION,
             OMPI_RELEASE_VERSION,
 
             /* Component open and close functions */
-
             sm_open,
             sm_close,
         },
-        
-        /* Next the MCA v1.0.0 component meta data */
-
         {
-            /* Whether the component is checkpointable or not */
-            
-            true
+            /* The component is not checkpoint ready */
+            MCA_BASE_METADATA_PARAM_NONE
         },
 
         /* Initialization / querying functions */
         
         mca_coll_sm_init_query,
         mca_coll_sm_comm_query,
-        mca_coll_sm_comm_unquery,
     },
 
     /* sm-component specifc information */
@@ -225,13 +216,13 @@ static int sm_open(void)
                            cs->sm_tree_degree,
                            &cs->sm_tree_degree);
     if (cs->sm_tree_degree > cs->sm_control_size) {
-        opal_show_help("help-coll-sm.txt", 
+        orte_show_help("help-coll-sm.txt", 
                        "tree-degree-larger-than-control", true,
                        cs->sm_tree_degree, cs->sm_control_size);
         cs->sm_tree_degree = cs->sm_control_size;
     }
     if (cs->sm_tree_degree > 255) {
-        opal_show_help("help-coll-sm.txt", 
+        orte_show_help("help-coll-sm.txt", 
                        "tree-degree-larger-than-255", true,
                        cs->sm_tree_degree);
         cs->sm_tree_degree = 255;
@@ -247,7 +238,7 @@ static int sm_open(void)
     mca_base_param_reg_int(c, "shared_mem_used_bootstrap",
                            "Amount of shared memory used in the shared memory bootstrap area (in bytes)",
                            false, true,
-                           size1, NULL);
+                           (int)size1, NULL);
 
     /* Calculate how much space we need in the data mpool.  This
        formula taken directly from coll_sm_module.c. */
@@ -265,7 +256,7 @@ static int sm_open(void)
     mca_base_param_reg_int(c, "shared_mem_used_data",
                            "Amount of shared memory used in the shared memory data area for info_num_procs processes (in bytes)",
                            false, true,
-                           size2, NULL);
+                           (int)size2, NULL);
 
     return OMPI_SUCCESS;
 }
@@ -285,3 +276,45 @@ static int sm_close(void)
 
     return OMPI_SUCCESS;
 }
+
+
+static void
+mca_coll_sm_module_construct(mca_coll_sm_module_t *module)
+{
+    module->sm_data = NULL;
+    module->previous_reduce_module = NULL;
+}
+
+static void
+mca_coll_sm_module_destruct(mca_coll_sm_module_t *module)
+{
+    mca_coll_sm_comm_t *data;
+
+    /* Free the space in the data mpool and the data hanging off the
+       communicator */
+
+    data = module->sm_data;
+    if (NULL != data) {
+        /* If this was the process that allocated the space in the
+           data mpool, then this is the process that frees it */
+
+        if (NULL != data->mcb_data_mpool_malloc_addr) {
+            mca_coll_sm_component.sm_data_mpool->mpool_free(mca_coll_sm_component.sm_data_mpool,
+                                                       data->mcb_data_mpool_malloc_addr, NULL);
+        }
+
+        /* Now free the data hanging off the communicator */
+
+        free(data);
+    }
+
+    if (NULL != module->previous_reduce_module) {
+        OBJ_RELEASE(module->previous_reduce_module);
+    }
+}
+
+
+OBJ_CLASS_INSTANCE(mca_coll_sm_module_t,
+                   mca_coll_base_module_t,
+                   mca_coll_sm_module_construct,
+                   mca_coll_sm_module_destruct);
