@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -42,7 +43,11 @@
 #include "mpi.h"
 #include "opal/mca/maffinity/maffinity.h"
 #include "opal/mca/maffinity/base/base.h"
-#include "orte/mca/ns/ns.h"
+#include "opal/util/os_path.h"
+
+#include "orte/util/proc_info.h"
+#include "orte/util/name_fns.h"
+
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/base.h"
@@ -56,18 +61,17 @@
  * Global variables
  */
 uint32_t mca_coll_sm_iov_size = 1;
-int32_t mca_coll_sm_bogus_free_after = 0;
 
 
 /*
  * Local functions
  */
-static const struct mca_coll_base_module_1_0_0_t *
-    sm_module_init(struct ompi_communicator_t *comm);
-static int sm_module_finalize(struct ompi_communicator_t *comm);
-static bool have_local_peers(ompi_proc_t **procs, size_t size);
+static int sm_module_enable(mca_coll_base_module_t *module,
+                          struct ompi_communicator_t *comm);
+static bool have_local_peers(ompi_group_t *group, size_t size);
 static int bootstrap_init(void);
-static int bootstrap_comm(ompi_communicator_t *comm);
+static int bootstrap_comm(ompi_communicator_t *comm,
+                          mca_coll_sm_module_t *module);
 
 
 /*
@@ -76,36 +80,6 @@ static int bootstrap_comm(ompi_communicator_t *comm);
 static bool bootstrap_inited = false;
 
 
-/*
- * Linear set of collective algorithms
- */
-static const mca_coll_base_module_1_0_0_t module = {
-
-    /* Initialization / finalization functions */
-
-    sm_module_init,
-    sm_module_finalize,
-
-    /* Collective function pointers */
-
-    NULL,
-    NULL,
-    mca_coll_sm_allreduce_intra,
-    NULL,
-    NULL,
-    NULL,
-    mca_coll_sm_barrier_intra,
-    mca_coll_sm_bcast_intra,
-    NULL,
-    NULL,
-    NULL,
-    mca_coll_sm_reduce_intra,
-    NULL,
-    NULL,
-    NULL,
-    NULL
-};
-
 
 /*
  * Initial query function that is invoked during MPI_INIT, allowing
@@ -155,10 +129,11 @@ int mca_coll_sm_init_query(bool enable_progress_threads,
  * Look at the communicator and decide which set of functions and
  * priority we want to return.
  */
-const mca_coll_base_module_1_0_0_t *
-mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority,
-                       struct mca_coll_base_comm_t **data)
+mca_coll_base_module_t *
+mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
 {
+    mca_coll_sm_module_t *sm_module;
+
     /* See if someone has previously lazily initialized and failed */
 
     if (mca_coll_sm_component.sm_component_setup &&
@@ -171,8 +146,7 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority,
        are not on this node, then we don't want to run */
 
     if (OMPI_COMM_IS_INTER(comm) || 1 == ompi_comm_size(comm) ||
-        !have_local_peers(comm->c_local_group->grp_proc_pointers,
-                          ompi_comm_size(comm))) {
+        !have_local_peers(comm->c_local_group, ompi_comm_size(comm))) {
 	return NULL;
     }
 
@@ -187,41 +161,65 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority,
 
     /* Get our priority */
 
+
+    /* Get the priority level attached to this module. If priority is less
+     * than or equal to 0, then the module is unavailable. */
     *priority = mca_coll_sm_component.sm_priority;
+    if (0 >= mca_coll_sm_component.sm_priority) {
+	return NULL;
+    }
+
     
     /* All is good -- return a module */
+    sm_module = OBJ_NEW(mca_coll_sm_module_t);
 
-    return &module;
-}
+    sm_module->super.coll_module_enable = sm_module_enable;
+    sm_module->super.ft_event        = mca_coll_sm_ft_event;
+    sm_module->super.coll_allgather  = NULL;
+    sm_module->super.coll_allgatherv = NULL;
+    sm_module->super.coll_allreduce  = mca_coll_sm_allreduce_intra;
+    sm_module->super.coll_alltoall   = NULL;
+    sm_module->super.coll_alltoallv  = NULL;
+    sm_module->super.coll_alltoallw  = NULL;
+    sm_module->super.coll_barrier    = mca_coll_sm_barrier_intra;
+    sm_module->super.coll_bcast      = mca_coll_sm_bcast_intra;
+    sm_module->super.coll_exscan     = NULL;
+    sm_module->super.coll_gather     = NULL;
+    sm_module->super.coll_gatherv    = NULL;
+    sm_module->super.coll_reduce     = mca_coll_sm_reduce_intra;
+    sm_module->super.coll_reduce_scatter = NULL;
+    sm_module->super.coll_scan       = NULL;
+    sm_module->super.coll_scatter    = NULL;
+    sm_module->super.coll_scatterv   = NULL;
 
-
-/* 
- * Unquery the coll on comm
- */
-int mca_coll_sm_comm_unquery(struct ompi_communicator_t *comm,
-                             struct mca_coll_base_comm_t *data)
-{
-    return OMPI_SUCCESS;
+    return &(sm_module->super);
 }
 
 
 /*
  * Init module on the communicator
  */
-static const struct mca_coll_base_module_1_0_0_t *
-sm_module_init(struct ompi_communicator_t *comm)
+static int
+sm_module_enable(mca_coll_base_module_t *module,
+                 struct ompi_communicator_t *comm)
 {
-    int i, j, root;
+    int i, j, root, ret;
     int rank = ompi_comm_rank(comm);
     int size = ompi_comm_size(comm);
-    mca_coll_base_comm_t *data;
+    mca_coll_sm_module_t *sm_module = (mca_coll_sm_module_t*) module;
+    mca_coll_sm_comm_t *data = NULL;
     size_t control_size, frag_size;
     mca_coll_sm_component_t *c = &mca_coll_sm_component;
     opal_maffinity_base_segment_t *maffinity;
     int parent, min_child, max_child, num_children;
-    char *base;
+    char *base = NULL;
     const int num_barrier_buffers = 2;
 
+    if (NULL == comm->c_coll.coll_reduce ||
+        NULL == comm->c_coll.coll_reduce_module) {
+        return OMPI_ERROR;
+    }
+
     /* Once-per-component setup.  This may happen at any time --
        during MPI_INIT or later.  So we must protect this with locks
        to ensure that only one thread in the process actually does
@@ -231,10 +229,10 @@ sm_module_init(struct ompi_communicator_t *comm)
     if (!mca_coll_sm_component.sm_component_setup) {
         mca_coll_sm_component.sm_component_setup = true;
 
-        if (OMPI_SUCCESS != bootstrap_init()) {
+        if (OMPI_SUCCESS != (ret = bootstrap_init())) {
             mca_coll_sm_component.sm_component_setup_success = false;
             opal_atomic_unlock(&mca_coll_sm_component.sm_component_setup_lock);
-            return NULL;
+            return ret;
         }
 
         /* Can we get an mpool allocation?  See if there was one created
@@ -250,7 +248,7 @@ sm_module_init(struct ompi_communicator_t *comm)
                 mca_coll_sm_bootstrap_finalize();
                 mca_coll_sm_component.sm_component_setup_success = false;
                 opal_atomic_unlock(&mca_coll_sm_component.sm_component_setup_lock);
-                return NULL;
+                return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
             }
             mca_coll_sm_component.sm_data_mpool_created = true;
         } else {
@@ -264,16 +262,16 @@ sm_module_init(struct ompi_communicator_t *comm)
        we got in here */
 
     if (!mca_coll_sm_component.sm_component_setup_success) {
-        return NULL;
+        return OMPI_ERROR;
     }
 
     /* Get some space to setup memory affinity (just easier to try to
        alloc here to handle the error case) */
 
-    maffinity = malloc(sizeof(opal_maffinity_base_segment_t) * 
-                      c->sm_comm_num_segments * 3);
+    maffinity = (opal_maffinity_base_segment_t*)malloc(sizeof(opal_maffinity_base_segment_t) * 
+                                                       c->sm_comm_num_segments * 3);
     if (NULL == maffinity) {
-        return NULL;
+        return OMPI_ERROR;
     }
 
     /* Allocate data to hang off the communicator.  The memory we
@@ -289,8 +287,8 @@ sm_module_init(struct ompi_communicator_t *comm)
           mca_coll_sm_tree_node_t
     */
 
-    comm->c_coll_selected_data = data =
-        malloc(sizeof(mca_coll_base_comm_t) + 
+    sm_module->sm_data = data = (mca_coll_sm_comm_t*)
+        malloc(sizeof(mca_coll_sm_comm_t) + 
                (c->sm_comm_num_segments * 
                 sizeof(mca_coll_base_mpool_index_t)) +
                (size * 
@@ -298,7 +296,8 @@ sm_module_init(struct ompi_communicator_t *comm)
                  (sizeof(mca_coll_sm_tree_node_t*) * c->sm_tree_degree))));
 
     if (NULL == data) {
-        return NULL;
+        free(maffinity);
+        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
     }
     data->mcb_data_mpool_malloc_addr = NULL;
 
@@ -360,10 +359,11 @@ sm_module_init(struct ompi_communicator_t *comm)
        mpool that has been allocated among my peers for this
        communicator. */
 
-    if (OMPI_SUCCESS != bootstrap_comm(comm)) {
+    if (OMPI_SUCCESS != (ret = bootstrap_comm(comm, sm_module))) {
         free(data);
-        comm->c_coll_selected_data = NULL;
-        return NULL;
+        free(maffinity);
+        sm_module->sm_data = NULL;
+        return ret;
     }
 
     /* Once the communicator is bootstrapped, setup the pointers into
@@ -474,47 +474,25 @@ sm_module_init(struct ompi_communicator_t *comm)
                c->sm_control_size);
     }
 
+    /* Save previous component's reduce information */
+    sm_module->previous_reduce = comm->c_coll.coll_reduce;
+    sm_module->previous_reduce_module = comm->c_coll.coll_reduce_module;
+    OBJ_RETAIN(sm_module->previous_reduce_module);
+
     /* All done */
 
-    return &module;
-}
-
-
-/*
- * Finalize module on the communicator
- */
-static int sm_module_finalize(struct ompi_communicator_t *comm)
-{
-    mca_coll_base_comm_t *data;
-
-    /* Free the space in the data mpool and the data hanging off the
-       communicator */
-
-    data = comm->c_coll_selected_data;
-    if (NULL != data) {
-        /* If this was the process that allocated the space in the
-           data mpool, then this is the process that frees it */
-
-        if (NULL != data->mcb_data_mpool_malloc_addr) {
-            mca_coll_sm_component.sm_data_mpool->mpool_free(mca_coll_sm_component.sm_data_mpool,
-                                                       data->mcb_data_mpool_malloc_addr, NULL);
-        }
-
-        /* Now free the data hanging off the communicator */
-
-        free(data);
-    }
-
     return OMPI_SUCCESS;
 }
 
 
-static bool have_local_peers(ompi_proc_t **procs, size_t size)
+static bool have_local_peers(ompi_group_t *group, size_t size)
 {
     size_t i;
+    ompi_proc_t *proc;
 
     for (i = 0; i < size; ++i) {
-        if (0 == (procs[i]->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
+        proc = ompi_group_peer_lookup(group,i);
+        if (0 == (proc->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
             return false;
         }
     }
@@ -541,8 +519,8 @@ static int bootstrap_init(void)
         return OMPI_ERROR;
     }
     orte_proc_info();
-    asprintf(&fullpath, "%s/%s", orte_process_info.job_session_dir,
-             mca_coll_sm_component.sm_bootstrap_filename);
+    fullpath = opal_os_path( false, orte_process_info.job_session_dir,
+                             mca_coll_sm_component.sm_bootstrap_filename, NULL );
     if (NULL == fullpath) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -596,20 +574,22 @@ static int bootstrap_init(void)
 }
 
 
-static int bootstrap_comm(ompi_communicator_t *comm)
+static int bootstrap_comm(ompi_communicator_t *comm,
+                          mca_coll_sm_module_t *module)
 {
     int i, empty_index, err;
     bool found;
     mca_coll_sm_component_t *c = &mca_coll_sm_component;
     mca_coll_sm_bootstrap_header_extension_t *bshe;
     mca_coll_sm_bootstrap_comm_setup_t *bscs;
-    mca_coll_base_comm_t *data = comm->c_coll_selected_data;
+    mca_coll_sm_comm_t *data = module->sm_data;
     int comm_size = ompi_comm_size(comm);
     int num_segments = c->sm_comm_num_segments;
     int num_in_use = c->sm_comm_num_in_use_flags;
     int frag_size = c->sm_fragment_size;
     int control_size = c->sm_control_size;
     orte_process_name_t *rank0;
+    ompi_proc_t * proc;
 
     /* Is our CID in the CIDs array?  If not, loop until we can find
        an open slot in the array to use in the bootstrap to setup our
@@ -619,14 +599,16 @@ static int bootstrap_comm(ompi_communicator_t *comm)
         c->sm_bootstrap_meta->map_seg;
     bscs = bshe->smbhe_segments;
     opal_atomic_lock(&bshe->super.seg_lock);
-    rank0 = &(comm->c_local_group->grp_proc_pointers[0]->proc_name);
+    proc = ompi_group_peer_lookup(comm->c_local_group,0);
+    rank0 = &(proc->proc_name);
+
     while (1) {
         opal_atomic_wmb();
         found = false;
         empty_index = -1;
         for (i = 0; i < mca_coll_sm_component.sm_bootstrap_num_segments; ++i) {
             if (comm->c_contextid == bshe->smbhe_keys[i].mcsbck_cid &&
-                0 == orte_ns.compare(ORTE_NS_CMP_ALL,
+                OPAL_EQUAL == orte_util_compare_name_fields(ORTE_NS_CMP_ALL,
                                      rank0,
                                      &bshe->smbhe_keys[i].mcsbck_rank0_name)) {
                 found = true;
@@ -686,8 +668,8 @@ static int bootstrap_comm(ompi_communicator_t *comm)
                 (num_segments * (comm_size * frag_size));
 
             data->mcb_data_mpool_malloc_addr = tmp =
-                c->sm_data_mpool->mpool_alloc(c->sm_data_mpool, size, 
-                                              c->sm_control_size, 0, NULL);
+                (char*)c->sm_data_mpool->mpool_alloc(c->sm_data_mpool, size, 
+                                                     c->sm_control_size, 0, NULL);
             if (NULL == tmp) {
                 /* Cleanup before returning; allow other processes in
                    this communicator to learn of the failure.  Note
@@ -722,9 +704,7 @@ static int bootstrap_comm(ompi_communicator_t *comm)
 
         else {
             opal_atomic_unlock(&bshe->super.seg_lock);
-#ifdef HAVE_SCHED_YIELD
-            sched_yield();
-#endif
+            SPIN;
             opal_atomic_lock(&bshe->super.seg_lock);
         }
     }
@@ -740,7 +720,7 @@ static int bootstrap_comm(ompi_communicator_t *comm)
 
     else {
         err = OMPI_SUCCESS;
-        data->mcb_mpool_base = c->sm_data_mpool->mpool_base(c->sm_data_mpool);
+        data->mcb_mpool_base = (unsigned char*)c->sm_data_mpool->mpool_base(c->sm_data_mpool);
         data->mcb_mpool_offset = bscs[i].smbcs_data_mpool_offset;
         data->mcb_mpool_area = data->mcb_mpool_base + data->mcb_mpool_offset;
         data->mcb_operation_count = 0;
@@ -764,7 +744,6 @@ static int bootstrap_comm(ompi_communicator_t *comm)
     return err;
 }
 
-
 /*
  * This function is not static and has a prefix-rule-enabled name
  * because it gets called from the component (but may also be called
@@ -787,7 +766,30 @@ int mca_coll_sm_bootstrap_finalize(void)
            anything in here -- all data structures are referencing
            within the bootstrap area, so the one top-level unmap does
            it all) */
-        mca_common_sm_mmap_fini( meta );
+        if (OMPI_SUCCESS == mca_common_sm_mmap_fini( meta )) {
+            unlink(meta->map_path);
+        }
+        OBJ_RELEASE(meta);
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int mca_coll_sm_ft_event(int state) {
+    if(OPAL_CRS_CHECKPOINT == state) {
+        ;
+    }
+    else if(OPAL_CRS_CONTINUE == state) {
+        ;
+    }
+    else if(OPAL_CRS_RESTART == state) {
+        ;
+    }
+    else if(OPAL_CRS_TERM == state ) {
+        ;
+    }
+    else {
+        ;
     }
 
     return OMPI_SUCCESS;
