@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -25,6 +26,7 @@
 #include "ompi/mca/io/io.h"
 #include "io_romio.h"
 
+#define ROMIO_VERSION_STRING "from MPICH2 v1.0.7 with additional compilation/bug patches from romio-maint@mcs.anl.gov"
 
 /*
  * Private functions
@@ -33,7 +35,7 @@ static int open_component(void);
 static int close_component(void);
 static int init_query(bool enable_progress_threads,
                       bool enable_mpi_threads);
-static const struct mca_io_base_module_1_0_0_t *
+static const struct mca_io_base_module_2_0_0_t *
   file_query(struct ompi_file_t *file, 
              struct mca_io_base_file_t **private_data,
              int *priority);
@@ -73,20 +75,18 @@ opal_list_t mca_io_romio_pending_requests;
 
 
 /*
- * Public string showing the coll ompi_demo component version number
+ * Public string showing this component's version number
  */
 const char *mca_io_romio_component_version_string =
-  "OMPI/MPI ROMIO io MCA component version " OMPI_VERSION;
+"OMPI/MPI ROMIO io MCA component version " OMPI_VERSION ", " ROMIO_VERSION_STRING;
 
 
-mca_io_base_component_1_0_0_t mca_io_romio_component = {
+mca_io_base_component_2_0_0_t mca_io_romio_component = {
     /* First, the mca_base_component_t struct containing meta information
        about the component itself */
 
     {
-        /* Indicate that we are a io v1.0.0 component (which also implies a
-           specific MCA version) */
-        MCA_IO_BASE_VERSION_1_0_0,
+        MCA_IO_BASE_VERSION_2_0_0,
         "romio",
         OMPI_MAJOR_VERSION,
         OMPI_MINOR_VERSION,
@@ -94,12 +94,9 @@ mca_io_base_component_1_0_0_t mca_io_romio_component = {
         open_component,
         close_component,
     },
-
-    /* Next the MCA v1.0.0 component meta data */
-
     {
-        /* Whether the component is checkpointable or not */
-        false
+        /* The component is checkpoint ready */
+        MCA_BASE_METADATA_PARAM_CHECKPOINT
     },
 
     /* Additional number of bytes required for this component's
@@ -141,10 +138,21 @@ static int open_component(void)
                                "Delete priority of the io romio component",
                                false, false, 10, NULL);
 
-    mca_base_param_reg_int(&mca_io_romio_component.io_version,
-                           "enable_parallel_optimizations",
-                           "Enable set of Open MPI-added options to improve collective file i/o performance",
-                           false, false, 0, NULL);
+    mca_base_param_reg_string(&mca_io_romio_component.io_version,
+                              "version", 
+                              "Version of ROMIO",
+                              false, true, ROMIO_VERSION_STRING, NULL);
+
+    mca_base_param_reg_string(&mca_io_romio_component.io_version,
+                              "user_configure_params", 
+                              "User-specified command line parameters passed to ROMIO's configure script",
+                              false, true, 
+                              MCA_io_romio_USER_CONFIGURE_FLAGS, NULL);
+    mca_base_param_reg_string(&mca_io_romio_component.io_version,
+                              "complete_configure_params", 
+                              "Complete set of command line parameters passed to ROMIO's configure script",
+                              false, true, 
+                              MCA_io_romio_COMPLETE_CONFIGURE_FLAGS, NULL);
 
     /* Create the mutex */
     OBJ_CONSTRUCT(&mca_io_romio_mutex, opal_mutex_t);
@@ -183,7 +191,7 @@ static int init_query(bool enable_progress_threads,
 }
 
 
-static const struct mca_io_base_module_1_0_0_t *
+static const struct mca_io_base_module_2_0_0_t *
 file_query(struct ompi_file_t *file, 
            struct mca_io_base_file_t **private_data,
            int *priority)
@@ -262,7 +270,7 @@ static int progress()
 {
     opal_list_item_t *item, *next;
     int ret, flag, count;
-    ROMIO_PREFIX(MPIO_Request) romio_rq;
+    MPI_Request romio_rq;
     mca_io_base_request_t *ioreq;
 
     /* Troll through all pending requests and try to progress them.
@@ -277,15 +285,15 @@ static int progress()
 
         ioreq = (mca_io_base_request_t*) item;
         romio_rq = ((mca_io_romio_request_t *) item)->romio_rq;
-        ret = ROMIO_PREFIX(MPIO_Test)(&romio_rq, &flag, 
-                                      &(((ompi_request_t *) item)->req_status));
+        ret = MPI_Test(&romio_rq, &flag, 
+                       &(((ompi_request_t *) item)->req_status));
         if ((0 != ret) || (0 != flag)) {
             ioreq->super.req_status.MPI_ERROR = ret;
             ++count;
             /* we're done, so remove us from the pending list */
             opal_list_remove_item(&mca_io_romio_pending_requests, item);
             /* mark as complete (and make sure to wake up any waiters */
-            ompi_request_complete((ompi_request_t*) item);
+            ompi_request_complete((ompi_request_t*) item, true);
             mca_io_base_request_progress_del();
             /* if the request has been freed already, the user isn't
              * going to call test or wait on us, so we need to do it
