@@ -30,11 +30,58 @@
 #include "mtl_portals.h"
 #include "mtl_portals_endpoint.h"
 #include "mtl_portals_request.h"
+#include "mtl_portals_recv.h"
 #include "mtl_portals_recv_short.h"
 
 #define CHECK_MATCH(incoming_bits, match_bits, ignore_bits)     \
     (((incoming_bits ^ match_bits) & ~ignore_bits) == 0)
 
+static int
+ompi_mtl_portals_recv_progress(ptl_event_t *, struct ompi_mtl_portals_request_t* );
+
+static int
+ompi_mtl_portals_rendezvous_get(ptl_event_t *ev,
+                                ompi_mtl_portals_request_t *ptl_request)
+{
+    ptl_md_t md;
+    ptl_handle_md_t md_h;
+    int ret;
+
+    md.start = ev->md.start;
+    md.length = ev->md.length;
+    md.threshold = 2; /* send and reply */
+    md.options = PTL_MD_EVENT_START_DISABLE;
+    md.user_ptr = ptl_request;
+    md.eq_handle = ompi_mtl_portals.ptl_eq_h;
+
+    ret = PtlMDBind(ompi_mtl_portals.ptl_ni_h, md, PTL_UNLINK, &md_h);
+    if (PTL_OK != ret) {
+        opal_output(fileno(stderr)," Error returned from PtlMDBind().  Error code - %d \n",ret);
+        abort();
+    }
+
+    ptl_request->is_complete = false;
+    ptl_request->event_callback = ompi_mtl_portals_recv_progress;
+
+    ret = PtlGet(md_h,
+                 ev->initiator,
+                 OMPI_MTL_PORTALS_READ_TABLE_ID,
+                 0,
+                 ev->hdr_data,
+                 0);
+    if (PTL_OK != ret) {
+        opal_output(fileno(stderr)," Error returned from PtlGet.  Error code - %d \n",ret);
+        abort();
+    }
+
+    /* stay here until the reply comes */
+    while (ptl_request->is_complete == false) {
+        ompi_mtl_portals_progress();
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /* called when a receive should be progressed */
 static int
 ompi_mtl_portals_recv_progress(ptl_event_t *ev,
@@ -44,9 +91,19 @@ ompi_mtl_portals_recv_progress(ptl_event_t *ev,
 
     switch (ev->type) {
     case PTL_EVENT_PUT_END:
+        if (PTL_IS_LONG_MSG(ev->match_bits) && (ompi_mtl_portals.ptl_use_rendezvous == true)) {
+	    /* get the data */
+	    ret = ompi_mtl_portals_rendezvous_get(ev, ptl_request);
+            if ( OMPI_SUCCESS != ret ) {
+                opal_output(fileno(stderr)," Error returned from ompi_mtl_portals_rendezvous_get().  Error code - %d \n",ret);
+                return ret;
+            }
+	}
+
         /* make sure the data is in the right place */
-        ompi_mtl_datatype_unpack(ptl_request->convertor,
-                                 ev->md.start, ev->mlength);
+        ret = ompi_mtl_datatype_unpack(ptl_request->convertor,
+                                       ev->md.start, ev->mlength);
+        if (OMPI_SUCCESS != ret) return ret;
 
         /* set the status */
         ptl_request->super.ompi_req->req_status.MPI_SOURCE =
@@ -63,17 +120,13 @@ ompi_mtl_portals_recv_progress(ptl_event_t *ev,
                              "recv complete: 0x%016llx\n", ev->match_bits));
         
         ptl_request->super.completion_callback(&ptl_request->super);
+        ptl_request->is_complete = true;
         break;
 
     case PTL_EVENT_REPLY_END:
         /* make sure the data is in the right place */
-        ompi_mtl_datatype_unpack(ptl_request->convertor,
-                                 ev->md.start, ev->mlength);
-
-        ret=PtlMDUnlink(ev->md_handle);
-        if( ret !=PTL_OK) {
-            return ompi_common_portals_error_ptl_to_ompi(ret);
-        }
+        ret = ompi_mtl_datatype_unpack(ptl_request->convertor, ev->md.start, ev->mlength);
+        if (OMPI_SUCCESS != ret) return ret;
 
         /* set the status - most of this filled in right after issuing
            the PtlGet*/
@@ -84,6 +137,7 @@ ompi_mtl_portals_recv_progress(ptl_event_t *ev,
                              "recv complete: 0x%016llx\n", ev->match_bits));
         
         ptl_request->super.completion_callback(&ptl_request->super);
+        ptl_request->is_complete = true;
         break;
 
     default:
@@ -127,8 +181,9 @@ ompi_mtl_portals_get_data(ompi_mtl_portals_event_t *recv_event,
 
         /* pull out the data */
         if (iov.iov_len > 0) {
-            ompi_convertor_unpack(convertor, &iov, &iov_count,
-                                  &max_data );
+            ret = ompi_convertor_unpack(convertor, &iov, &iov_count,
+                                        &max_data );
+            if (0 > ret) return ret;
         }
 
         /* if synchronous, return an ack */
@@ -185,6 +240,7 @@ ompi_mtl_portals_get_data(ompi_mtl_portals_event_t *recv_event,
                              recv_event->ev.match_bits));
         
         ptl_request->super.completion_callback(&ptl_request->super);
+        ptl_request->is_complete = true;
 
     } else {
         ret = ompi_mtl_datatype_recv_buf(convertor, &md.start, &buflen,
@@ -266,7 +322,7 @@ ompi_mtl_portals_wait_for_put_end(ptl_seq_t link)
 
     /* wait for a PUT_END event that matches the message we're looking for */
     while (true) {
-        ret = PtlEQWait(ompi_mtl_portals.ptl_unexpected_recv_eq_h,&ev);
+        ret = PtlEQWait(ompi_mtl_portals.ptl_unex_eq_h,&ev);
         if (PTL_OK == ret) {
             if (PTL_EVENT_PUT_START == ev.type) {
                 ompi_free_list_item_t *item;
@@ -304,16 +360,17 @@ ompi_mtl_portals_wait_for_put_end(ptl_seq_t link)
 }
 
 
-static ompi_mtl_portals_event_t*
+ompi_mtl_portals_event_t*
 ompi_mtl_portals_search_unex_events(ptl_match_bits_t match_bits,
-                                    ptl_match_bits_t ignore_bits)
+                                    ptl_match_bits_t ignore_bits,
+                                    bool             probe)
 {
     ptl_event_t ev;
     int ret;
 
     /* check to see if there are any events in the unexpected event queue */ 
     while (true) {
-        ret = PtlEQGet(ompi_mtl_portals.ptl_unexpected_recv_eq_h,&ev);
+        ret = PtlEQGet(ompi_mtl_portals.ptl_unex_eq_h,&ev);
         if (PTL_OK == ret) {
             if (PTL_EVENT_PUT_START == ev.type) {
                 ompi_free_list_item_t *item;
@@ -331,7 +388,13 @@ ompi_mtl_portals_search_unex_events(ptl_match_bits_t match_bits,
                }
                 if (CHECK_MATCH(recv_event->ev.match_bits, match_bits, ignore_bits)) {
                     /* the one we want */
-                    ompi_mtl_portals_wait_for_put_end(recv_event->ev.link);
+                    if (probe == false) {
+                        ompi_mtl_portals_wait_for_put_end(recv_event->ev.link);
+                    } else {
+                        /* just probing, so add it to the unex list */
+                        opal_list_append(&(ompi_mtl_portals.unexpected_messages),
+                                         (opal_list_item_t*) recv_event);
+                    }
                     return recv_event; 
                 } else {
                     /* not the one we want, so add it to the unex list */
@@ -357,9 +420,10 @@ ompi_mtl_portals_search_unex_events(ptl_match_bits_t match_bits,
 }
 
 
-static ompi_mtl_portals_event_t*
-ompi_mtl_portals_search_unex_q( ptl_match_bits_t match_bits,
-                                ptl_match_bits_t ignore_bits )
+ompi_mtl_portals_event_t*
+ompi_mtl_portals_search_unex_q(ptl_match_bits_t match_bits,
+                               ptl_match_bits_t ignore_bits,
+                               bool             probe)
 {
     opal_list_item_t *list_item;
     ompi_mtl_portals_event_t *recv_event = NULL;
@@ -372,12 +436,14 @@ ompi_mtl_portals_search_unex_q( ptl_match_bits_t match_bits,
         recv_event = (ompi_mtl_portals_event_t*) list_item;
         if (CHECK_MATCH(recv_event->ev.match_bits, match_bits, ignore_bits)) {
             /* we have a match... */
-            if ( false == recv_event->is_complete) {
-                /* wait for put end event */
-                ompi_mtl_portals_wait_for_put_end(recv_event->ev.link);
+            if ( probe == false ) {
+                if ( false == recv_event->is_complete) {
+                    /* wait for put end event */
+                    ompi_mtl_portals_wait_for_put_end(recv_event->ev.link);
+                }
+                opal_list_remove_item(&(ompi_mtl_portals.unexpected_messages),
+                                      list_item);
             }
-            opal_list_remove_item(&(ompi_mtl_portals.unexpected_messages),
-                                  list_item);
             return recv_event;
         }
         list_item = next_item;
@@ -407,6 +473,7 @@ ompi_mtl_portals_irecv(struct mca_mtl_base_module_t* mtl,
         (ompi_mtl_portals_request_t*) mtl_request;
     ompi_mtl_portals_event_t *recv_event = NULL;
     size_t buflen;
+    bool   did_once = false;
 
     ptl_request->convertor = convertor;
 
@@ -427,20 +494,20 @@ ompi_mtl_portals_irecv(struct mca_mtl_base_module_t* mtl,
                          match_bits, ignore_bits));
 
     /* first, check the queue of processed unexpected messages */
-    recv_event = ompi_mtl_portals_search_unex_q(match_bits, ignore_bits);
+    recv_event = ompi_mtl_portals_search_unex_q(match_bits, ignore_bits, false);
     if (NULL != recv_event) {
         /* found it */
-        ompi_mtl_portals_get_data(recv_event, convertor, ptl_request);
+        ret = ompi_mtl_portals_get_data(recv_event, convertor, ptl_request);
         OMPI_FREE_LIST_RETURN(&ompi_mtl_portals.event_fl,
                               (ompi_free_list_item_t*)recv_event);
         goto cleanup;
     } else {
 restart_search:
         /* check unexpected events */
-        recv_event = ompi_mtl_portals_search_unex_events(match_bits, ignore_bits);
+        recv_event = ompi_mtl_portals_search_unex_events(match_bits, ignore_bits, false);
         if (NULL != recv_event) {
             /* found it */
-            ompi_mtl_portals_get_data(recv_event, convertor, ptl_request);
+            ret = ompi_mtl_portals_get_data(recv_event, convertor, ptl_request);
             OMPI_FREE_LIST_RETURN(&ompi_mtl_portals.event_fl,
                                   (ompi_free_list_item_t*)recv_event);
             goto cleanup;
@@ -448,43 +515,36 @@ restart_search:
     }
 
     /* didn't find it, now post the receive */
-    ret = ompi_mtl_datatype_recv_buf(convertor, &md.start, &buflen,
-                                     &ptl_request->free_after);
-    md.length = buflen;
-
-    /* create ME entry */
-    ret = PtlMEInsert(ompi_mtl_portals.ptl_match_ins_me_h,
-                remote_proc,
-                match_bits,
-                ignore_bits,
-                PTL_UNLINK,
-                PTL_INS_BEFORE,
-                &me_h);
-    if( ret !=PTL_OK) {
-        return ompi_common_portals_error_ptl_to_ompi(ret);
+    if ( false == did_once ) {
+	ret = ompi_mtl_datatype_recv_buf(convertor, &md.start, &buflen,
+					 &ptl_request->free_after);
+        if (OMPI_SUCCESS != ret) {
+            opal_output(fileno(stderr)," Error returned from ompi_mtl_datatype_recv_buf().  Error code - %d \n",ret);
+            abort();
+        }
+	did_once = true;
     }
 
-    /* associate a memory descriptor with the Match list Entry */
-    md.threshold = 0;
-    md.options = PTL_MD_OP_PUT | PTL_MD_TRUNCATE | PTL_MD_EVENT_START_DISABLE;
-    md.user_ptr = ptl_request;
-    md.eq_handle = ompi_mtl_portals.ptl_eq_h;
-    ret=PtlMDAttach(me_h, md, PTL_UNLINK, &md_h);
-    if( ret !=PTL_OK) {
-        return ompi_common_portals_error_ptl_to_ompi(ret);
-    }
-
-    /* now try to make active */
+    md.length    = buflen;
     md.threshold = 1;
+    md.options   = PTL_MD_OP_PUT | PTL_MD_TRUNCATE | PTL_MD_EVENT_START_DISABLE;
+    md.user_ptr  = ptl_request;
+    md.eq_handle = ompi_mtl_portals.ptl_eq_h;
 
-    /* enable the memory descritor, if the ptl_unexpected_recv_eq_h
-     *   queue is empty */
-    ret = PtlMDUpdate(md_h, NULL, &md,
-                      ompi_mtl_portals.ptl_unexpected_recv_eq_h);
+    ret = PtlMEMDPost(ompi_mtl_portals.ptl_ni_h,
+		      ompi_mtl_portals.ptl_unex_long_me_h,
+		      remote_proc,
+		      match_bits,
+		      ignore_bits,
+		      PTL_UNLINK,
+		      PTL_INS_BEFORE,
+		      md,
+		      PTL_UNLINK,
+		      &me_h,
+		      &md_h,
+		      ompi_mtl_portals.ptl_unex_eq_h);
     if (ret == PTL_MD_NO_UPDATE) {
         /* a message has arrived since we searched - look again */
-        PtlMDUnlink(md_h);
-        if (ptl_request->free_after) { free(md.start); }
         goto restart_search;
     } else if( PTL_OK != ret ) {
         return ompi_common_portals_error_ptl_to_ompi(ret);
@@ -492,8 +552,14 @@ restart_search:
 
     ptl_request->event_callback = ompi_mtl_portals_recv_progress;
 
+    return OMPI_SUCCESS; 
+
  cleanup:
 
-    return OMPI_SUCCESS;
+    if ((did_once == true) && (ptl_request->free_after)) {
+        free(md.start);
+    }
+
+    return ret;
 }
 
