@@ -28,6 +28,7 @@
 #include "ompi/proc/proc.h"
 #include "ompi/op/op.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/memchecker.h"
 
 
 struct ompi_osc_pt2pt_replyreq_t {
@@ -126,6 +127,10 @@ ompi_osc_pt2pt_replyreq_init_origin(ompi_osc_pt2pt_replyreq_t *replyreq,
 static inline int
 ompi_osc_pt2pt_replyreq_free(ompi_osc_pt2pt_replyreq_t *replyreq)
 {
+    MEMCHECKER(
+        memchecker_convertor_call(&opal_memchecker_base_mem_defined,
+                                  &replyreq->rep_target_convertor);
+    );
     ompi_convertor_cleanup(&replyreq->rep_target_convertor);
 
     OBJ_RELEASE(replyreq->rep_target_datatype);
