@@ -9,6 +9,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -33,6 +34,7 @@
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
 #include "ompi/datatype/datatype.h"
+#include "ompi/op/op.h"
 #include "ompi/memchecker.h"
 
 static inline int32_t
@@ -910,7 +912,7 @@ ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_osc_rdma_longreq_t *longreq)
     /* lock the window for accumulates */
     OPAL_THREAD_LOCK(&longreq->req_module->m_acc_lock);
 
-    if (longreq->req_op == &ompi_mpi_op_replace) {
+    if (longreq->req_op == &ompi_mpi_op_replace.op) {
         ompi_convertor_t convertor;
         struct iovec iov;
         uint32_t iov_count = 1;
@@ -1000,7 +1002,7 @@ ompi_osc_rdma_sendreq_recv_accum(ompi_osc_rdma_module_t *module,
         /* lock the window for accumulates */
         OPAL_THREAD_LOCK(&module->m_acc_lock);
 
-        if (op == &ompi_mpi_op_replace) {
+        if (op == &ompi_mpi_op_replace.op) {
             ompi_convertor_t convertor;
             struct iovec iov;
             uint32_t iov_count = 1;
