@@ -2,10 +2,10 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -24,19 +24,19 @@ static void mca_pml_base_send_request_construct(mca_pml_base_send_request_t* req
 static void mca_pml_base_send_request_destruct(mca_pml_base_send_request_t* req);
 
 
-opal_class_t mca_pml_base_send_request_t_class = { 
-    "mca_pml_base_send_request_t", 
-    OBJ_CLASS(mca_pml_base_request_t),
-    (opal_construct_t) mca_pml_base_send_request_construct, 
-    (opal_destruct_t) mca_pml_base_send_request_destruct 
-};
+OBJ_CLASS_INSTANCE(
+    mca_pml_base_send_request_t,
+    mca_pml_base_request_t,
+    mca_pml_base_send_request_construct,
+    mca_pml_base_send_request_destruct
+);
 
 
 static void mca_pml_base_send_request_construct(mca_pml_base_send_request_t* request)
 {
     /* no need to reinit for every send -- never changes */
     request->req_base.req_type = MCA_PML_REQUEST_SEND;
-    OBJ_CONSTRUCT(&request->req_convertor, ompi_convertor_t);
+    OBJ_CONSTRUCT(&request->req_base.req_convertor, ompi_convertor_t);
 }
 
 static void mca_pml_base_send_request_destruct(mca_pml_base_send_request_t* req)
