@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -22,6 +22,7 @@
 #include "pml_ob1.h"
 #include "pml_ob1_recvreq.h"
 #include "pml_ob1_sendreq.h"
+#include "ompi/memchecker.h"
 
 
 int mca_pml_ob1_start(size_t count, ompi_request_t** requests)
@@ -32,11 +33,12 @@ int mca_pml_ob1_start(size_t count, ompi_request_t** requests)
 
     for(i=0; i<count; i++) {
         mca_pml_base_request_t *pml_request = (mca_pml_base_request_t*)requests[i];
+        if(NULL == pml_request) {
+            continue;
+        }
         if (OMPI_REQUEST_PML != requests[i]->req_type) {
             continue;
         }
-        if(NULL == pml_request)
-            continue;
 
         /* If the persistent request is currently active - obtain the
          * request lock and verify the status is incomplete. if the
@@ -113,10 +115,8 @@ int mca_pml_ob1_start(size_t count, ompi_request_t** requests)
                 if( reuse_old_request && (sendreq->req_send.req_bytes_packed != 0) ) {
                     size_t offset = 0;
                     /**
-                     * Reset the convertor in case we're dealing with the original request,
-                     * which when completed do not reset the convertor. For the other case
-                     * (freshly allocated request) the convertor_set_position is optimized,
-                     * it will not do anything is it's not required.
+                     * Reset the convertor in case we're dealing with the original
+                     * request, which when completed do not reset the convertor.
                      */
                     ompi_convertor_set_position( &sendreq->req_send.req_base.req_convertor,
                                                  &offset );
