@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -34,6 +35,10 @@ static const char FUNC_NAME[] = "MPI_File_set_errhandler";
 
 int MPI_File_set_errhandler( MPI_File file, MPI_Errhandler errhandler) 
 {
+    MPI_Errhandler tmp;
+
+    OPAL_CR_NOOP_PROGRESS();
+
     /* Error checking */
 
     if (MPI_PARAM_CHECK) {
@@ -54,14 +59,21 @@ int MPI_File_set_errhandler( MPI_File file, MPI_Errhandler errhandler)
         }
     }
 
-    /* Ditch the old errhandler, and decrement its refcount */
-
-    OBJ_RELEASE(file->error_handler);
-
-    /* We have a valid comm and errhandler, so increment its refcount */
+    /* Ditch the old errhandler, and decrement its refcount.  On 64
+       bits environments we have to make sure the reading of the
+       error_handler became atomic. */
+    do {
+        tmp = file->error_handler;
+    } while (!OPAL_ATOMIC_CMPSET(&(file->error_handler), tmp, tmp));
+    OBJ_RELEASE(tmp);
 
+    /* Now set the new errhandler and increase its refcount */
+    do {
+        file->error_handler = errhandler;
+    } while (!OPAL_ATOMIC_CMPSET(&(errhandler), file->error_handler, 
+                                 file->error_handler));
     file->error_handler = errhandler;
-    OBJ_RETAIN(file->error_handler);
+    OBJ_RETAIN(errhandler);
 
     /* All done */
 
