@@ -1,15 +1,17 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008      University of Houston.  All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +23,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/datatype/datatype.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Gather = PMPI_Gather
@@ -38,6 +41,47 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
                int root, MPI_Comm comm) 
 {
     int err;
+
+    MEMCHECKER(
+        int rank;
+        ptrdiff_t ext;
+
+        rank = ompi_comm_rank(comm);
+        ompi_ddt_type_extent(recvtype, &ext);
+
+        memchecker_comm(comm);
+        if(OMPI_COMM_IS_INTRA(comm)) {
+            if(ompi_comm_rank(comm) == root) {
+                /* check whether root's send buffer is defined. */
+                if (MPI_IN_PLACE == sendbuf) {
+                  memchecker_call(&opal_memchecker_base_isdefined, 
+                                  (char *)(recvbuf)+rank*ext, 
+                                  recvcount, recvtype);
+                } else {
+                    memchecker_datatype(sendtype);
+                    memchecker_call(&opal_memchecker_base_isdefined, sendbuf, sendcount, sendtype);
+                }
+                
+                memchecker_datatype(recvtype);
+                /* check whether root's receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+            } else {
+                memchecker_datatype(sendtype);
+                /* check whether send buffer is defined on other processes. */
+                memchecker_call(&opal_memchecker_base_isdefined, sendbuf, sendcount, sendtype);
+            }
+        } else {
+            if (MPI_ROOT == root) {
+                memchecker_datatype(recvtype);
+                /* check whether root's receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+            } else if (MPI_PROC_NULL != root) {
+                memchecker_datatype(sendtype);
+                /* check whether send buffer is defined. */
+                memchecker_call(&opal_memchecker_base_isdefined, sendbuf, sendcount, sendtype);
+            }
+        }
+    );
     
     if (MPI_PARAM_CHECK) {
         err = MPI_SUCCESS;
@@ -45,7 +89,7 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
         if (ompi_comm_invalid(comm)) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, 
                                           FUNC_NAME);
-	} else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == sendbuf) ||
+        } else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == sendbuf) ||
                    (ompi_comm_rank(comm) == root && MPI_IN_PLACE == recvbuf)) {
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ARG, FUNC_NAME);
         }
@@ -72,7 +116,7 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
              them out into individual tests. */
 
           if (ompi_comm_rank(comm) == root) {
-            if (recvtype == MPI_DATATYPE_NULL) {
+            if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
               return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TYPE, FUNC_NAME); 
             }
 
@@ -106,7 +150,7 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
               return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_COUNT, FUNC_NAME);
             }
 
-            if (MPI_DATATYPE_NULL == recvtype) {
+            if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
               return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TYPE, FUNC_NAME); 
             }
           }
@@ -115,17 +159,21 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
 
     /* Do we need to do anything? */
 
-    if ((0 == sendcount &&
+    if ((0 == sendcount && MPI_ROOT != root && 
          (ompi_comm_rank(comm) != root ||
           (ompi_comm_rank(comm) == root && MPI_IN_PLACE != sendbuf))) ||
         (ompi_comm_rank(comm) == root && MPI_IN_PLACE == sendbuf && 
-         0 == recvcount)) {
+         0 == recvcount) || 
+	(0 == recvcount && (MPI_ROOT == root || MPI_PROC_NULL == root))) {
         return MPI_SUCCESS;
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Invoke the coll component to perform the back-end operation */
 	
     err = comm->c_coll.coll_gather(sendbuf, sendcount, sendtype, recvbuf,
-                                   recvcount, recvtype, root, comm);
+                                   recvcount, recvtype, root, comm,
+                                   comm->c_coll.coll_gather_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
