@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -37,6 +38,8 @@ MPI_Request MPI_Request_f2c(MPI_Fint request)
 {
     int request_index = OMPI_FINT_2_INT(request);
 
+    OPAL_CR_NOOP_PROGRESS();
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
     }
@@ -47,10 +50,10 @@ MPI_Request MPI_Request_f2c(MPI_Fint request)
     
     if (request_index < 0 || 
         request_index >= 
-        ompi_pointer_array_get_size(&ompi_request_f_to_c_table)) {
+        opal_pointer_array_get_size(&ompi_request_f_to_c_table)) {
         return NULL;
     }
 
-    return (MPI_Request)ompi_pointer_array_get_item(&ompi_request_f_to_c_table,
+    return (MPI_Request)opal_pointer_array_get_item(&ompi_request_f_to_c_table,
                                                     request_index);
 }
