@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -22,6 +22,7 @@
 #include "ompi/datatype/convertor.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/proc/proc.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Sendrecv_replace = PMPI_Sendrecv_replace
@@ -41,6 +42,12 @@ int MPI_Sendrecv_replace(void * buf, int count, MPI_Datatype datatype,
 {
     int rc = MPI_SUCCESS;
 
+    MEMCHECKER(
+        memchecker_datatype(datatype);
+        memchecker_call(&opal_memchecker_base_isdefined, buf, count, datatype);
+        memchecker_comm(comm);
+    );
+
     if ( MPI_PARAM_CHECK ) {
         rc = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -59,10 +66,15 @@ int MPI_Sendrecv_replace(void * buf, int count, MPI_Datatype datatype,
         }
         OMPI_ERRHANDLER_CHECK(rc, comm, rc, FUNC_NAME);
     }
- 
+
+    OPAL_CR_ENTER_LIBRARY();
+
     /* simple case */
     if ( source == MPI_PROC_NULL || dest == MPI_PROC_NULL || count == 0 ) {
-        return MPI_Sendrecv(buf,count,datatype,dest,sendtag,buf,count,datatype,source,recvtag,comm,status);
+        rc = MPI_Sendrecv(buf,count,datatype,dest,sendtag,buf,count,datatype,source,recvtag,comm,status);
+
+        OPAL_CR_EXIT_LIBRARY();
+        return rc;
     } else {
 
         ompi_convertor_t convertor;
@@ -119,6 +131,8 @@ int MPI_Sendrecv_replace(void * buf, int count, MPI_Datatype datatype,
             MPI_Free_mem(iov.iov_base);
         }
         OBJ_DESTRUCT(&convertor);
+
+        OPAL_CR_EXIT_LIBRARY();
         return MPI_SUCCESS;
     }
 }
