@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -24,6 +24,7 @@
 #include "ompi/info/info.h"
 #include "ompi/win/win.h"
 #include "ompi/attribute/attribute.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Win_create = PMPI_Win_create
@@ -40,7 +41,10 @@ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit,
                    MPI_Info info, MPI_Comm comm, MPI_Win *win) 
 {
     int ret = MPI_SUCCESS;
-
+    
+    MEMCHECKER(
+        memchecker_comm(comm);
+    );
     /* argument checking */
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -55,6 +59,10 @@ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit,
 
         } else if (NULL == win) {
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_WIN, FUNC_NAME);
+        } else if ( size < 0 ) {
+            return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_SIZE, FUNC_NAME);            
+        } else if ( disp_unit <= 0 ) {
+            return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_DISP, FUNC_NAME);
         }
     }
 
@@ -63,13 +71,17 @@ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit,
         return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_COMM, FUNC_NAME);
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* create window and return */
-    ret = ompi_win_create(base, size, disp_unit, comm,
+    ret = ompi_win_create(base, (size_t)size, disp_unit, comm,
                           info, win);
     if (OMPI_SUCCESS != ret) {
         *win = MPI_WIN_NULL;
+        OPAL_CR_EXIT_LIBRARY();
         return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_WIN, FUNC_NAME);
     }
 
+    OPAL_CR_EXIT_LIBRARY();
     return MPI_SUCCESS;
 }
