@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -34,10 +35,14 @@ static const char FUNC_NAME[] = "MPI_Win_set_errhandler";
 
 int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler) 
 {
+    MPI_Errhandler tmp;
+
+    OPAL_CR_NOOP_PROGRESS();
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
-        if (ompi_win_invalid(win)) {
+        if (ompi_win_invalid(win) && win != MPI_WIN_NULL) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
         } else if (NULL == errhandler ||
                    MPI_ERRHANDLER_NULL == errhandler ||
@@ -47,12 +52,21 @@ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler)
         }
     }
 
-    /* Ditch the old errhandler, and decrement its refcount */
-    OBJ_RELEASE(win->error_handler);
+    /* Ditch the old errhandler, and decrement its refcount.  On 64
+       bits environments we have to make sure the reading of the
+       error_handler became atomic. */
+    do {
+        tmp = win->error_handler;
+    } while (!OPAL_ATOMIC_CMPSET(&(win->error_handler), tmp, tmp));
+    OBJ_RELEASE(tmp);
 
-    /* We have a valid comm and errhandler, so increment its refcount */
+    /* Now set the new errhandler and increase its refcount */
+    do {
+        win->error_handler = errhandler;
+    } while (!OPAL_ATOMIC_CMPSET(&(errhandler), win->error_handler, 
+                                 win->error_handler));
     win->error_handler = errhandler;
-    OBJ_RETAIN(win->error_handler);
+    OBJ_RETAIN(errhandler);
 
     /* All done */
     return MPI_SUCCESS;
