@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Voltaire All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,9 +22,16 @@
 
 #include "opal_config.h"
 #include "opal/class/opal_list.h"
-#include "opal/sys/atomic.h"
 
-/* Atomic Last In First Out lists. If we are in a multi-threaded environment then the
+#if OMPI_HAVE_THREAD_SUPPORT
+#include "opal/sys/atomic.h"
+#endif  /* OMPI_HAVE_THREAD_SUPPORT */
+
+#if defined(c_plusplus) || defined(__cplusplus)
+extern "C" {
+#endif
+
+    /* Atomic Last In First Out lists. If we are in a multi-threaded environment then the
  * atomicity is insured via the compare-and-swap operation, if not we simply do a read
  * and/or a write.
  *
@@ -41,7 +49,7 @@ struct opal_atomic_lifo_t
 
 typedef struct opal_atomic_lifo_t opal_atomic_lifo_t;
 
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(opal_atomic_lifo_t);
+OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_atomic_lifo_t);
 
 /* The ghost pointer will never change. The head will change via an atomic
  * compare-and-swap. On most architectures the reading of a pointer is an
@@ -64,8 +72,10 @@ static inline opal_list_item_t* opal_atomic_lifo_push( opal_atomic_lifo_t* lifo,
         item->opal_list_next = lifo->opal_lifo_head;
         if( opal_atomic_cmpset_ptr( &(lifo->opal_lifo_head),
                                     (void*)item->opal_list_next,
-                                    item ) )
+                                    item ) ) {
+            opal_atomic_cmpset_32((volatile int32_t*)&item->item_free, 1, 0);
             return (opal_list_item_t*)item->opal_list_next;
+        }
         /* DO some kind of pause to release the bus */
     } while( 1 );
 #else
@@ -82,14 +92,17 @@ static inline opal_list_item_t* opal_atomic_lifo_pop( opal_atomic_lifo_t* lifo )
 {
     opal_list_item_t* item;
 #if OMPI_HAVE_THREAD_SUPPORT
-    do {
-        item = lifo->opal_lifo_head;
+    while((item = lifo->opal_lifo_head) != &(lifo->opal_lifo_ghost))
+    {
+        if(!opal_atomic_cmpset_32((volatile int32_t*)&item->item_free, 0, 1))
+            continue;
         if( opal_atomic_cmpset_ptr( &(lifo->opal_lifo_head),
                                     item,
                                     (void*)item->opal_list_next ) )
             break;
+        opal_atomic_cmpset_32((volatile int32_t*)&item->item_free, 1, 0);
         /* Do some kind of pause to release the bus */
-    } while( 1 );
+    } 
 #else
     item = lifo->opal_lifo_head;
     lifo->opal_lifo_head = (opal_list_item_t*)item->opal_list_next;
@@ -99,5 +112,9 @@ static inline opal_list_item_t* opal_atomic_lifo_pop( opal_atomic_lifo_t* lifo )
     return item;
 }
 
+#if defined(c_plusplus) || defined(__cplusplus)
+}
+#endif
+
 #endif  /* OPAL_ATOMIC_LIFO_H_HAS_BEEN_INCLUDED */
 
