@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserverd.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -82,11 +83,11 @@ static inline int opal_atomic_cmpset_32( volatile int32_t *addr,
                                         int32_t oldval, int32_t newval)
 {
    unsigned char ret;
-   __asm__ __volatile (
+   __asm__ __volatile__ (
                        SMPLOCK "cmpxchgl %1,%2   \n\t"
                                "sete     %0      \n\t"
                        : "=qm" (ret)
-                       : "q"(newval), "m"(*((volatile long*)addr)), "a"(oldval)
+                       : "q"(newval), "m"(*addr), "a"(oldval)
                        : "memory");
 
    return (int)ret;
@@ -103,7 +104,7 @@ static inline int opal_atomic_cmpset_64( volatile int64_t *addr,
                                          int64_t oldval, int64_t newval)
 {
    unsigned char ret;
-   __asm__ __volatile (
+   __asm__ __volatile__ (
                        SMPLOCK "cmpxchgq %1,%2   \n\t"
                                "sete     %0      \n\t"
                        : "=qm" (ret)
