@@ -2,13 +2,15 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,12 +24,20 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <unistd.h>
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 
 #if OMPI_WANT_LIBLTDL
 #include "opal/libltdl/ltdl.h"
 #endif
 
 #include "opal/util/output.h"
+#include "opal/util/show_help.h"
 #include "opal/class/opal_list.h"
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
@@ -35,8 +45,9 @@
 #include "opal/constants.h"
 
 
+#if OMPI_WANT_LIBLTDL
 /*
- * Private types
+ * Private types; only necessary when we're dlopening components.
  */
 typedef enum component_status {
   UNVISITED,
@@ -50,10 +61,10 @@ typedef enum component_status {
 struct component_file_item_t {
   opal_list_item_t super;
 
-  char type[MCA_BASE_MAX_TYPE_NAME_LEN];
-  char name[MCA_BASE_MAX_COMPONENT_NAME_LEN];
-  char basename[OMPI_PATH_MAX];
-  char filename[OMPI_PATH_MAX];
+  char type[MCA_BASE_MAX_TYPE_NAME_LEN + 1];
+  char name[MCA_BASE_MAX_COMPONENT_NAME_LEN + 1];
+  char basename[OMPI_PATH_MAX + 1];
+  char filename[OMPI_PATH_MAX + 1];
   component_status_t status;
 };
 typedef struct component_file_item_t component_file_item_t;
@@ -70,18 +81,24 @@ typedef struct dependency_item_t dependency_item_t;
 static OBJ_CLASS_INSTANCE(dependency_item_t, opal_list_item_t, NULL, NULL);
 
 struct ltfn_data_holder_t {
-  char type[MCA_BASE_MAX_TYPE_NAME_LEN];
-  char name[MCA_BASE_MAX_COMPONENT_NAME_LEN];
+  char type[MCA_BASE_MAX_TYPE_NAME_LEN + 1];
+  char name[MCA_BASE_MAX_COMPONENT_NAME_LEN + 1];
 };
 typedef struct ltfn_data_holder_t ltfn_data_holder_t;
 
+#if OPAL_HAVE_LTDL_ADVISE
+extern lt_dladvise opal_mca_dladvise;
+#endif
+#endif /* OMPI_WANT_LIBLTDL */
+
 
 #if OMPI_WANT_LIBLTDL
 /*
  * Private functions
  */
 static void find_dyn_components(const char *path, const char *type, 
-                             const char *name, opal_list_t *found_components);
+                                const char **name, bool include_mode,
+                                opal_list_t *found_components);
 static int save_filename(const char *filename, lt_ptr data);
 static int open_component(component_file_item_t *target_file, 
                        opal_list_t *found_components);
@@ -93,7 +110,6 @@ static int check_dependency(char *line, component_file_item_t *target_file,
                             opal_list_t *found_components);
 static void free_dependency_list(opal_list_t *dependencies);
 
-
 /*
  * Private variables
  */
@@ -103,6 +119,11 @@ static const char component_template[] = "mca_%s_";
 static opal_list_t found_files;
 #endif /* OMPI_WANT_LIBLTDL */
 
+static bool use_component(const bool include_mode,
+                          const char **requested_component_names,
+                          const char *component_name);
+
+
 /*
  * Function to find as many components of a given type as possible.  This
  * includes statically-linked in components as well as opening up a
@@ -113,45 +134,77 @@ static opal_list_t found_files;
  * available components.
  */
 int mca_base_component_find(const char *directory, const char *type, 
-                         const mca_base_component_t *static_components[], 
+                            const mca_base_component_t *static_components[], 
+                            char **requested_component_names,
+                            bool include_mode,
                             opal_list_t *found_components,
                             bool open_dso_components)
 {
-  int i;
-  mca_base_component_list_item_t *cli;
+    int i;
+    opal_list_item_t *item;
+    mca_base_component_list_item_t *cli;
 
-  /* Find all the components that were statically linked in */
-
-  OBJ_CONSTRUCT(found_components, opal_list_t);
-  for (i = 0; NULL != static_components[i]; ++i) {
-    cli = OBJ_NEW(mca_base_component_list_item_t);
-    if (NULL == cli) {
-      return OPAL_ERR_OUT_OF_RESOURCE;
+    /* Find all the components that were statically linked in */
+    OBJ_CONSTRUCT(found_components, opal_list_t);
+    for (i = 0; NULL != static_components[i]; ++i) {
+        if ( use_component(include_mode,
+                           (const char**)requested_component_names,
+                           static_components[i]->mca_component_name) ) {
+            cli = OBJ_NEW(mca_base_component_list_item_t);
+            if (NULL == cli) {
+                return OPAL_ERR_OUT_OF_RESOURCE;
+            }
+            cli->cli_component = static_components[i];
+            opal_list_append(found_components, (opal_list_item_t *) cli);
+        }
     }
-    cli->cli_component = static_components[i];
-    opal_list_append(found_components, (opal_list_item_t *) cli);
-  }
 
 #if OMPI_WANT_LIBLTDL
-  /* Find any available dynamic components in the specified directory */
-  if (open_dso_components) {
-      int param, param_disable_dlopen;
-      param = mca_base_param_find("mca", NULL, "component_disable_dlopen");
-      mca_base_param_lookup_int(param, &param_disable_dlopen);
+    /* Find any available dynamic components in the specified directory */
+    if (open_dso_components) {
+        int param, param_disable_dlopen;
+        param = mca_base_param_find("mca", NULL, "component_disable_dlopen");
+        mca_base_param_lookup_int(param, &param_disable_dlopen);
 
-      if (0 == param_disable_dlopen) {
-          find_dyn_components(directory, type, NULL, found_components);
-      }
-  } else {
-    opal_output_verbose(40, 0, 
-                        "mca: base: component_find: dso loading for %s MCA components disabled", 
-                        type);
-  }
+        if (0 == param_disable_dlopen) {
+            find_dyn_components(directory, type,
+                                (const char**)requested_component_names,
+                                include_mode, found_components); 
+        }
+    } else {
+        opal_output_verbose(40, 0, 
+                            "mca: base: component_find: dso loading for %s MCA components disabled", 
+                            type);
+    }
 #endif
 
-  /* All done */
+    /* Ensure that *all* requested components exist.  Print a warning
+       and abort if they do not. */
+    for (i = 0; include_mode && NULL != requested_component_names && 
+             NULL != requested_component_names[i]; ++i) {
+        for (item = opal_list_get_first(found_components);
+             opal_list_get_end(found_components) != item; 
+             item = opal_list_get_next(item)) {
+            cli = (mca_base_component_list_item_t*) item;
+            if (0 == strcmp(requested_component_names[i], 
+                            cli->cli_component->mca_component_name)) {
+                break;
+            }
+        }
 
-  return OPAL_SUCCESS;
+        if (opal_list_get_end(found_components) == item) {
+            char h[MAXHOSTNAMELEN];
+            gethostname(h, sizeof(h));
+            opal_show_help("help-mca-base.txt", 
+                           "find-available:not-valid", true,
+                           h, type, requested_component_names[i]);
+            return OPAL_ERR_NOT_FOUND;
+        }
+    }
+
+    /* All done */
+
+    return OPAL_SUCCESS;
 }
 
 #if OMPI_WANT_LIBLTDL
@@ -168,97 +221,89 @@ int mca_base_component_find(const char *directory, const char *type,
  * finally opened in recursive dependency traversals.
  */
 static void find_dyn_components(const char *path, const char *type_name, 
-                                const char *name,
+                                const char **name, bool include_mode,
                                 opal_list_t *found_components)
 {
-  ltfn_data_holder_t params;
-  char *path_to_use, *dir, *end, *param;
-  component_file_item_t *file;
-  opal_list_item_t *cur;
+    ltfn_data_holder_t params;
+    char *path_to_use, *dir, *end;
+    component_file_item_t *file;
+    opal_list_item_t *cur;
 
-  strcpy(params.type, type_name);
+    strncpy(params.type, type_name, MCA_BASE_MAX_TYPE_NAME_LEN);
+    params.type[MCA_BASE_MAX_TYPE_NAME_LEN] = '\0';
 
-  if (NULL == name) {
     params.name[0] = '\0';
-    opal_output_verbose(40, 0, "mca: base: component_find: looking for all dynamic %s MCA components", 
-                       type_name, NULL);
-  } else {
-    strcpy(params.name, name);
-    opal_output_verbose(40, 0,
-                       "mca: base: component_find: looking for dynamic %s MCA component named \"%s\"",
-                       type_name, name, NULL);
-  }
-
-  /* If path is NULL, iterate over the set of directories specified by
-     the MCA param mca_base_component_path.  If path is not NULL, then
-     use that as the path. */
-
-  param = NULL;
-  if (NULL == path) {
-    mca_base_param_lookup_string(mca_base_param_component_path, &param);
-    if (NULL == param) {
-      /* If there's no path, then there's nothing to search -- we're
-         done */
-      return;
-    } else {
-      path_to_use = strdup(param);
-    }
-  } else {
-    path_to_use = strdup(path);
-  }
-
-  /* Iterate over all the files in the directories in the path and
-     make a master array of all the matching filenames that we
-     find. */
-
-  OBJ_CONSTRUCT(&found_files, opal_list_t);
-  dir = path_to_use;
-  if (NULL != dir) {
-    do {
-      end = strchr(dir, OPAL_ENV_SEP);
-      if (NULL != end) {
-        *end = '\0';
-      }
-      if (0 != lt_dlforeachfile(dir, save_filename, &params)) {
-        break;
-      }
-      dir = end + 1;
-    } while (NULL != end);
-  }
-
-  /* Iterate through all the filenames that we found.  Since one
-     component may [try to] call another to be loaded, only try to load
-     the UNVISITED files.  Also, ignore the return code -- basically,
-     give every file one chance to try to load.  If they load, great.
-     If not, great. */
-
-  for (cur = opal_list_get_first(&found_files); 
-       opal_list_get_end(&found_files) != cur;
-       cur = opal_list_get_next(cur)) {
-    file = (component_file_item_t *) cur;
-    if (UNVISITED == file->status) {
-      open_component(file, found_components);
-    }
-  }
-
-  /* So now we have a final list of loaded components.  We can free all
-     the file information. */
   
-  for (cur = opal_list_remove_first(&found_files); 
-       NULL != cur;
-       cur = opal_list_remove_first(&found_files)) {
-    OBJ_RELEASE(cur);
-  }
+    /* If path is NULL, iterate over the set of directories specified by
+       the MCA param mca_base_component_path.  If path is not NULL, then
+       use that as the path. */
+  
+    if (NULL == path) {
+        mca_base_param_lookup_string(mca_base_param_component_path, &path_to_use);
+        if (NULL == path_to_use) {
+            /* If there's no path, then there's nothing to search -- we're
+               done */
+            return;
+        }
+    } else {
+        path_to_use = strdup(path);
+    }
+  
+    /* Iterate over all the files in the directories in the path and
+       make a master array of all the matching filenames that we
+       find. */
+  
+    OBJ_CONSTRUCT(&found_files, opal_list_t);
+    dir = path_to_use;
+    if (NULL != dir) {
+        do {
+            end = strchr(dir, OPAL_ENV_SEP);
+            if (NULL != end) {
+                *end = '\0';
+            }
+            if (0 != lt_dlforeachfile(dir, save_filename, &params)) {
+                break;
+            }
+            dir = end + 1;
+        } while (NULL != end);
+    }
+  
+    /* Iterate through all the filenames that we found.  Since one
+       component may [try to] call another to be loaded, only try to load
+       the UNVISITED files.  Also, ignore the return code -- basically,
+       give every file one chance to try to load.  If they load, great.
+       If not, great. */
 
-  /* All done */
+    for (cur = opal_list_get_first(&found_files); 
+         opal_list_get_end(&found_files) != cur;
+         cur = opal_list_get_next(cur)) {
+        file = (component_file_item_t *) cur;
 
-  if (NULL != param) {
-    free(param);
-  }
-  if (NULL != path_to_use) {
+        if( UNVISITED == file->status ) {
+            bool op = true;
+            file->status = CHECKING_CYCLE;
+
+            op = use_component(include_mode, name, file->name);
+            if( true == op ) {
+                open_component(file, found_components);
+            }
+        }
+    }
+    
+
+    /* So now we have a final list of loaded components.  We can free all
+       the file information. */
+  
+    for (cur = opal_list_remove_first(&found_files); 
+         NULL != cur;
+         cur = opal_list_remove_first(&found_files)) {
+        OBJ_RELEASE(cur);
+    }
+
+    /* All done, now let's cleanup */
     free(path_to_use);
-  }
-  OBJ_DESTRUCT(&found_files);
+
+    OBJ_DESTRUCT(&found_files);
 }
 
 
@@ -279,14 +324,14 @@ static int save_filename(const char *filename, lt_ptr data)
      named */
 
   len = sizeof(component_template) + strlen(params->type) + 32;
-  if (NULL != params->name) {
+  if ( 0 < strlen(params->name) ) {
     len += strlen(params->name);
   }
   prefix = (char*)malloc(len);
   snprintf(prefix, len, component_template, params->type);
   prefix_len = strlen(prefix);
-  if (NULL != params->name) {
-    strcat(prefix, params->name);
+  if (0 < strlen(params->name)) {
+    strncat(prefix, params->name, len - prefix_len);
   }
   total_len = strlen(prefix);
 
@@ -306,12 +351,18 @@ static int save_filename(const char *filename, lt_ptr data)
 
   component_file = OBJ_NEW(component_file_item_t);
   if (NULL == component_file) {
+    free(prefix);
     return OPAL_ERR_OUT_OF_RESOURCE;
   }
-  strcpy(component_file->type, params->type);
-  strcpy(component_file->name, basename + prefix_len);
-  strcpy(component_file->basename, basename);
-  strcpy(component_file->filename, filename);
+  strncpy(component_file->type, params->type, MCA_BASE_MAX_TYPE_NAME_LEN);
+  component_file->type[MCA_BASE_MAX_TYPE_NAME_LEN] = '\0';
+  strncpy(component_file->name, basename + prefix_len,
+          MCA_BASE_MAX_COMPONENT_NAME_LEN);
+  component_file->name[MCA_BASE_MAX_COMPONENT_NAME_LEN] = '\0';
+  strncpy(component_file->basename, basename, OMPI_PATH_MAX);
+  component_file->basename[OMPI_PATH_MAX] = '\0';
+  strncpy(component_file->filename, filename, OMPI_PATH_MAX);
+  component_file->filename[OMPI_PATH_MAX] = '\0';
   component_file->status = UNVISITED;
   opal_list_append(&found_files, (opal_list_item_t *) component_file);
 
@@ -337,17 +388,19 @@ static int open_component(component_file_item_t *target_file,
   mca_base_component_list_item_t *mitem;
   dependency_item_t *ditem;
   size_t len;
+  int vl;
 
   opal_output_verbose(40, 0, "mca: base: component_find: examining dyanmic %s MCA component \"%s\"",
-                     target_file->type, target_file->name, NULL);
-  opal_output_verbose(40, 0, "mca: base: component_find: %s", target_file->filename, NULL);
+                     target_file->type, target_file->name);
+  opal_output_verbose(40, 0, "mca: base: component_find: %s", target_file->filename);
   param = mca_base_param_find("mca", NULL, "component_show_load_errors");
   mca_base_param_lookup_int(param, &show_errors);
+  vl = show_errors ? 0 : 40;
 
   /* Was this component already loaded (e.g., via dependency)? */
 
   if (LOADED == target_file->status) {
-    opal_output_verbose(40, 0, "mca: base: component_find: already loaded (ignored)", NULL);
+    opal_output_verbose(40, 0, "mca: base: component_find: already loaded (ignored)");
     return OPAL_SUCCESS;
   }
 
@@ -363,7 +416,7 @@ static int open_component(component_file_item_t *target_file,
     mitem = (mca_base_component_list_item_t *) cur;
     if (0 == strcmp(mitem->cli_component->mca_type_name, target_file->type) &&
         0 == strcmp(mitem->cli_component->mca_component_name, target_file->name)) {
-      opal_output_verbose(40, 0, "mca: base: component_find: already loaded (ignored)", NULL);
+      opal_output_verbose(40, 0, "mca: base: component_find: already loaded (ignored)");
       target_file->status = FAILED_TO_LOAD;
       return OPAL_ERR_BAD_PARAM;
     }
@@ -382,15 +435,15 @@ static int open_component(component_file_item_t *target_file,
 
   /* Now try to load the component */
 
+#if OPAL_HAVE_LTDL_ADVISE
+  component_handle = lt_dlopenadvise(target_file->filename, opal_mca_dladvise);
+#else
   component_handle = lt_dlopenext(target_file->filename);
+#endif
   if (NULL == component_handle) {
     err = strdup(lt_dlerror());
-    if (0 != show_errors) {
-        opal_output(0, "mca: base: component_find: unable to open %s %s: %s (ignored)", 
-                    target_file->type, target_file->name, err);
-    }
-    opal_output_verbose(40, 0, "mca: base: component_find: unable to open %s: %s (ignored)", 
-                        target_file->filename, err, NULL);
+    opal_output_verbose(vl, 0, "mca: base: component_find: unable to open %s: %s (ignored)", 
+                        target_file->filename, err);
     free(err);
     target_file->status = FAILED_TO_LOAD;
     free_dependency_list(&dependencies);
@@ -422,14 +475,9 @@ static int open_component(component_file_item_t *target_file,
 
   component_struct = (mca_base_component_t*)lt_dlsym(component_handle, struct_name);
   if (NULL == component_struct) {
-    if (0 != show_errors) {
-        opal_output(0, "mca: base: component_find: \"%s\" does not appear to be a valid "
-                       "%s MCA dynamic component (ignored)", 
-                       target_file->basename, target_file->type, NULL);
-    }
-    opal_output_verbose(40, 0, "mca: base: component_find: \"%s\" does not appear to be a valid "
-                       "%s MCA dynamic component (ignored)", 
-                       target_file->basename, target_file->type, NULL);
+    opal_output_verbose(vl, 0, "mca: base: component_find: \"%s\" does not appear to be a valid "
+                        "%s MCA dynamic component (ignored)", 
+                        target_file->basename, target_file->type);
     free(mitem);
     free(struct_name);
     lt_dlclose(component_handle);
@@ -438,8 +486,44 @@ static int open_component(component_file_item_t *target_file,
     return OPAL_ERR_BAD_PARAM;
   }
 
-  /* We found the public struct.  Save it, and register this component to
-     be closed later. */
+  /* We found the public struct.  Make sure its MCA major.minor
+     version is the same as ours. */
+  if (!(MCA_BASE_VERSION_MAJOR == component_struct->mca_major_version &&
+        MCA_BASE_VERSION_MINOR == component_struct->mca_minor_version)) {
+      opal_output_verbose(vl, 0, "mca: base: component_find: %s \"%s\" uses an MCA interface that is not recognized (component MCA v%d.%d.%d != supported MCA v%d.%d.%d) -- ignored",
+                          target_file->type, target_file->basename, 
+                          component_struct->mca_major_version,
+                          component_struct->mca_minor_version,
+                          component_struct->mca_release_version,
+                          MCA_BASE_VERSION_MAJOR,
+                          MCA_BASE_VERSION_MINOR,
+                          MCA_BASE_VERSION_RELEASE);
+    free(mitem);
+    free(struct_name);
+    lt_dlclose(component_handle);
+    target_file->status = FAILED_TO_LOAD;
+    free_dependency_list(&dependencies);
+    return OPAL_ERR_BAD_PARAM;
+  }
+
+  /* Also check that the component struct framework and component
+     names match the expected names from the filename */
+  if (0 != strcmp(component_struct->mca_type_name, target_file->type) ||
+      0 != strcmp(component_struct->mca_component_name, target_file->name)) {
+      opal_output_verbose(vl, 0, "Component file data does not match filename: %s (%s / %s) != %s %s -- ignored",
+                          target_file->filename, target_file->type, target_file->name,
+                          component_struct->mca_type_name, 
+                          component_struct->mca_component_name);
+      free(mitem);
+      free(struct_name);
+      lt_dlclose(component_handle);
+      target_file->status = FAILED_TO_LOAD;
+      free_dependency_list(&dependencies);
+      return OPAL_ERR_BAD_PARAM;
+  }
+
+  /* Alles gut.  Save the component struct, and register this
+     component to be closed later. */
 
   mitem->cli_component = component_struct;
   opal_list_append(found_components, (opal_list_item_t *) mitem);
@@ -462,7 +546,7 @@ static int open_component(component_file_item_t *target_file,
   OBJ_DESTRUCT(&dependencies);
 
   opal_output_verbose(40, 0, "mca: base: component_find: opened dynamic %s MCA component \"%s\"",
-                     target_file->type, target_file->name, NULL);
+                     target_file->type, target_file->name);
   target_file->status = LOADED;
     
   /* All done */
@@ -508,7 +592,7 @@ static int check_ompi_info(component_file_item_t *target_file,
      them.  Return failure upon the first component that fails to
      load. */
 
-  opal_output_verbose(40, 0, "mca: base: component_find: opening ompi_info file: %s", depname, NULL);
+  opal_output_verbose(40, 0, "mca: base: component_find: opening ompi_info file: %s", depname);
   while (NULL != fgets(buffer, BUFSIZ, fp)) {
 
     /* Perl chomp */
@@ -551,7 +635,7 @@ static int check_ompi_info(component_file_item_t *target_file,
     }
   }
   opal_output_verbose(40, 0, "mca: base: component_find: ompi_info file closed (%s)", 
-                     target_file->basename, NULL);
+                     target_file->basename);
 
   /* All done -- all depenencies satisfied */
 
@@ -573,6 +657,7 @@ static int check_dependency(char *line, component_file_item_t *target_file,
   bool happiness;
   char buffer[BUFSIZ];
   char *type, *name;
+  int len;
   component_file_item_t *mitem;
   dependency_item_t *ditem;
   opal_list_item_t *cur;
@@ -594,7 +679,8 @@ static int check_dependency(char *line, component_file_item_t *target_file,
     return OPAL_ERR_OUT_OF_RESOURCE;
   }
   snprintf(buffer, BUFSIZ, component_template, type);
-  strcat(buffer, name);
+  len = strlen(buffer);
+  strncat(buffer, name, BUFSIZ - len);
 
   /* Traverse down the list of files that we have, and see if we can
      find it */
@@ -615,8 +701,7 @@ static int check_dependency(char *line, component_file_item_t *target_file,
 
     else if (mitem == target_file) {
       opal_output_verbose(40, 0,
-                         "mca: base: component_find: component depends on itself (ignored dependency)", 
-                         NULL);
+                         "mca: base: component_find: component depends on itself (ignored dependency)");
       happiness = true;
       break;
     }
@@ -626,7 +711,7 @@ static int check_dependency(char *line, component_file_item_t *target_file,
 
     else if (LOADED == mitem->status) {
       opal_output_verbose(40, 0, "mca: base: component_find: dependency has already been loaded (%s)",
-                         mitem->basename, NULL);
+                         mitem->basename);
       happiness = true;
       break;
     }
@@ -637,7 +722,7 @@ static int check_dependency(char *line, component_file_item_t *target_file,
 
     else if (FAILED_TO_LOAD == mitem->status) {
       opal_output_verbose(40, 0, "mca: base: component_find: dependency previously failed to load (%s)",
-                         mitem->basename, NULL);
+                         mitem->basename);
       break;
     }
 
@@ -645,7 +730,7 @@ static int check_dependency(char *line, component_file_item_t *target_file,
 
     else if (CHECKING_CYCLE == mitem->status) {
       opal_output_verbose(40, 0, "mca: base: component_find: found cycle! (%s)",
-                         mitem->basename, NULL);
+                         mitem->basename);
       break;
     }
 
@@ -654,12 +739,12 @@ static int check_dependency(char *line, component_file_item_t *target_file,
 
     else if (UNVISITED == mitem->status) {
       opal_output_verbose(40, 0, "mca: base: component_find: loading dependency (%s)",
-                         mitem->basename, NULL);
+                         mitem->basename);
       if (OPAL_SUCCESS == open_component(target_file, found_components)) {
         happiness = true;
       } else {
         opal_output_verbose(40, 0, "mca: base: component_find: dependency failed to load (%s)",
-                           mitem->basename, NULL);
+                           mitem->basename);
       }
       break;
     }
@@ -709,3 +794,47 @@ static void free_dependency_list(opal_list_t *dependencies)
 }
 
 #endif /* OMPI_WANT_LIBLTDL */
+
+static bool use_component(const bool include_mode,
+                          const char **requested_component_names,
+                          const char *component_name)
+{
+    bool found = false;
+    const char **req_comp_name = requested_component_names;
+    
+    /*
+     * If no selection is specified then we use all components
+     * we can find.
+     */
+    if (NULL == req_comp_name) {
+        return true;
+    }
+
+    while ( *req_comp_name != NULL ) {
+        if ( strcmp(component_name, *req_comp_name) == 0 ) {
+            found = true;
+            break;
+        }
+        req_comp_name++;
+    }
+
+    /*
+     * include_mode  found |   use
+     * --------------------+------
+     *            0      0 |  true
+     *            0      1 | false
+     *            1      0 | false
+     *            1      1 |  true
+     *
+     * -> inverted xor
+     * As xor is a binary operator let's implement it manually before
+     * a compiler screws it up.
+     */
+    if ( (include_mode && found) || !(include_mode || found) ) {
+        return true;
+    } else {    
+        return false;
+    }
+}
+
+
