@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -34,6 +34,7 @@
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_component_repository.h"
 #include "opal/constants.h"
+#include "opal/util/opal_environ.h"
 
 /*
  * Public variables
@@ -54,7 +55,7 @@ static void parse_verbose(char *e, opal_output_stream_t *lds);
 int mca_base_open(void)
 {
   int param_index;
-  char *value, *home;
+  char *value;
   opal_output_stream_t lds;
   char hostname[64];
 
@@ -65,13 +66,11 @@ int mca_base_open(void)
   }
 
   /* Register some params */
-#if !defined(__WINDOWS__)
-  home = getenv("HOME");
-  asprintf(&value, "%s:%s"OPAL_PATH_SEP".openmpi"OPAL_PATH_SEP"components", opal_install_dirs.pkglibdir, home);
+#if OMPI_WANT_HOME_CONFIG_FILES
+  asprintf(&value, "%s%c%s"OPAL_PATH_SEP".openmpi"OPAL_PATH_SEP"components", opal_install_dirs.pkglibdir, OPAL_ENV_SEP, opal_home_directory() );
 #else
-  home = getenv("USERPROFILE");
-  asprintf(&value, "%s;%s"OPAL_PATH_SEP".openmpi"OPAL_PATH_SEP"components", opal_install_dirs.pkglibdir, home);
-#endif  /* !defined(__WINDOWS__) */
+  asprintf(&value, "%s", opal_install_dirs.pkglibdir); 
+#endif
 
   mca_base_param_component_path = 
     mca_base_param_reg_string_name("mca", "component_path",
@@ -104,7 +103,7 @@ int mca_base_open(void)
   asprintf(&lds.lds_prefix, "[%s:%05d] ", hostname, getpid());
   opal_output_reopen(0, &lds);
   opal_output_verbose(5, 0, "mca: base: opening components");
-
+  free(lds.lds_prefix);
   /* Open up the component repository */
 
   return mca_base_component_repository_init();
