@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2006 The University of Tennessee and The University
@@ -39,15 +39,19 @@ extern "C" {
     /**
      * Globally exported variable
      */
-    OPAL_MODULE_DECLSPEC extern const opal_maffinity_base_component_1_0_0_t
+    OPAL_MODULE_DECLSPEC extern const opal_maffinity_base_component_2_0_0_t
         mca_maffinity_first_use_component;
 
 
     /**
      * maffinity query API function
+     *
+     * Query function for maffinity components.  Simply returns a priority
+     * to rank it against other available maffinity components (assumedly,
+     * only one component will be available per platform, but it's
+     * possible that there could be more than one available).
      */
-    const opal_maffinity_base_module_1_0_0_t *
-        opal_maffinity_first_use_component_query(int *query);
+    int opal_maffinity_first_use_component_query(mca_base_module_t **module, int *priority);
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
