@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -32,16 +33,14 @@ extern "C" {
      * Globally exported variable
      */
 
-    OPAL_DECLSPEC extern const opal_paffinity_base_component_1_0_0_t
+    OPAL_DECLSPEC extern const opal_paffinity_base_component_2_0_0_t
         mca_paffinity_solaris_component;
 
 
     /*
      * paffinity API functions
      */
-
-    const opal_paffinity_base_module_1_0_0_t *
-        opal_paffinity_solaris_component_query(int *query);
+    int opal_paffinity_solaris_component_query(mca_base_module_t **module, int *priority);
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
