@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Sun Microsystems, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,19 +20,23 @@
 #ifndef OPAL_MEMORY_MEMORY_INTERNAL_H
 #define OPAL_MEMORY_MEMORY_INTERNAL_H
 
+#include "opal_config.h"
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
+/* see memory.h for meaning */
 #define OPAL_MEMORY_FREE_SUPPORT   0x0001
-#define OPAL_MEMORY_MALLOC_SUPPORT 0x0002
+#define OPAL_MEMORY_MUNMAP_SUPPORT 0x0002
 #define OPAL_MEMORY_CHUNK_SUPPORT  0x0004
-#define OPAL_MEMORY_MMAP_SUPPORT   0x0008
 
 #if defined(c_plusplus) || defined(__cplusplus)
 extern "C" {
 #endif
 
-void opal_mem_hooks_set_support(int support);
+OPAL_DECLSPEC void opal_mem_hooks_set_support(int support);
 
-void opal_mem_hooks_release_hook(void *buf, size_t length, bool from_alloc);
-void opal_mem_hooks_alloc_hook(void *buf, size_t length, bool from_alloc);
+OPAL_DECLSPEC void opal_mem_hooks_release_hook(void *buf, size_t length, bool from_alloc);
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
