@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,11 +22,14 @@
 #include "opal_config.h"
 
 #include "opal/class/opal_object.h"
+#include "opal/dss/dss.h"
 #include "opal/util/trace.h"
 #include "opal/util/output.h"
 #include "opal/util/malloc.h"
 #include "opal/util/if.h"
+#include "opal/util/net.h"
 #include "opal/util/keyval_parse.h"
+#include "opal/util/show_help.h"
 #include "opal/memoryhooks/memory.h"
 #include "opal/mca/base/base.h"
 #include "opal/runtime/opal.h"
@@ -36,10 +40,26 @@
 #include "opal/mca/backtrace/base/base.h"
 #include "opal/mca/timer/base/base.h"
 #include "opal/mca/paffinity/base/base.h"
+#include "opal/event/event.h"
+#include "opal/runtime/opal_progress.h"
+#include "opal/mca/carto/base/base.h"
+
+#include "opal/runtime/opal_cr.h"
+#include "opal/mca/crs/base/base.h"
+
+extern int opal_initialized;
+extern int opal_util_initialized;
 
 int
 opal_finalize_util(void)
 {
+    if( --opal_util_initialized != 0 ) {
+        if( opal_util_initialized < 0 ) {
+            return OPAL_ERROR;
+        }
+        return OPAL_SUCCESS;
+    }
+
     /* Clear out all the registered MCA params */
     mca_base_param_finalize();
 
@@ -47,6 +67,8 @@ opal_finalize_util(void)
        close when not opened internally */
     opal_iffinalize();
 
+    opal_net_finalize();
+
     /* keyval lex-based parser */
     opal_util_keyval_parse_finalize();
 
@@ -58,11 +80,17 @@ opal_finalize_util(void)
     /* finalize the trace system */
     opal_trace_finalize();
 
+    /* finalize the show_help system */
+    opal_show_help_finalize();
+
     /* finalize the output system.  This has to come *after* the
        malloc code, as the malloc code needs to call into this, but
        the malloc code turning off doesn't affect opal_output that
        much */
     opal_output_finalize();
+    
+    /* close the dss */
+    opal_dss_close();
 
     /* finalize the class/object system */
     opal_class_finalize();
@@ -70,7 +98,6 @@ opal_finalize_util(void)
     return OPAL_SUCCESS;
 }
 
-extern int opal_initialized;
 
 int
 opal_finalize(void)
@@ -81,6 +108,14 @@ opal_finalize(void)
         }
         return OPAL_SUCCESS;
     }
+
+    /* close the checkpoint and restart service */
+    opal_cr_finalize();
+
+    opal_progress_finalize();
+
+    opal_event_fini();
+
     /* close high resolution timers */
     opal_timer_base_close();
 
@@ -96,6 +131,9 @@ opal_finalize(void)
     /* finalize the memory manager / tracker */
     opal_mem_hooks_finalize();
 
+    /* close the carto framework */
+    opal_carto_base_close();
+    
     /* close the processor affinity base */
     opal_paffinity_base_close();
 
