@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -29,7 +29,6 @@
 #include "opal/threads/mutex.h"
 #include "opal/util/argv.h"
 #include "opal/util/cmd_line.h"
-#include "opal/util/strncpy.h"
 #include "opal/util/output.h"
 #include "opal/mca/base/mca_base_param.h"
 #include "opal/constants.h"
@@ -68,9 +67,10 @@ struct cmd_line_option_t {
 typedef struct cmd_line_option_t cmd_line_option_t;
 static void option_constructor(cmd_line_option_t *cmd);
 static void option_destructor(cmd_line_option_t *cmd);
-static OBJ_CLASS_INSTANCE(cmd_line_option_t,
-                          opal_list_item_t,
-                          option_constructor, option_destructor);
+
+OBJ_CLASS_INSTANCE(cmd_line_option_t,
+                   opal_list_item_t,
+                   option_constructor, option_destructor);
 
 /*
  * An option that was used in the argv that was parsed
@@ -99,9 +99,9 @@ struct cmd_line_param_t {
 typedef struct cmd_line_param_t cmd_line_param_t;
 static void param_constructor(cmd_line_param_t *cmd);
 static void param_destructor(cmd_line_param_t *cmd);
-static OBJ_CLASS_INSTANCE(cmd_line_param_t,
-                          opal_list_item_t,
-                          param_constructor, param_destructor);
+OBJ_CLASS_INSTANCE(cmd_line_param_t,
+                   opal_list_item_t,
+                   param_constructor, param_destructor);
 
 /*
  * Instantiate the opal_cmd_line_t class
@@ -142,7 +142,7 @@ static int qsort_callback(const void *a, const void *b);
 int opal_cmd_line_create(opal_cmd_line_t *cmd,
                          opal_cmd_line_init_t *table)
 {
-    int i, ret;
+    int i, ret = OPAL_SUCCESS;
 
     /* Check bozo case */
 
@@ -174,6 +174,26 @@ int opal_cmd_line_create(opal_cmd_line_t *cmd,
         ret = make_opt(cmd, &table[i]);
     }
 
+    return ret;
+}
+
+/*
+ * Append a command line entry to the previously constructed command line
+ */
+int opal_cmd_line_make_opt_mca(opal_cmd_line_t *cmd,
+                               opal_cmd_line_init_t entry)
+{
+    int ret;
+
+    /* Ensure we got an entry */
+    if ('\0' == entry.ocl_cmd_short_name &&
+        NULL == entry.ocl_cmd_single_dash_name &&
+        NULL == entry.ocl_cmd_long_name) {
+        return OPAL_SUCCESS;
+    }
+
+    ret = make_opt(cmd, &entry);
+
     return OPAL_SUCCESS;
 }
 
@@ -481,7 +501,7 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
  */
 char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
 {
-    int i, len = MAX_WIDTH * 2, prev_len;
+    size_t i, len;
     int argc;
     size_t j;
     char **argv;
@@ -497,15 +517,14 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
 
     /* Make an argv of all the usage strings */
 
-    prev_len = 0;
     argc = 0;
     argv = NULL;
     ret = NULL;
 
     /* First, take the original list and sort it */
 
-    sorted = malloc(sizeof(cmd_line_option_t *) * 
-                    opal_list_get_size(&cmd->lcl_options));
+    sorted = (cmd_line_option_t**)malloc(sizeof(cmd_line_option_t *) * 
+                                         opal_list_get_size(&cmd->lcl_options));
     if (NULL == sorted) {
         return NULL;
     }
@@ -553,8 +572,9 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
                 filled = true;
             }
             strcat(line, " ");
-            for (i = 0; i < option->clo_num_params; ++i) {
-                snprintf(temp, len, "<arg%d> ", i);
+            for (i = 0; (int)i < option->clo_num_params; ++i) {
+                len = sizeof(temp);
+                snprintf(temp, len, "<arg%d> ", (int)i);
                 strcat(line, temp);
             }
             if (option->clo_num_params > 0) {
@@ -594,6 +614,7 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
 
             desc = strdup(option->clo_description);
             if (NULL == desc) {
+                free(sorted);
                 return strdup("");
             }
             start = desc;
@@ -674,13 +695,12 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
     } else {
         ret = strdup("");
     }
+    free(sorted);
 
     /* Thread serialization */
-
     opal_mutex_unlock(&cmd->lcl_mutex);
 
     /* All done */
- 
     return ret;
 }
 
@@ -1036,7 +1056,7 @@ static int split_shorts(opal_cmd_line_t *cmd, char *token, char **args,
        (argv[i] + 1), will be empty by the time it gets down here),
        just return that we didn't find a short option. */
 
-    len = strlen(token);
+    len = (int)strlen(token);
     if (0 == len) {
         return OPAL_ERR_BAD_PARAM;
     }
@@ -1117,7 +1137,7 @@ static void set_dest(cmd_line_option_t *option, char *sval)
 {
     int ival = atoi(sval);
     long lval = strtol(sval, NULL, 10);
-    char *str;
+    char *str = NULL;
 
     /* Set MCA param.  We do this in the environment because the MCA
        parameter may not have been registered yet -- and if it isn't
