@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -35,7 +35,7 @@
  *
  * The "single dash" name is a multi-character name that only
  * requires a single dash.  This only exists to provide backwards
- * compatability for some well-known command line options in prior
+ * compatibility for some well-known command line options in prior
  * MPI implementations (e.g., "mpirun -np 3").  It should be used
  * sparingly.
  *
@@ -238,7 +238,7 @@ extern "C" {
      * destructor for opal_cmd_line_t handles will free all memory
      * associated with the handle.
      */
-    OMPI_DECLSPEC OBJ_CLASS_DECLARATION(opal_cmd_line_t);
+    OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_cmd_line_t);
 
     /**
      * Make a command line handle from a table of initializers.
@@ -274,9 +274,21 @@ extern "C" {
      * };
      * \endcode
      */
-    OMPI_DECLSPEC int opal_cmd_line_create(opal_cmd_line_t *cmd,
+    OPAL_DECLSPEC int opal_cmd_line_create(opal_cmd_line_t *cmd,
                                            opal_cmd_line_init_t *table);
 
+    /**
+     * Create a command line option.
+     *
+     * @param cmd OPAL command line handle.
+     * @param entry Command line entry to add to the command line.
+     *
+     * @retval OPAL_SUCCESS Upon success.
+     *
+     */
+    OPAL_DECLSPEC int opal_cmd_line_make_opt_mca(opal_cmd_line_t *cmd,
+                                                 opal_cmd_line_init_t entry);
+
     /**
      * \deprecated
      *
@@ -287,11 +299,11 @@ extern "C" {
      * opal_cmd_line_make_opt3(cmd, short_name, NULL, long_name,
      * num_params, desc).
      */
-    OMPI_DECLSPEC int opal_cmd_line_make_opt(opal_cmd_line_t *cmd,
+    OPAL_DECLSPEC int opal_cmd_line_make_opt(opal_cmd_line_t *cmd,
                                              char short_name, 
                                              const char *long_name,
                                              int num_params, 
-                                             const char *desc);
+                                             const char *desc) __opal_attribute_deprecated__;
 
     /**
      * Create a command line option.
@@ -321,7 +333,7 @@ extern "C" {
      * used to generate the output from opal_cmd_line_get_usage_msg().
      *
      */
-    OMPI_DECLSPEC int opal_cmd_line_make_opt3(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC int opal_cmd_line_make_opt3(opal_cmd_line_t *cmd, 
                                               char short_name, 
                                               const char *sd_name,
                                               const char *long_name, 
@@ -379,7 +391,7 @@ extern "C" {
      * Invoking this function multiple times on different sets of argv
      * tokens is safe, but will erase any previous parsing results.
      */
-    OMPI_DECLSPEC int opal_cmd_line_parse(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC int opal_cmd_line_parse(opal_cmd_line_t *cmd, 
                                           bool ignore_unknown,
                                           int argc, char **argv);
 
@@ -404,7 +416,7 @@ extern "C" {
      *
      * The returned string must be freed by the caller.
      */
-    OMPI_DECLSPEC char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd);
+    OPAL_DECLSPEC char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd);
 
     /**
      * Test if a given option was taken on the parsed command line.
@@ -425,7 +437,7 @@ extern "C" {
      * (either by its short or long name) during token parsing.
      * Otherwise, it will return false.
      */
-    OMPI_DECLSPEC bool opal_cmd_line_is_taken(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC bool opal_cmd_line_is_taken(opal_cmd_line_t *cmd, 
                                               const char *opt);
 
     /**
@@ -439,7 +451,7 @@ extern "C" {
      * Arguments are added to the handle via the opal_cmd_line_parse()
      * function.
      */
-    OMPI_DECLSPEC int opal_cmd_line_get_argc(opal_cmd_line_t *cmd);
+    OPAL_DECLSPEC int opal_cmd_line_get_argc(opal_cmd_line_t *cmd);
 
     /**
      * Return a string argument parsed on a OPAL command line handle.
@@ -458,7 +470,7 @@ extern "C" {
      * What is returned is a pointer to the actual string that is on
      * the handle; it should not be modified or freed.
      */
-    OMPI_DECLSPEC char *opal_cmd_line_get_argv(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC char *opal_cmd_line_get_argv(opal_cmd_line_t *cmd, 
                                                int index);
 
     /**
@@ -470,18 +482,18 @@ extern "C" {
      * @retval num Number of instances (to include 0) of a given potion
      * found during opal_cmd_line_parse().
      *
-     * @retval OMPI_ERR If the command line option was not found during
+     * @retval OPAL_ERR If the command line option was not found during
      * opal_cmd_line_parse(), or opal_cmd_line_parse() was not invoked on
      * this handle.
      *
      * This function should only be called after opal_cmd_line_parse().
      *
      * The function will return the number of instances of a given option
-     * (either by its short or long name) -- to include 0 -- or OMPI_ERR if
+     * (either by its short or long name) -- to include 0 -- or OPAL_ERR if
      * either the option was not specified as part of the OPAL command line
      * handle, or opal_cmd_line_parse() was not invoked on this handle.
      */
-    OMPI_DECLSPEC int opal_cmd_line_get_ninsts(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC int opal_cmd_line_get_ninsts(opal_cmd_line_t *cmd, 
                                                const char *opt);
 
     /**
@@ -511,7 +523,7 @@ extern "C" {
      * The returned string should \em not be modified or freed by the
      * caller.
      */
-    OMPI_DECLSPEC char *opal_cmd_line_get_param(opal_cmd_line_t *cmd, 
+    OPAL_DECLSPEC char *opal_cmd_line_get_param(opal_cmd_line_t *cmd, 
                                                 const char *opt, 
                                                 int instance_num,
                                                 int param_num);
@@ -543,7 +555,7 @@ extern "C" {
      * of the tail parameters, and must be freed (likely with a call
      * to opal_argv_free()) by the caller.
      */
-    OMPI_DECLSPEC int opal_cmd_line_get_tail(opal_cmd_line_t *cmd, int *tailc, 
+    OPAL_DECLSPEC int opal_cmd_line_get_tail(opal_cmd_line_t *cmd, int *tailc, 
                                              char ***tailv);
 
 #if defined(c_plusplus) || defined(__cplusplus)
