@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -47,22 +48,11 @@ static char *output_dir = NULL;
 static char *output_prefix = NULL;
 
 
-/*
- * Private functions
- */
-static void construct(opal_object_t *stream);
-static int do_open(int output_id, opal_output_stream_t * lds);
-static int open_file(int i);
-static void free_descriptor(int output_id);
-static void output(int output_id, const char *format, va_list arglist);
-
-
 /*
  * Internal data structures and helpers for the generalized output
  * stream mechanism.
  */
-struct output_desc_t
-{
+typedef struct {
     bool ldi_used;
     bool ldi_enabled;
     int ldi_verbose_level;
@@ -78,6 +68,9 @@ struct output_desc_t
     char *ldi_prefix;
     int ldi_prefix_len;
 
+    char *ldi_suffix;
+    int ldi_suffix_len;
+    
     bool ldi_stdout;
     bool ldi_stderr;
 
@@ -86,16 +79,32 @@ struct output_desc_t
     char *ldi_file_suffix;
     int ldi_fd;
     int ldi_file_num_lines_lost;
-};
-typedef struct output_desc_t output_desc_t;
+} output_desc_t;
 
-#define OPAL_OUTPUT_MAX_STREAMS 32
+/*
+ * Private functions
+ */
+static void construct(opal_object_t *stream);
+static int do_open(int output_id, opal_output_stream_t * lds);
+static int open_file(int i);
+static void free_descriptor(int output_id);
+static int make_string(char **no_newline_string, output_desc_t *ldi, 
+                       const char *format, va_list arglist);
+static int output(int output_id, const char *format, va_list arglist);
 
 
+#define OPAL_OUTPUT_MAX_STREAMS 64
+#if defined(__WINDOWS__) || defined(HAVE_SYSLOG)
+#define USE_SYSLOG 1
+#else
+#define USE_SYSLOG 0
+#endif
+
 /*
  * Local state
  */
 static bool initialized = false;
+static int default_stderr_fd = -1;
 static output_desc_t info[OPAL_OUTPUT_MAX_STREAMS];
 static char *temp_str = 0;
 static size_t temp_str_len = 0;
@@ -111,13 +120,18 @@ OBJ_CLASS_INSTANCE(opal_output_stream_t, opal_object_t, construct, NULL);
 bool opal_output_init(void)
 {
     int i;
-    char *str;
     char hostname[32];
+    char *str;
 
     if (initialized) {
         return true;
     }
 
+    str = getenv("OPAL_OUTPUT_STDERR_FD");
+    if (NULL != str) {
+        default_stderr_fd = atoi(str);
+    }
+
     OBJ_CONSTRUCT(&verbose, opal_output_stream_t);
 #if defined(__WINDOWS__)
     {
@@ -149,20 +163,9 @@ bool opal_output_init(void)
     /* Set some defaults */
 
     asprintf(&output_prefix, "output-pid%d-", getpid());
-    if (NULL != (str = getenv("TMPDIR"))) {
-        output_dir = strdup(str);
-	} else if (NULL != (str = getenv("TEMP"))) {
-        output_dir = strdup(str);
-	} else if (NULL != (str = getenv("TMP"))) {
-        output_dir = strdup(str);
-    } else if (NULL != (str = getenv("HOME"))) {
-        output_dir = strdup(str);
-    } else {
-        output_dir = strdup(".");
-    }
+    output_dir = strdup(opal_tmp_directory());
 
     /* Open the default verbose stream */
-
     verbose_stream = opal_output_open(&verbose);
     return true;
 }
@@ -187,7 +190,7 @@ int opal_output_reopen(int output_id, opal_output_stream_t * lds)
 
 
 /*
- * Enable and disable outptu streams
+ * Enable and disable output streams
  */
 bool opal_output_switch(int output_id, bool enable)
 {
@@ -230,10 +233,15 @@ void opal_output_reopen_all(void)
          */
         info[i].ldi_used = false;
 
+#if USE_SYSLOG
         lds.lds_want_syslog = info[i].ldi_syslog;
         lds.lds_syslog_priority = info[i].ldi_syslog_priority;
         lds.lds_syslog_ident = info[i].ldi_syslog_ident;
+#else
+        lds.lds_want_syslog = false;
+#endif
         lds.lds_prefix = info[i].ldi_prefix;
+        lds.lds_suffix = info[i].ldi_suffix;
         lds.lds_want_stdout = info[i].ldi_stdout;
         lds.lds_want_stderr = info[i].ldi_stderr;
         lds.lds_want_file = (-1 == info[i].ldi_fd) ? false : true;
@@ -279,12 +287,14 @@ void opal_output_close(int output_id)
             }
         }
 
-#ifndef __WINDOWS__
+#if defined(HAVE_SYSLOG)
         if (i >= OPAL_OUTPUT_MAX_STREAMS && syslog_opened) {
             closelog();
         }
-#else
-        DeregisterEventSource(info[output_id].ldi_syslog_ident);
+#elif defined(__WINDOWS__)
+        if(info[output_id].ldi_syslog_ident != NULL) {
+            DeregisterEventSource(info[output_id].ldi_syslog_ident);
+        }
 #endif
     }
 
@@ -328,6 +338,63 @@ void opal_output_verbose(int level, int output_id, const char *format, ...)
 }
 
 
+/*
+ * Send a message to a stream if the verbose level is high enough
+ */
+void opal_output_vverbose(int level, int output_id, const char *format, 
+                          va_list arglist)
+{
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
+        info[output_id].ldi_verbose_level >= level) {
+        output(output_id, format, arglist);
+    }
+}
+
+
+/*
+ * Send a message to a string if the verbose level is high enough
+ */
+char *opal_output_string(int level, int output_id, const char *format, ...)
+{
+    int rc;
+    char *ret = NULL;
+
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
+        info[output_id].ldi_verbose_level >= level) {
+        va_list arglist;
+        va_start(arglist, format);
+        rc = make_string(&ret, &info[output_id], format, arglist);
+        va_end(arglist);
+        if (OPAL_SUCCESS != rc) {
+            ret = NULL;
+        }
+    }
+
+    return ret;
+}
+
+
+/*
+ * Send a message to a string if the verbose level is high enough
+ */
+char *opal_output_vstring(int level, int output_id, const char *format,  
+                          va_list arglist)
+{
+    int rc;
+    char *ret = NULL;
+
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
+        info[output_id].ldi_verbose_level >= level) {
+        rc = make_string(&ret, &info[output_id], format, arglist);
+        if (OPAL_SUCCESS != rc) {
+            ret = NULL;
+        }
+    }
+
+    return ret;
+}
+
+
 /*
  * Set the verbosity level of a stream
  */
@@ -374,6 +441,7 @@ void opal_output_finalize(void)
         if (verbose_stream != -1) {
             opal_output_close(verbose_stream);
         }
+        free(verbose.lds_prefix);
         verbose_stream = -1;
 
         free (output_prefix);
@@ -399,6 +467,7 @@ static void construct(opal_object_t *obj)
     stream->lds_syslog_priority = 0;
     stream->lds_syslog_ident = NULL;
     stream->lds_prefix = NULL;
+    stream->lds_suffix = NULL;
     stream->lds_is_debugging = false;
     stream->lds_want_syslog = false;
     stream->lds_want_stdout = false;
@@ -464,10 +533,11 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         (bool) OMPI_ENABLE_DEBUG : true;
     info[i].ldi_verbose_level = lds->lds_verbose_level;
 
+#if USE_SYSLOG
     info[i].ldi_syslog = lds->lds_want_syslog;
     if (lds->lds_want_syslog) {
 
-#ifndef __WINDOWS__
+#if defined(HAVE_SYSLOG)
         if (NULL != lds->lds_syslog_ident) {
             info[i].ldi_syslog_ident = strdup(lds->lds_syslog_ident);
             openlog(lds->lds_syslog_ident, LOG_PID, LOG_USER);
@@ -475,7 +545,7 @@ static int do_open(int output_id, opal_output_stream_t * lds)
             info[i].ldi_syslog_ident = NULL;
             openlog("opal", LOG_PID, LOG_USER);
         }
-#else
+#elif defined(__WINDOWS__)
         if (NULL == (info[i].ldi_syslog_ident =
                      RegisterEventSource(NULL, TEXT("opal: ")))) {
             /* handle the error */
@@ -486,6 +556,9 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         syslog_opened = true;
         info[i].ldi_syslog_priority = lds->lds_syslog_priority;
     }
+#else
+    info[i].ldi_syslog = false;
+#endif
 
     if (NULL != lds->lds_prefix) {
         info[i].ldi_prefix = strdup(lds->lds_prefix);
@@ -495,6 +568,14 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         info[i].ldi_prefix_len = 0;
     }
 
+    if (NULL != lds->lds_suffix) {
+        info[i].ldi_suffix = strdup(lds->lds_suffix);
+        info[i].ldi_suffix_len = (int)strlen(lds->lds_suffix);
+    } else {
+        info[i].ldi_suffix = NULL;
+        info[i].ldi_suffix_len = 0;
+    }
+    
     info[i].ldi_stdout = lds->lds_want_stdout;
     info[i].ldi_stderr = lds->lds_want_stderr;
 
@@ -520,43 +601,43 @@ static int open_file(int i)
     /* Setup the filename and open flags */
 
     if (NULL != output_dir) {
-	filename = (char *) malloc(MAXPATHLEN);
-	if (NULL == filename) {
-	    return OPAL_ERR_OUT_OF_RESOURCE;
-	}
-	strcpy(filename, output_dir);
-	strcat(filename, "/");
+        filename = (char *) malloc(OMPI_PATH_MAX);
+        if (NULL == filename) {
+            return OPAL_ERR_OUT_OF_RESOURCE;
+        }
+        strncpy(filename, output_dir, OMPI_PATH_MAX);
+        strcat(filename, "/");
         if (NULL != output_prefix) {
             strcat(filename, output_prefix);
         }
-	if (info[i].ldi_file_suffix != NULL) {
-	    strcat(filename, info[i].ldi_file_suffix);
-	} else {
-	    info[i].ldi_file_suffix = NULL;
-	    strcat(filename, "output.txt");
-	}
-	flags = O_CREAT | O_RDWR;
-	if (!info[i].ldi_file_want_append) {
-	    flags |= O_TRUNC;
-	}
+        if (info[i].ldi_file_suffix != NULL) {
+            strcat(filename, info[i].ldi_file_suffix);
+        } else {
+            info[i].ldi_file_suffix = NULL;
+            strcat(filename, "output.txt");
+        }
+        flags = O_CREAT | O_RDWR;
+        if (!info[i].ldi_file_want_append) {
+            flags |= O_TRUNC;
+        }
 
-	/* Actually open the file */
+        /* Actually open the file */
+        info[i].ldi_fd = open(filename, flags, 0644);
+        if (-1 == info[i].ldi_fd) {
+            info[i].ldi_used = false;
+            free(filename);
+            return OPAL_ERR_IN_ERRNO;
+        }
 
-	info[i].ldi_fd = open(filename, flags, 0644);
-	if (-1 == info[i].ldi_fd) {
-	    info[i].ldi_used = false;
-	    return OPAL_ERR_IN_ERRNO;
-	}
-
-	/* Make the file be close-on-exec to prevent child inheritance
-	 * problems */
+        /* Make the file be close-on-exec to prevent child inheritance
+         * problems */
 
 #ifndef __WINDOWS__
-	/* TODO: Need to find out the equivalent in windows */
-	fcntl(info[i].ldi_fd, F_SETFD, 1);
+        /* TODO: Need to find out the equivalent in windows */
+        fcntl(info[i].ldi_fd, F_SETFD, 1);
 #endif
 
-	free(filename);
+        free(filename);
     }
 
     /* Return successfully even if the session dir did not exist yet;
@@ -589,7 +670,12 @@ static void free_descriptor(int output_id)
 	}
 	ldi->ldi_prefix = NULL;
 
-	if (NULL != ldi->ldi_file_suffix) {
+    if (NULL != ldi->ldi_suffix) {
+        free(ldi->ldi_suffix);
+    }
+    ldi->ldi_suffix = NULL;
+    
+    if (NULL != ldi->ldi_file_suffix) {
 	    free(ldi->ldi_file_suffix);
 	}
 	ldi->ldi_file_suffix = NULL;
@@ -604,113 +690,175 @@ static void free_descriptor(int output_id)
 }
 
 
+static int make_string(char **no_newline_string, output_desc_t *ldi, 
+                       const char *format, va_list arglist)
+{
+    size_t len, total_len;
+    bool want_newline = false;
+
+    /* Make the formatted string */
+
+    vasprintf(no_newline_string, format, arglist);
+    total_len = len = strlen(*no_newline_string);
+    if ('\n' != (*no_newline_string)[len - 1]) {
+        want_newline = true;
+        ++total_len;
+    } else if (NULL != ldi->ldi_suffix) {
+        /* if we have a suffix, then we don't want a
+         * newline to appear before it
+         */
+        (*no_newline_string)[len - 1] = '\0';
+        want_newline = true; /* add newline to end after suffix */
+        /* total_len won't change since we just moved the newline
+         * to appear after the suffix
+         */
+    }
+    if (NULL != ldi->ldi_prefix) {
+        total_len += strlen(ldi->ldi_prefix);
+    }
+    if (NULL != ldi->ldi_suffix) {
+        total_len += strlen(ldi->ldi_suffix);
+    }
+    if (temp_str_len < total_len + want_newline) {
+        if (NULL != temp_str) {
+            free(temp_str);
+        }
+        temp_str = (char *) malloc(total_len * 2);
+        if (NULL == temp_str) {
+            return OPAL_ERR_OUT_OF_RESOURCE;
+        }
+        temp_str_len = total_len * 2;
+    }
+    if (NULL != ldi->ldi_prefix && NULL != ldi->ldi_suffix) {
+        if (want_newline) {
+            snprintf(temp_str, temp_str_len, "%s%s%s\n",
+                     ldi->ldi_prefix, *no_newline_string, ldi->ldi_suffix);
+        } else {
+            snprintf(temp_str, temp_str_len, "%s%s%s", ldi->ldi_prefix, 
+                     *no_newline_string, ldi->ldi_suffix);
+        }
+    } else if (NULL != ldi->ldi_prefix) {
+        if (want_newline) {
+            snprintf(temp_str, temp_str_len, "%s%s\n",
+                     ldi->ldi_prefix, *no_newline_string);
+        } else {
+            snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix, 
+                     *no_newline_string);
+        }
+    } else if (NULL != ldi->ldi_suffix) {
+        if (want_newline) {
+            snprintf(temp_str, temp_str_len, "%s%s\n",
+                     *no_newline_string, ldi->ldi_suffix);
+        } else {
+            snprintf(temp_str, temp_str_len, "%s%s", 
+                     *no_newline_string, ldi->ldi_suffix);
+        }
+    } else {
+        if (want_newline) {
+            snprintf(temp_str, temp_str_len, "%s\n", *no_newline_string);
+        } else {
+            snprintf(temp_str, temp_str_len, "%s", *no_newline_string);
+        }
+    }
+    
+    return OPAL_SUCCESS;
+}
+    
 /*
  * Do the actual output.  Take a va_list so that we can be called from
  * multiple different places, even functions that took "..." as input
  * arguments.
  */
-static void output(int output_id, const char *format, va_list arglist)
+static int output(int output_id, const char *format, va_list arglist)
 {
-    size_t len, total_len;
-    bool want_newline = false;
-    char *str;
+    int rc = OPAL_SUCCESS;
+    char *str, *out = NULL;
     output_desc_t *ldi;
 
     /* Setup */
 
     if (!initialized) {
-	opal_output_init();
+        opal_output_init();
     }
 
     /* If it's valid, used, and enabled, output */
 
     if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-	info[output_id].ldi_used && info[output_id].ldi_enabled) {
-	ldi = &info[output_id];
+        info[output_id].ldi_used && info[output_id].ldi_enabled) {
+        OPAL_THREAD_LOCK(&mutex);
+        ldi = &info[output_id];
 
-	/* Make the formatted string */
+        /* Make the strings */
+        if (OPAL_SUCCESS != (rc = make_string(&str, ldi, format, arglist))) {
+            OPAL_THREAD_UNLOCK(&mutex);
+            return rc;
+        }
 
-	OPAL_THREAD_LOCK(&mutex);
-	vasprintf(&str, format, arglist);
-	total_len = len = strlen(str);
-	if ('\n' != str[len - 1]) {
-	    want_newline = true;
-	    ++total_len;
-	}
-	if (NULL != ldi->ldi_prefix) {
-	    total_len += strlen(ldi->ldi_prefix);
-	}
-	if (temp_str_len < total_len + want_newline) {
-	    if (NULL != temp_str) {
-		free(temp_str);
-	    }
-	    temp_str = (char *) malloc(total_len * 2);
-	    temp_str_len = total_len * 2;
-	}
-	if (NULL != ldi->ldi_prefix) {
-	    if (want_newline) {
-		snprintf(temp_str, temp_str_len, "%s%s\n", ldi->ldi_prefix,
-			 str);
-	    } else {
-		snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix,
-			 str);
-	    }
-	} else {
-	    if (want_newline) {
-		snprintf(temp_str, temp_str_len, "%s\n", str);
-	    } else {
-		snprintf(temp_str, temp_str_len, "%s", str);
-	    }
-	}
-
-	/* Syslog output */
-
-	if (ldi->ldi_syslog) {
-
-#ifndef __WINDOWS__
-	    syslog(ldi->ldi_syslog_priority, str);
+        /* Syslog output -- does not use the newline-appended string */
+#if defined(HAVE_SYSLOG)
+        if (ldi->ldi_syslog) {
+            syslog(ldi->ldi_syslog_priority, "%s", str);
+        }
 #endif
-	}
 
-	/* stdout output */
+        /* All others (stdout, stderr, file) use temp_str, potentially
+           with a newline appended */
 
-	if (ldi->ldi_stdout) {
-            write(fileno(stdout), temp_str, strlen(temp_str)); 
+        out = temp_str;
+
+        /* stdout output */
+        if (ldi->ldi_stdout) {
+            write(fileno(stdout), out, (int)strlen(out)); 
             fflush(stdout);
-	}
+        }
 
-	/* stderr output */
-
-	if (ldi->ldi_stderr) {
-            write(fileno(stderr),temp_str, strlen(temp_str)); 
+        /* stderr output */
+        if (ldi->ldi_stderr) {
+            write((-1 == default_stderr_fd) ? 
+                  fileno(stderr) : default_stderr_fd,
+                  out, (int)strlen(out)); 
             fflush(stderr);
 	}
 
-	/* File output -- first check to see if the file opening was
-	 * delayed.  If so, try to open it.  If we failed to open it,
-	 * then just discard (there are big warnings in the
-	 * opal_output.h docs about this!). */
+        /* File output -- first check to see if the file opening was
+         * delayed.  If so, try to open it.  If we failed to open it,
+         * then just discard (there are big warnings in the
+         * opal_output.h docs about this!). */
 
-	if (ldi->ldi_file) {
-	    if (ldi->ldi_fd == -1) {
-		    if (OPAL_SUCCESS != open_file(output_id)) {
-	            ++ldi->ldi_file_num_lines_lost;
-		    } else if (ldi->ldi_file_num_lines_lost > 0) {
-		        char buffer[BUFSIZ];
-		        memset(buffer, 0, BUFSIZ);
-		        snprintf(buffer, BUFSIZ - 1,
-			             "[WARNING: %d lines lost because the Open MPI process session directory did\n not exist when opal_output() was invoked]\n",
-			             ldi->ldi_file_num_lines_lost);
-		        write(ldi->ldi_fd, buffer, strlen(buffer));
-		        ldi->ldi_file_num_lines_lost = 0;
-		    }
-	    }
-	    if (ldi->ldi_fd != -1) {
-            write(ldi->ldi_fd, temp_str, total_len);
-	    }
-	}
-	OPAL_THREAD_UNLOCK(&mutex);
+        if (ldi->ldi_file) {
+            if (ldi->ldi_fd == -1) {
+                if (OPAL_SUCCESS != open_file(output_id)) {
+                    ++ldi->ldi_file_num_lines_lost;
+                } else if (ldi->ldi_file_num_lines_lost > 0) {
+                    char buffer[BUFSIZ];
+                    char *out = buffer;
+                    memset(buffer, 0, BUFSIZ);
+                    snprintf(buffer, BUFSIZ - 1,
+                             "[WARNING: %d lines lost because the Open MPI process session directory did\n not exist when opal_output() was invoked]\n",
+                             ldi->ldi_file_num_lines_lost);
+                   write(ldi->ldi_fd, buffer, (int)strlen(buffer));
+                    ldi->ldi_file_num_lines_lost = 0;
+                    if (out != buffer) {
+                        free(out);
+                    }
+                }
+            }
+            if (ldi->ldi_fd != -1) {
+                write(ldi->ldi_fd, out, (int)strlen(out));
+            }
+        }
+        OPAL_THREAD_UNLOCK(&mutex);
+        free(str);
+    }
 
-	free(str);
+    return rc;
+}
+
+int opal_output_get_verbosity(int output_id)
+{
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS && info[output_id].ldi_used) {
+        return info[output_id].ldi_verbose_level;
+    } else {
+        return -1;
     }
 }
