@@ -38,6 +38,7 @@
 #include "opal/mca/backtrace/backtrace.h"
 #include "opal/constants.h"
 #include "opal/util/output.h"
+#include "opal/util/show_help.h"
 
 #ifndef _NSIG
 #define _NSIG 32
@@ -400,11 +401,12 @@ void opal_stackframe_output(int stream)
 int opal_util_register_stackhandlers (void)
 {
 #if OMPI_WANT_PRETTY_PRINT_STACKTRACE && ! defined(__WINDOWS__)
-    struct sigaction act;
+    struct sigaction act, old;
     char * string_value;
     char * tmp;
     char * next;
     int param, i;
+    bool complain, showed_help = false;
 
     gethostname(stacktrace_hostname, sizeof(stacktrace_hostname));
     stacktrace_hostname[sizeof(stacktrace_hostname) - 1] = '\0';
@@ -435,6 +437,7 @@ int opal_util_register_stackhandlers (void)
       int sig;
       int ret;
 
+      complain = false;
       sig = strtol (tmp, &next, 10);
 
       /*
@@ -443,15 +446,41 @@ int opal_util_register_stackhandlers (void)
        */
       if (((0 == sig) && (tmp == next)) || (0 > sig) || (_NSIG <= sig)) {
 	 return OPAL_ERR_BAD_PARAM;
-      }
-
-      if ((next == NULL) || ((*next != ',') && (*next != '\0'))) {
+      } else if (next == NULL) {
 	 return OPAL_ERR_BAD_PARAM;
+      } else if (':' == *next &&
+                 0 == strncasecmp(next, ":complain", 9)) {
+          complain = true;
+          next += 9;
+      } else if (',' != *next && '\0' != *next) {
+          return OPAL_ERR_BAD_PARAM;
       }
 
-      ret = sigaction (sig, &act, NULL);
-      if (ret != 0) {
-        return OPAL_ERR_IN_ERRNO;
+      /* Just query first */
+      ret = sigaction (sig, NULL, &old);
+      if (0 != ret) {
+          return OPAL_ERR_IN_ERRNO;
+      }
+      /* Was there something already there? */
+      if (SIG_IGN != old.sa_handler && SIG_DFL != old.sa_handler) {
+          if (!showed_help && complain) {
+              /* JMS This is icky; there is no error message
+                 aggregation here so this message may be repeated for
+                 every single MPI process...  This should be replaced
+                 with OPAL_SOS when that is done so that it can be
+                 properly aggregated. */
+              opal_show_help("help-opal-util.txt",
+                             "stacktrace signal override",
+                             true, sig, sig, sig, string_value);
+              showed_help = true;
+          }
+      }
+
+      /* Nope, nothing was there, so put in ours */
+      else {
+          if (0 != sigaction(sig, &act, NULL)) {
+              return OPAL_ERR_IN_ERRNO;
+          }
       }
     }
     free(string_value);
