@@ -48,53 +48,6 @@ BEGIN_C_DECLS
  * General ODLS types
  */
 
-/*
- * List object to locally store the process names and pids of
- * our children. This can subsequently be used to order termination
- * or pass signals without looking the info up again.
- */
-typedef struct {
-    opal_list_item_t super;      /* required to place this on a list */
-    orte_process_name_t *name;   /* the OpenRTE name of the proc */
-    pid_t pid;                   /* local pid of the proc */
-    orte_std_cntr_t app_idx;     /* index of the app_context for this proc */
-    bool alive;                  /* is this proc alive? */
-    bool coll_recvd;             /* collective operation recvd */
-    orte_proc_state_t state;     /* the state of the process */
-    orte_exit_code_t exit_code;  /* process exit code */
-    char *rml_uri;               /* contact info for this child */
-    char *slot_list;             /* list of slots for this child */
-    bool waitpid_recvd;          /* waitpid has detected proc termination */
-    bool iof_complete;           /* IOF has noted proc terminating all channels */
-} orte_odls_child_t;
-ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_odls_child_t);
-
-/*
- * List object to locally store job related info
- */
-typedef struct orte_odls_job_t {
-    opal_list_item_t    super;                  /* required to place this on a list */
-    orte_jobid_t        jobid;                  /* jobid for this data */
-    orte_app_context_t  **apps;                 /* app_contexts for this job */
-    orte_std_cntr_t     num_apps;               /* number of app_contexts */
-    orte_job_controls_t controls;               /* control flags for job */
-    orte_vpid_t         stdin_target;           /* where stdin is to go */
-    orte_std_cntr_t     total_slots_alloc;
-    orte_vpid_t         num_procs;
-    int32_t             num_local_procs;
-    opal_value_array_t  procmap;                /* map of procs/node, local ranks */
-    opal_byte_object_t  *pmap;                  /* byte object version of procmap */
-    opal_buffer_t       collection_bucket;
-    opal_buffer_t       local_collection;
-    orte_grpcomm_coll_t collective_type;
-    int32_t             num_contributors;
-    int                 num_participating;
-    int                 num_collected;
-} orte_odls_job_t;
-ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_odls_job_t);
-
-
-
 typedef struct {
     /** Verbose/debug output stream */
     int output;
@@ -104,12 +57,12 @@ typedef struct {
     opal_mutex_t mutex;
     /* condition variable */
     opal_condition_t cond;
-    /* list of children for this orted */
-    opal_list_t children;
-    /* list of job data for this orted */
-    opal_list_t jobs;
     /* byte object to store daemon map for later xmit to procs */
     opal_byte_object_t *dmap;
+    /* list of ranks to be displayed on separate xterms */
+    opal_list_t xterm_ranks;
+    /* the xterm cmd to be used */
+    char **xtermcmd;
 } orte_odls_globals_t;
 
 ORTE_DECLSPEC extern orte_odls_globals_t orte_odls_globals;
@@ -124,6 +77,9 @@ ORTE_DECLSPEC int
 orte_odls_base_default_get_add_procs_data(opal_buffer_t *data,
                                           orte_jobid_t job);
 
+ORTE_DECLSPEC int
+orte_odls_base_default_update_daemon_info(opal_buffer_t *data);
+
 ORTE_DECLSPEC int
 orte_odls_base_default_construct_child_list(opal_buffer_t *data,
                                             orte_jobid_t *job);
