@@ -18,15 +18,26 @@
 
 
 #include "orte_config.h"
-#include "orte/orte_constants.h"
+#include "orte/constants.h"
+
+#if !ORTE_DISABLE_FULL_SUPPORT
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_param.h"
-#include "opal/util/output.h"
+
+#include "orte/mca/ras/base/ras_private.h"
+
+#endif
+
 #include "orte/mca/ras/base/base.h"
 
 
+/* NOTE: the RAS does not require a proxy as only the
+ * HNP can open the framework in orte_init - non-HNP
+ * procs are not allowed to allocate resources
+ */
+ 
 /*
  * The following file was created by configure.  It contains extern
  * statements and the definition of an array of pointers to each
@@ -35,11 +46,25 @@
 
 #include "orte/mca/ras/base/static-components.h"
 
+#if ORTE_DISABLE_FULL_SUPPORT
+/* have to include a bogus function here so that
+ * the build system sees at least one function
+ * in the library
+ */
+int orte_ras_base_open(void)
+{
+    return ORTE_SUCCESS;
+}
+
+#else
 
 /*
  * Global variables
  */
-orte_ras_base_module_t orte_ras;
+orte_ras_t orte_ras = {
+    orte_ras_base_allocate
+};
+
 orte_ras_base_t orte_ras_base;
 
 
@@ -50,26 +75,33 @@ orte_ras_base_t orte_ras_base;
 int orte_ras_base_open(void)
 {
     int value;
+    bool btmp;
+    
+    /* set default flags */
+    orte_ras_base.active_module = NULL;
+    orte_ras_base.allocation_read = false;
+    
+    /* should we display the allocation after determining it? */
+    mca_base_param_reg_int_name("ras", "base_display_alloc",
+                                "Whether to display the allocation after it is determined",
+                                false, false, (int)false, &value);
+    orte_ras_base.display_alloc = OPAL_INT_TO_BOOL(value);
 
-    /* Debugging / verbose output */
-
-    orte_ras_base.ras_output = opal_output_open(NULL);
-    mca_base_param_reg_int_name("ras_base", "verbose", 
-                                "Enable debugging for the RAS framework (nonzero = enabled)",
-                                false, false, 0, &value);
-    if (value != 0) {
-        orte_ras_base.ras_output = opal_output_open(NULL);
-    } else {
-        orte_ras_base.ras_output = -1;
+    /* should we display a detailed (developer-quality) version of the allocation after determining it? */
+    mca_base_param_reg_int_name("ras", "base_display_devel_alloc",
+                                "Whether to display a developer-detail allocation after it is determined",
+                                false, false, (int)false, &value);
+    btmp = OPAL_INT_TO_BOOL(value);
+    if (btmp) {
+        orte_ras_base.display_alloc = true;
+        orte_devel_level_output = true;
     }
 
-    /* Defaults */
-
-    orte_ras_base.ras_opened_valid = false;
-    orte_ras_base.ras_available_valid = false;
-
+    /* Debugging / verbose output.  Always have stream open, with
+        verbose set by the mca open system... */
+    orte_ras_base.ras_output = opal_output_open(NULL);
+    
     /* Open up all available components */
-
     if (ORTE_SUCCESS != 
         mca_base_components_open("ras", orte_ras_base.ras_output,
                                  mca_ras_base_static_components, 
@@ -78,8 +110,7 @@ int orte_ras_base_open(void)
     }
 
     /* All done */
-
-    orte_ras_base.ras_opened_valid = true;
     return ORTE_SUCCESS;
 }
 
+#endif /* ORTE_DISABLE_FULL_SUPPORT */
