@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -18,13 +18,14 @@
  */
 
 #include "orte_config.h"
+#include "orte/constants.h"
 
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_param.h"
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "opal/util/basename.h"
-#include "orte/orte_constants.h"
-#include "orte/util/proc_info.h"
+
+#include "orte/mca/ras/base/ras_private.h"
 #include "ras_tm.h"
 
 
@@ -38,7 +39,7 @@ static int param_priority;
  * Local functions
  */
 static int ras_tm_open(void);
-static orte_ras_base_module_t *ras_tm_init(int*);
+static int orte_ras_tm_component_query(mca_base_module_t **module, int *priority);
 
 
 orte_ras_tm_component_t mca_ras_tm_component = {
@@ -47,38 +48,30 @@ orte_ras_tm_component_t mca_ras_tm_component = {
            information about the component itself */
 
         {
-            /* Indicate that we are a ras v1.3.0 component (which also
-               implies a specific MCA version) */
-            
-            ORTE_RAS_BASE_VERSION_1_3_0,
+            ORTE_RAS_BASE_VERSION_2_0_0,
             
             /* Component name and version */
-            
             "tm",
             ORTE_MAJOR_VERSION,
             ORTE_MINOR_VERSION,
             ORTE_RELEASE_VERSION,
             
             /* Component open and close functions */
-            
             ras_tm_open,
-            NULL
+            NULL,
+            orte_ras_tm_component_query
         },
-        
-        /* Next the MCA v1.0.0 component meta data */
         {
-            /* Whether the component is checkpointable or not */
-            false
-        },
-        
-        ras_tm_init
+            /* The component is checkpoint ready */
+            MCA_BASE_METADATA_PARAM_CHECKPOINT
+        }
     }
 };
 
 
 static int ras_tm_open(void)
 {
-    mca_base_component_t *c        = &mca_ras_tm_component.super.ras_version;
+    mca_base_component_t *c        = &mca_ras_tm_component.super.base_version;
     char *pbs_nodefile_env         = NULL;
     char *default_nodefile_dir     = NULL;
     bool free_default_nodefile_dir = false;
@@ -115,24 +108,17 @@ static int ras_tm_open(void)
 }
 
 
-static orte_ras_base_module_t *ras_tm_init(int* priority)
+static int orte_ras_tm_component_query(mca_base_module_t **module, int *priority)
 {
-    /* if we are not an HNP, then we must not be selected */
-    if (!orte_process_info.seed) {
-        return NULL;
-    }
-    
     /* Are we running under a TM job? */
     if (NULL != getenv("PBS_ENVIRONMENT") &&
         NULL != getenv("PBS_JOBID")) {
         mca_base_param_lookup_int(param_priority, priority);
-        opal_output(orte_ras_base.ras_output,
-                    "ras:tm: available for selection");
-        return &orte_ras_tm_module;
+        *module = (mca_base_module_t *) &orte_ras_tm_module;
+        return ORTE_SUCCESS;
     }
 
     /* Sadly, no */
-    opal_output(orte_ras_base.ras_output,
-                "ras:tm: NOT available for selection");
-    return NULL;
+    *module = NULL;
+    return ORTE_ERROR;
 }
