@@ -24,33 +24,13 @@
 #define ORTE_RMAPS_RR_H
 
 #include "orte/mca/rmaps/rmaps.h"
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
 
+BEGIN_C_DECLS
 
-/**
- * RMGR Component 
- */
-struct orte_rmaps_round_robin_component_t {
-    orte_rmaps_base_component_t super;
-    int debug;
-    int priority;
-    bool bynode;
-    bool per_node;
-    bool n_per_node;
-    bool no_use_local;
-    bool oversubscribe;
-    bool no_allocate_range;
-};
-typedef struct orte_rmaps_round_robin_component_t orte_rmaps_round_robin_component_t;
-
-ORTE_MODULE_DECLSPEC extern orte_rmaps_round_robin_component_t mca_rmaps_round_robin_component;
+ORTE_MODULE_DECLSPEC extern orte_rmaps_base_component_t mca_rmaps_round_robin_component;
 extern orte_rmaps_base_module_t orte_rmaps_round_robin_module;
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
