@@ -12,6 +12,7 @@
  * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -68,6 +69,7 @@
 #include "orte/mca/rml/base/rml_contact.h"
 #include "orte/mca/odls/odls.h"
 #include "orte/mca/odls/base/base.h"
+#include "orte/mca/odls/base/odls_private.h"
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/plm/base/plm_private.h"
 #include "orte/mca/routed/routed.h"
@@ -85,36 +87,24 @@ static int process_commands(orte_process_name_t* sender,
                             opal_buffer_t *buffer,
                             orte_rml_tag_t tag);
 
+static bool exit_reqd;
 
-
-/* local callback function for non-blocking sends */
-static void send_callback(int status, orte_process_name_t *peer,
-                          opal_buffer_t *buf, orte_rml_tag_t tag,
-                          void *cbdata)
+static void send_relay(opal_buffer_t *buf)
 {
-    /* nothing to do here - just release buffer and return */
-    OBJ_RELEASE(buf);
-}
-
-static void send_relay(int fd, short event, void *data)
-{
-    orte_message_event_t *mev = (orte_message_event_t*)data;
-    opal_buffer_t *buffer=NULL;
-    orte_rml_tag_t tag = mev->tag;
-    orte_jobid_t target_job = mev->sender.jobid;
     opal_list_t recips;
     opal_list_item_t *item;
     orte_namelist_t *nm;
+    orte_process_name_t target;
     int ret;
     
     OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
                          "%s orte:daemon:send_relay",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-
+    
     /* get the list of next recipients from the routed module */
     OBJ_CONSTRUCT(&recips, opal_list_t);
     /* ignore returned parent vpid - we don't care here */
-    orte_routed.get_routing_tree(target_job, &recips);
+    orte_routed.get_routing_tree(ORTE_PROC_MY_NAME->jobid, &recips);
     
     /* if list is empty, nothing for us to do */
     if (opal_list_is_empty(&recips)) {
@@ -124,137 +114,28 @@ static void send_relay(int fd, short event, void *data)
         goto CLEANUP;
     }
     
-    /* get the first recipient so we can look at it */
-    item = opal_list_get_first(&recips);
-    nm = (orte_namelist_t*)item;
-    
-    /* check to see if this message is going directly to the
-     * target jobid and that jobid is not my own
-     */
-    if (nm->name.jobid == target_job &&
-        target_job != ORTE_PROC_MY_NAME->jobid) {
-        orte_daemon_cmd_flag_t command;
-        orte_jobid_t job;
-        orte_rml_tag_t msg_tag;
-        int32_t n;
-        OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
-                             "%s orte:daemon:send_relay sending directly to job %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(nm->name.jobid)));
-        /* this is going directly to the job, and not being
-         * relayed any further. We need to remove the process-and-relay
-         * command and the target jobid/tag from the buffer so the
-         * recipients can correctly process it
-         */
-        n = 1;
-        if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &command, &n, ORTE_DAEMON_CMD))) {
-            ORTE_ERROR_LOG(ret);
-            goto CLEANUP;
-        }
-        n = 1;
-        if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &job, &n, ORTE_JOBID))) {
-            ORTE_ERROR_LOG(ret);
-            goto CLEANUP;
-        }
-        n = 1;
-        if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &msg_tag, &n, ORTE_RML_TAG))) {
-            ORTE_ERROR_LOG(ret);
-            goto CLEANUP;
-        }
-        /* if this isn't going to the daemon tag, then we have more to extract */
-        if (ORTE_RML_TAG_DAEMON != tag) {
-            /* remove the message_local_procs cmd data */
-            n = 1;
-            if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &command, &n, ORTE_DAEMON_CMD))) {
-                ORTE_ERROR_LOG(ret);
-                goto CLEANUP;
-            }
-            n = 1;
-            if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &job, &n, ORTE_JOBID))) {
-                ORTE_ERROR_LOG(ret);
-                goto CLEANUP;
-            }
-            n = 1;
-            if (ORTE_SUCCESS != (ret = opal_dss.unpack(mev->buffer, &msg_tag, &n, ORTE_RML_TAG))) {
-                ORTE_ERROR_LOG(ret);
-                goto CLEANUP;
-            }
-        }
-        buffer = OBJ_NEW(opal_buffer_t);
-        opal_dss.copy_payload(buffer, mev->buffer);
-    } else {
-        /* buffer is already setup - just point to it */
-        buffer = mev->buffer;
-        /* tag needs to be set to daemon_tag */
-        tag = ORTE_RML_TAG_DAEMON;
-    }
-            
-    /* if the list has only one entry, and that entry has a wildcard
-     * vpid, then we will handle it separately
-     */
-    if (1 == opal_list_get_size(&recips) && nm->name.vpid == ORTE_VPID_WILDCARD) {
-        /* okay, this is a wildcard case. First, look up the #procs in the
-         * specified job - only the HNP can do this. Fortunately, the routed
-         * modules are smart enough not to ask a remote daemon to do it!
-         * However, just to be safe, in case some foolish future developer
-         * doesn't get that logic right... ;-)
-         */
-        orte_job_t *jdata;
-        orte_vpid_t i;
-        orte_process_name_t target;
+    /* send the message to each recipient on list, deconstructing it as we go */
+    target.jobid = ORTE_PROC_MY_NAME->jobid;
+    while (NULL != (item = opal_list_remove_first(&recips))) {
+        nm = (orte_namelist_t*)item;
         
-        if (!orte_process_info.hnp) {
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_IMPLEMENTED);
+        OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+                             "%s orte:daemon:send_relay sending relay msg to %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             ORTE_VPID_PRINT(nm->name.vpid)));
+        
+        /* retain buffer so callback function can release it */
+        target.vpid = nm->name.vpid;
+        if (0 > (ret = orte_rml.send_buffer(&target, buf, ORTE_RML_TAG_DAEMON, 0))) {
+            ORTE_ERROR_LOG(ret);
             goto CLEANUP;
         }
-        if (NULL == (jdata = orte_get_job_data_object(nm->name.jobid))) {
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-            ret = ORTE_ERR_NOT_FOUND;
-            goto CLEANUP;
-        }
-        /* send the buffer to all members of the specified job */
-        target.jobid = nm->name.jobid;
-        for (i=0; i < jdata->num_procs; i++) {
-            if (target.jobid == ORTE_PROC_MY_NAME->jobid &&
-                i == ORTE_PROC_MY_NAME->vpid) {
-                /* do not send to myself! */
-                continue;
-            }
-            
-            target.vpid = i;
-            OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
-                                 "%s orte:daemon:send_relay sending relay msg to %s tag %d",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 ORTE_NAME_PRINT(&target), tag));
-            if (0 > (ret = orte_rml.send_buffer(&target, buffer, tag, 0))) {
-                ORTE_ERROR_LOG(ret);
-                goto CLEANUP;
-            }
-        }
-    } else {
-        /* send the message to each recipient on list, deconstructing it as we go */
-        while (NULL != (item = opal_list_remove_first(&recips))) {
-            nm = (orte_namelist_t*)item;
-            
-            OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
-                                 "%s orte:daemon:send_relay sending relay msg to %s",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 ORTE_NAME_PRINT(&nm->name)));
-            
-            if (0 > (ret = orte_rml.send_buffer(&nm->name, buffer, tag, 0))) {
-                ORTE_ERROR_LOG(ret);
-                goto CLEANUP;
-            }
-        }
+        OBJ_RELEASE(nm);
     }
     
 CLEANUP:
     /* cleanup */
     OBJ_DESTRUCT(&recips);
-    if (NULL != buffer && buffer != mev->buffer) {
-        OBJ_RELEASE(buffer);
-    }
-    OBJ_RELEASE(mev);
 }
 
 void orte_daemon_recv(int status, orte_process_name_t* sender,
@@ -289,7 +170,7 @@ static int wait_time=1;
 void orte_daemon_cmd_processor(int fd, short event, void *data)
 {
     orte_message_event_t *mev = (orte_message_event_t*)data;
-    orte_process_name_t *sender = &(mev->sender), target;
+    orte_process_name_t *sender = &(mev->sender);
     opal_buffer_t *buffer = mev->buffer;
     orte_rml_tag_t tag = mev->tag, target_tag;
     orte_jobid_t job;
@@ -372,17 +253,51 @@ void orte_daemon_cmd_processor(int fd, short event, void *data)
             ORTE_ERROR_LOG(ret);
             goto CLEANUP;
         }
-        /* let the send_relay function know the target jobid */
-        target.jobid = job;
-        target.vpid = ORTE_VPID_INVALID;  /* irrelevant, but better than random */
         /* save this buffer location */
         save = buffer->unpack_ptr;
-        /* rewind the buffer so we can relay it correctly */
-        buffer->unpack_ptr = unpack_ptr;
-        /* setup an event to actually perform the relay */
-        ORTE_MESSAGE_EVENT(&target, buffer, target_tag, send_relay);
-        /* rewind the buffer to the right place for processing */
+        /* unpack the command that will actually be executed */
+        n = 1;
+        if (ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &command, &n, ORTE_DAEMON_CMD))) {
+            ORTE_ERROR_LOG(ret);
+            goto CLEANUP;
+        }
+        /* is this an add-procs cmd? */
+        if (ORTE_DAEMON_ADD_LOCAL_PROCS == command) {
+            /* yes - then it contains daemon update info - process it */
+            if (ORTE_SUCCESS != (ret = orte_odls_base_default_update_daemon_info(buffer))) {
+                ORTE_ERROR_LOG(ret);
+                goto CLEANUP;
+            }
+            /* flag this location */
+            save = buffer->unpack_ptr;
+        }
+        
+        /* rewind the buffer to the right place for processing the cmd */
         buffer->unpack_ptr = save;
+        
+        /* init flag */
+        exit_reqd = false;
+        
+        /* process the command */
+        if (ORTE_SUCCESS != (ret = process_commands(sender, buffer, tag))) {
+            OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+                                 "%s orte:daemon:cmd:processor failed on error %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ORTE_ERROR_NAME(ret)));
+        }
+        
+        /* rewind the buffer to the beginning */
+        buffer->unpack_ptr = unpack_ptr;
+        /* do the relay */
+        send_relay(buffer);
+        
+        /* if we need to exit, do so now */
+        if (exit_reqd) {
+            orte_trigger_event(&orte_exit);
+        }
+        
+        /* done */
+        goto CLEANUP;
+        
     } else {
         /* rewind the buffer so we can process it correctly */
         buffer->unpack_ptr = unpack_ptr;
@@ -460,10 +375,6 @@ static int process_commands(orte_process_name_t* sender,
             
         /****    SIGNAL_LOCAL_PROCS   ****/
         case ORTE_DAEMON_SIGNAL_LOCAL_PROCS:
-            if (orte_debug_daemons_flag) {
-                opal_output(0, "%s orted_cmd: received signal_local_procs",
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            }
             /* unpack the jobid */
             n = 1;
             if (ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &job, &n, ORTE_JOBID))) {
@@ -477,7 +388,22 @@ static int process_commands(orte_process_name_t* sender,
                 ORTE_ERROR_LOG(ret);
                 goto CLEANUP;
             }
-                
+
+            /* Convert SIGTSTP to SIGSTOP so we can suspend a.out */
+            if (SIGTSTP == signal) {
+                if (orte_debug_daemons_flag) {
+                    opal_output(0, "%s orted_cmd: converted SIGTSTP to SIGSTOP before delivering",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                }
+                signal = SIGSTOP;
+            }
+
+            if (orte_debug_daemons_flag) {
+                opal_output(0, "%s orted_cmd: received signal_local_procs, delivering signal %d",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                            signal);
+            }
+
             /* signal them */
             if (ORTE_SUCCESS != (ret = orte_odls.signal_local_procs(NULL, signal))) {
                 ORTE_ERROR_LOG(ret);
@@ -647,7 +573,14 @@ static int process_commands(orte_process_name_t* sender,
             break;
 
             /****    EXIT COMMAND    ****/
-        case ORTE_DAEMON_EXIT_CMD:
+        case ORTE_DAEMON_EXIT_WITH_REPLY_CMD:
+            if (orte_debug_daemons_flag) {
+                opal_output(0, "%s orted_cmd: received exit",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+            }
+            /* if we are the HNP, kill our local procs and
+             * flag we are exited - but don't yet exit
+             */
             if (orte_process_info.hnp) {
                 orte_job_t *daemons;
                 orte_proc_t **procs;
@@ -666,25 +599,63 @@ static int process_commands(orte_process_name_t* sender,
                  * get triggered in single-daemon systems
                  */
                 orte_plm_base_check_job_completed(daemons);
-                /* all done! */
+               /* all done! */
                 return ORTE_SUCCESS;
             }
-            /* eventually, we need to revise this so we only
-             * exit if all our children are dead. For now, treat
-             * the same as a "hard kill" command
+            /* if we are not the HNP, send a message to the HNP telling
+             * it we are leaving - and then trigger our exit
              */
-            if (orte_debug_daemons_flag) {
-                opal_output(0, "%s orted_cmd: received exit",
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+            {
+                opal_buffer_t ack;
+                orte_proc_state_t state=ORTE_PROC_STATE_TERMINATED;
+                orte_exit_code_t exit_code=0;
+                orte_plm_cmd_flag_t cmd = ORTE_PLM_UPDATE_PROC_STATE;
+                
+                OBJ_CONSTRUCT(&ack, opal_buffer_t);
+                opal_dss.pack(&ack, &cmd, 1, ORTE_PLM_CMD);
+                opal_dss.pack(&ack, &(ORTE_PROC_MY_NAME->jobid), 1, ORTE_JOBID);        
+                opal_dss.pack(&ack, &(ORTE_PROC_MY_NAME->vpid), 1, ORTE_VPID);        
+                opal_dss.pack(&ack, &state, 1, ORTE_PROC_STATE);        
+                opal_dss.pack(&ack, &exit_code, 1, ORTE_EXIT_CODE);
+                orte_rml.send_buffer(ORTE_PROC_MY_HNP, &ack, ORTE_RML_TAG_PLM, 0);
+                OBJ_DESTRUCT(&ack);
             }
-            /* trigger our appropriate exit procedure
-             * NOTE: this event will fire -after- any zero-time events
-             * so any pending relays -do- get sent first
-             */
-            orte_trigger_event(&orte_exit);
+            exit_reqd = true;
             return ORTE_SUCCESS;
             break;
-
+            
+            /****    EXIT_NO_REPLY COMMAND    ****/
+        case ORTE_DAEMON_EXIT_NO_REPLY_CMD:
+            if (orte_debug_daemons_flag) {
+                opal_output(0, "%s orted_cmd: received exit_no_reply",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+            }
+            /* if we are the HNP, kill our local procs and
+             * flag we are exited - but don't yet exit
+             */
+            if (orte_process_info.hnp) {
+                orte_job_t *daemons;
+                orte_proc_t **procs;
+                /* if we are the HNP, ensure our local procs are terminated */
+                orte_odls.kill_local_procs(ORTE_JOBID_WILDCARD, false);
+                /* now lookup the daemon job object */
+                if (NULL == (daemons = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid))) {
+                    ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+                    return ORTE_ERR_NOT_FOUND;
+                }
+                procs = (orte_proc_t**)daemons->procs->addr;
+                /* declare us terminated so things can exit cleanly */
+                procs[0]->state = ORTE_PROC_STATE_TERMINATED;
+                daemons->num_terminated++;
+                /* There is nothing more to do here - actual exit will be
+                 * accomplished by the plm
+                 */
+                return ORTE_SUCCESS;
+            }
+            exit_reqd = true;
+            return ORTE_SUCCESS;
+            break;
+            
             /****    HALT VM COMMAND    ****/
         case ORTE_DAEMON_HALT_VM_CMD:
             if (orte_debug_daemons_flag) {
@@ -731,8 +702,7 @@ static int process_commands(orte_process_name_t* sender,
                 goto CLEANUP;
             }
             /* return response */
-            if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                            send_callback, NULL)) {
+            if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                 ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                 ret = ORTE_ERR_COMM_FAILURE;
             }
@@ -762,8 +732,7 @@ static int process_commands(orte_process_name_t* sender,
                 goto CLEANUP;
             }
             
-            if (0 > orte_rml.send_buffer_nb(sender, answer, tag, 0,
-                                            send_callback, NULL)) {
+            if (0 > orte_rml.send_buffer(sender, answer, tag, 0)) {
                 ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                 ret = ORTE_ERR_COMM_FAILURE;
             }
@@ -788,8 +757,7 @@ static int process_commands(orte_process_name_t* sender,
                     goto CLEANUP;
                 }
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
@@ -837,8 +805,7 @@ static int process_commands(orte_process_name_t* sender,
                     }
                 }
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
@@ -864,8 +831,7 @@ static int process_commands(orte_process_name_t* sender,
                     goto CLEANUP;
                 }
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
@@ -920,8 +886,7 @@ static int process_commands(orte_process_name_t* sender,
                     }
                 }
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
@@ -947,8 +912,7 @@ static int process_commands(orte_process_name_t* sender,
                     goto CLEANUP;
                 }
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
@@ -1013,110 +977,12 @@ PACK_ANSWER:
                 }
 SEND_ANSWER:
                 /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
+                if (0 > orte_rml.send_buffer(sender, answer, ORTE_RML_TAG_TOOL, 0)) {
                     ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                     ret = ORTE_ERR_COMM_FAILURE;
                 }
             }
             break;
-            
-            /****     ATTACH_STDIO COMMAND    ****/
-        case ORTE_DAEMON_ATTACH_STDOUT_CMD:
-            if (orte_debug_daemons_flag) {
-                opal_output(0, "%s orted_cmd: received attach stdio cmd",
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            }
-#if 0
-            /* if we are not the HNP, we can do nothing - report
-             * back error so the tool won't hang
-             */
-            if (!orte_process_info.hnp) {
-                int status=ORTE_ERR_NOT_SUPPORTED;
-                
-                answer = OBJ_NEW(opal_buffer_t);
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &status, 1, OPAL_INT))) {
-                    ORTE_ERROR_LOG(ret);
-                    OBJ_RELEASE(answer);
-                    goto CLEANUP;
-                }
-                /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
-                    ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
-                    ret = ORTE_ERR_COMM_FAILURE;
-                }
-            } else {
-                /* if we are the HNP, process the request */
-                int fd, status;
-                orte_vpid_t vpid;
-                orte_process_name_t source;
-                
-                /* setup the answer */
-                answer = OBJ_NEW(opal_buffer_t);
-                
-                /* unpack the jobid */
-                n = 1;
-                if (ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &job, &n, ORTE_JOBID))) {
-                    ORTE_ERROR_LOG(ret);
-                    goto CLEANUP;
-                }
-                
-                /* unpack the vpid */
-                n = 1;
-                if (ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &vpid, &n, ORTE_VPID))) {
-                    ORTE_ERROR_LOG(ret);
-                    goto PACK_ANSWER;
-                }
-                
-                /* unpack the file descriptor */
-                n = 1;
-                if (ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &fd, &n, OPAL_INT))) {
-                    ORTE_ERROR_LOG(ret);
-                    goto PACK_ANSWER;
-                }
-                
-                /* tell the iof to attach it */
-                status = orte_iof.pull(
-                /* if they asked for a specific proc, then just get that info */
-                if (ORTE_VPID_WILDCARD != vpid) {
-                    /* find this proc */
-                    procs = (orte_proc_t**)jdata->procs->addr;
-                    for (i=0; i < jdata->procs->size; i++) {
-                        if (NULL == procs[i]) break; /* stop when we get past the end of data */
-                        if (vpid == procs[i]->name.vpid) {
-                            procs = &procs[i];
-                            num_procs = 1;
-                            break;
-                        }
-                    }
-                } else {
-                    procs = (orte_proc_t**)jdata->procs->addr;
-                    num_procs = jdata->num_procs;
-                }
-                
-PACK_ANSWER:
-                /* pack number of procs */
-                if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &num_procs, 1, ORTE_VPID))) {
-                    ORTE_ERROR_LOG(ret);
-                    goto SEND_ANSWER;
-                }
-                if (0 < num_procs) {
-                    if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, procs, jdata->num_procs, ORTE_PROC))) {
-                        ORTE_ERROR_LOG(ret);
-                        goto SEND_ANSWER;
-                    }
-                }
-SEND_ANSWER:
-                /* callback function will release buffer */
-                if (0 > orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
-                                                send_callback, NULL)) {
-                    ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
-                    ret = ORTE_ERR_COMM_FAILURE;
-                }
-            }
-#endif
-            break;
 
             /****     HEARTBEAT COMMAND    ****/
         case ORTE_DAEMON_HEARTBEAT_CMD:
