@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -39,11 +40,11 @@
 #include "orte/runtime/runtime_internals.h"
 #include "orte/runtime/orte_globals.h"
 
+#if !ORTE_DISABLE_FULL_SUPPORT
+
 /* need the data type support functions here */
 #include "orte/runtime/data_type_support/orte_dt_support.h"
 
-#if !ORTE_DISABLE_FULL_SUPPORT
-
 /* globals used by RTE */
 bool orte_timing;
 bool orte_debug_daemons_file_flag = false;
@@ -52,7 +53,6 @@ bool orte_do_not_launch = false;
 bool orted_spin_flag = false;
 bool orte_static_ports = false;
 bool orte_keep_fqdn_hostnames = false;
-bool orte_tag_output;
 bool orte_show_resolved_nodenames;
 int orted_debug_failure;
 int orted_debug_failure_delay;
@@ -74,6 +74,7 @@ orte_trigger_event_t orte_exit, orteds_exit;
 int orte_exit_status = 0;
 bool orte_abnormal_term_ordered = false;
 bool orte_shutdown_in_progress = false;
+bool orte_job_term_ordered = false;
 
 int orte_heartbeat_rate;
 int orte_startup_timeout;
@@ -91,6 +92,21 @@ opal_pointer_array_t *orte_node_pool;
 /* a clean output channel without prefix */
 int orte_clean_output = -1;
 
+/* list of local children on a daemon */
+opal_list_t orte_local_children;
+/* list of job data for local children on a daemon */
+opal_list_t orte_local_jobdata;
+
+/* IOF controls */
+bool orte_tag_output;
+bool orte_timestamp_output;
+char *orte_output_filename;
+/* generate new xterm windows to display output from specified ranks */
+char *orte_xterm;
+
+/* whether or not to forward SIGTSTP and SIGCONT signals */
+bool orte_forward_job_control;
+
 #endif /* !ORTE_DISABLE_FULL_RTE */
 
 int orte_debug_output = -1;
@@ -119,7 +135,8 @@ int orte_dt_init(void)
             opal_output_set_verbosity(orte_debug_output, 1);
         }
     }
-        
+
+#if !ORTE_DISABLE_FULL_SUPPORT
     /** register the base system types with the DSS */
     tmp = ORTE_STD_CNTR;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_std_cntr,
@@ -176,7 +193,6 @@ int orte_dt_init(void)
         return rc;
     }
 
-#if !ORTE_DISABLE_FULL_SUPPORT
     /* get a clean output channel too */
     {
         opal_output_stream_t lds;
@@ -443,6 +459,7 @@ static void orte_app_context_construct(orte_app_context_t* app_context)
     app_context->preload_binary = false;
     app_context->preload_files  = NULL;
     app_context->preload_files_dest_dir  = NULL;
+    app_context->used_on_node = false;
 }
 
 static void orte_app_context_destructor(orte_app_context_t* app_context)
@@ -569,6 +586,7 @@ OBJ_CLASS_INSTANCE(orte_job_t,
 static void orte_node_construct(orte_node_t* node)
 {
     node->name = NULL;
+    node->alias = NULL;
     node->index = -1;
     node->daemon = NULL;
     node->daemon_launched = false;
@@ -601,6 +619,10 @@ static void orte_node_destruct(orte_node_t* node)
         free(node->name);
     }
 
+    if (NULL != node->alias) {
+        opal_argv_free(node->alias);
+    }
+    
     if (NULL != node->daemon) OBJ_RELEASE(node->daemon);
     
     for (i=0; i < node->num_procs; i++) {
