@@ -11,7 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Cisco Systems, Inc. All rights reserved.
- * Copyright (c) 2007-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright (c) 2007-2009 Sun Microsystems, Inc. All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * $COPYRIGHT$
@@ -63,7 +63,9 @@
 
 #include "opal/version.h"
 #include "opal/runtime/opal.h"
+#include "opal/util/os_dirpath.h"
 #include "opal/util/os_path.h"
+#include "opal/util/path.h"
 #include "opal/class/opal_pointer_array.h"
 #include "opal/dss/dss.h"
 
@@ -71,6 +73,7 @@
 #include "orte/util/pre_condition_transports.h"
 #include "orte/util/session_dir.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/parse_options.h"
 
 #include "orte/mca/odls/odls.h"
 #include "orte/mca/plm/plm.h"
@@ -99,6 +102,8 @@ static struct opal_event int_handler;
 #ifndef __WINDOWS__
 static struct opal_event sigusr1_handler;
 static struct opal_event sigusr2_handler;
+static struct opal_event sigtstp_handler;
+static struct opal_event sigcont_handler;
 #endif  /* __WINDOWS__ */
 static orte_job_t *jdata;
 static char *orterun_basename = NULL;
@@ -149,6 +154,20 @@ static opal_cmd_line_init_t cmd_line_init[] = {
       NULL, OPAL_CMD_LINE_TYPE_BOOL,
       "Provide all output in XML format" },
     
+    /* tag output */
+    { "orte", "tag", "output", '\0', "tag-output", "tag-output", 0,
+      NULL, OPAL_CMD_LINE_TYPE_BOOL,
+      "Tag all output with [job,rank]" },
+    { "orte", "timestamp", "output", '\0', "timestamp-output", "timestamp-output", 0,
+        NULL, OPAL_CMD_LINE_TYPE_BOOL,
+    "Timestamp all application process output" },
+    { "orte", "output", "filename", '\0', "output-filename", "output-filename", 1,
+        NULL, OPAL_CMD_LINE_TYPE_STRING,
+    "Redirect output from application processes into filename.rank" },
+    { "orte", "xterm", NULL, '\0', "xterm", "xterm", 1,
+        NULL, OPAL_CMD_LINE_TYPE_STRING,
+    "Create a new xterm window and display output from the specified ranks there" },
+    
     /* select stdin option */
     { NULL, NULL, NULL, '\0', "stdin", "stdin", 1,
       &orterun_globals.stdin_target, OPAL_CMD_LINE_TYPE_STRING,
@@ -502,7 +521,7 @@ int orterun(int argc, char *argv[])
     opal_signal_set(&int_handler, SIGINT,
                     abort_signal_callback, &int_handler);
     opal_signal_add(&int_handler, NULL);
-    
+
 #ifndef __WINDOWS__
     /** setup callbacks for signals we should foward */
     opal_signal_set(&sigusr1_handler, SIGUSR1,
@@ -511,6 +530,14 @@ int orterun(int argc, char *argv[])
     opal_signal_set(&sigusr2_handler, SIGUSR2,
                     signal_forward_callback, &sigusr2_handler);
     opal_signal_add(&sigusr2_handler, NULL);
+    if (orte_forward_job_control) {
+        opal_signal_set(&sigtstp_handler, SIGTSTP,
+                        signal_forward_callback, &sigtstp_handler);
+        opal_signal_add(&sigtstp_handler, NULL);
+        opal_signal_set(&sigcont_handler, SIGCONT,
+                        signal_forward_callback, &sigcont_handler);
+        opal_signal_add(&sigcont_handler, NULL);
+    }
 #endif  /* __WINDOWS__ */
     
     /* we are an hnp, so update the contact info field for later use */
@@ -763,6 +790,10 @@ static void terminated(int trigpipe, short event, void *arg)
     /** Remove the USR signal handlers */
     opal_signal_del(&sigusr1_handler);
     opal_signal_del(&sigusr2_handler);
+    if (orte_forward_job_control) {
+        opal_signal_del(&sigtstp_handler);
+        opal_signal_del(&sigcont_handler);
+    }
 #endif  /* __WINDOWS__ */
     
     /* get the daemon job object */
@@ -984,6 +1015,14 @@ static void abort_exit_callback(int fd, short ign, void *arg)
         fprintf(stderr, "%s: killing job...\n\n", orterun_basename);
     }
     
+    /* since we are being terminated by a user's signal, be
+     * sure to exit with a non-zero exit code - but don't
+     * overwrite any error code from a proc that might have
+     * failed, in case that is why the user ordered us
+     * to terminate
+     */
+    ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
+
     /* terminate the job - this will also wakeup orterun so
      * it can report to the user and kill all the orteds.
      * Check the jobid, though, just in case the user
@@ -1034,7 +1073,7 @@ static void abort_exit_callback(int fd, short ign, void *arg)
 
         orte_finalize();
         free(orterun_basename);
-        ORTE_UPDATE_EXIT_STATUS(1);
+        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
         exit(orte_exit_status);
     }
 }
@@ -1071,6 +1110,8 @@ static void abort_signal_callback(int fd, short flags, void *arg)
      * use the standard xcast for terminating orteds
      */
     orte_abnormal_term_ordered = true;
+    /* ensure that the forwarding of stdin stops */
+    orte_job_term_ordered = true;
 
     /* We are in an event handler; the job completed procedure
        will delete the signal handler that is currently running
@@ -1672,7 +1713,19 @@ static int create_app(int argc, char* argv[], orte_app_context_t **app_ptr,
     /* Did the user request a specific wdir? */
 
     if (NULL != orterun_globals.wdir) {
-        app->cwd = strdup(orterun_globals.wdir);
+        /* if this is a relative path, convert it to an absolute path */
+        if (opal_path_is_absolute(orterun_globals.wdir)) {
+            app->cwd = strdup(orterun_globals.wdir);
+        } else {
+            /* get the cwd */
+            if (OPAL_SUCCESS != (rc = opal_getcwd(cwd, sizeof(cwd)))) {
+                orte_show_help("help-orterun.txt", "orterun:init-failure",
+                               true, "get the cwd", rc);
+                goto cleanup;
+            }
+            /* construct the absolute path */
+            app->cwd = opal_os_path(false, cwd, orterun_globals.wdir, NULL);
+        }
         app->user_specified_cwd = true;
     } else {
         if (OPAL_SUCCESS != (rc = opal_getcwd(cwd, sizeof(cwd)))) {
