@@ -19,8 +19,8 @@
 /** @file **/
 
 #include "orte_config.h"
+#include "orte/constants.h"
 
-#include "orte/orte_constants.h"
 #include <stdio.h>
 
 const char *
@@ -37,9 +37,6 @@ orte_err2str(int errnum)
     case ORTE_ERR_NO_MATCH_YET:
         retval = "No match for receive posted";
         break;
-    case ORTE_ERR_BUFFER:
-        retval = "Buffer error";
-        break;
     case ORTE_ERR_REQUEST:
         retval = "Request error";
         break;
@@ -52,27 +49,9 @@ orte_err2str(int errnum)
     case ORTE_ERR_CONNECTION_FAILED:
         retval = "Connection failed";
         break;
-    case ORTE_ERR_PACK_MISMATCH:
-        retval = "Pack data mismatch";
-        break;
-    case ORTE_ERR_PACK_FAILURE:
-        retval = "Data pack failed";
-        break;
-    case ORTE_ERR_UNPACK_FAILURE:
-        retval = "Data unpack failed";
-        break;
     case ORTE_ERR_COMM_FAILURE:
         retval = "Communication failure";
         break;
-    case ORTE_ERR_UNPACK_INADEQUATE_SPACE:
-        retval = "Data unpack had inadequate space";
-        break;
-    case ORTE_ERR_UNPACK_READ_PAST_END_OF_BUFFER:
-        retval = "Data unpack would read past end of buffer";
-        break;
-    case ORTE_ERR_GPR_DATA_CORRUPT:
-        retval = "GPR data corruption";
-        break;
     case ORTE_ERR_TYPE_MISMATCH:
         retval = "Type mismatch";
         break;
@@ -82,18 +61,6 @@ orte_err2str(int errnum)
     case ORTE_ERR_COPY_FAILURE:
         retval = "Data copy failure";
         break;
-    case ORTE_ERR_UNKNOWN_DATA_TYPE:
-        retval = "Unknown data type";
-        break;
-    case ORTE_ERR_DATA_TYPE_REDEF:
-        retval = "Attempt to redefine an existing data type";
-        break;
-    case ORTE_ERR_DATA_OVERWRITE_ATTEMPT:
-        retval = "Attempt to overwrite a data value";
-        break;
-    case ORTE_ERR_OPERATION_UNSUPPORTED:
-        retval = "Requested operation is not supported on referenced data type";
-        break;
     case ORTE_ERR_PROC_STATE_MISSING:
         retval = "The process state information is missing on the registry";
         break;
@@ -112,6 +79,42 @@ orte_err2str(int errnum)
     case ORTE_ERR_SILENT:
         retval = NULL;
         break;
+    case ORTE_ERR_ADDRESSEE_UNKNOWN:
+        retval = "A message is attempting to be sent to a process whose contact information is unknown";
+        break;
+    case ORTE_ERR_SYS_LIMITS_PIPES:
+        retval = "The system limit on number of pipes a process can open was reached";
+        break;
+    case ORTE_ERR_PIPE_SETUP_FAILURE:
+        retval = "A pipe could not be setup between a daemon and one of its local processes";
+        break;
+    case ORTE_ERR_SYS_LIMITS_CHILDREN:
+        retval = "The system limit on number of children a process can have was reached";
+        break;
+    case ORTE_ERR_FAILED_GET_TERM_ATTRS:
+        retval = "The I/O forwarding system was unable to get the attributes of your terminal";
+        break;
+    case ORTE_ERR_WDIR_NOT_FOUND:
+        retval = "The specified working directory could not be found";
+        break;
+    case ORTE_ERR_EXE_NOT_FOUND:
+        retval = "The specified executable could not be found";
+        break;
+    case ORTE_ERR_PIPE_READ_FAILURE:
+        retval = "A pipe could not be read";
+        break;
+    case ORTE_ERR_EXE_NOT_ACCESSIBLE:
+        retval = "The specified executable could not be executed";
+        break;
+    case ORTE_ERR_FAILED_TO_START:
+        retval = "The specified application failed to start";
+        break;
+    case ORTE_ERR_FILE_NOT_EXECUTABLE:
+        retval = "A system-required executable either could not be found or was not executable by this user";
+        break;
+    case ORTE_ERR_HNP_COULD_NOT_START:
+        retval = "Unable to start a daemon on the local node";
+        break;
     default:
         retval = NULL;
     }
